@@ -8,17 +8,15 @@ import random
 import time
 
 from gevent import sleep
-from gevent.greenlet import Greenlet
-from gevent.event import Event
 from teuthology import misc as teuthology
 
 from tasks import ceph_manager
 from tasks.cephfs.filesystem import MDSCluster, Filesystem, FSMissing
-from tasks.thrasher import Thrasher
+from tasks.thrasher import ThrasherGreenlet
 
 log = logging.getLogger(__name__)
 
-class MDSThrasher(Thrasher, Greenlet):
+class MDSThrasher(ThrasherGreenlet):
     """
     MDSThrasher::
 
@@ -107,7 +105,6 @@ class MDSThrasher(Thrasher, Greenlet):
         self.manager = manager
         self.max_mds = max_mds
         self.name = 'thrasher.fs.[{f}]'.format(f = fs.name)
-        self.stopping = Event()
 
         self.randomize = bool(self.config.get('randomize', True))
         self.thrash_max_mds = float(self.config.get('thrash_max_mds', 0.05))
@@ -146,9 +143,6 @@ class MDSThrasher(Thrasher, Greenlet):
         """Write data to the logger assigned to MDSThrasher"""
         self.logger.info(x)
 
-    def stop(self):
-        self.stopping.set()
-
     def kill_mds(self, mds):
         if self.config.get('powercycle'):
             (remote,) = (self.ctx.cluster.only('mds.{m}'.format(m=mds)).
@@ -233,16 +227,14 @@ class MDSThrasher(Thrasher, Greenlet):
             "kill": 0,
         }
 
-        while not self.stopping.is_set():
+        while not self.is_stopped:
             delay = self.max_thrash_delay
             if self.randomize:
                 delay = random.randrange(0.0, self.max_thrash_delay)
 
             if delay > 0.0:
                 self.log('waiting for {delay} secs before thrashing'.format(delay=delay))
-                self.stopping.wait(delay)
-                if self.stopping.is_set():
-                    continue
+                self.sleep_unless_stopped(delay)
 
             status = self.fs.status()
 
@@ -272,7 +264,7 @@ class MDSThrasher(Thrasher, Greenlet):
                 weight = 1.0
                 if 'thrash_weights' in self.config:
                     weight = self.config['thrash_weights'].get(label, '0.0')
-                skip = random.randrange(0.0, 1.0)
+                skip = random.random()
                 if weight <= skip:
                     self.log('skipping thrash iteration with skip ({skip}) > weight ({weight})'.format(skip=skip, weight=weight))
                     continue
@@ -319,7 +311,7 @@ class MDSThrasher(Thrasher, Greenlet):
 
                 self.log('waiting for {delay} secs before reviving {label}'.format(
                     delay=delay, label=label))
-                sleep(delay)
+                self.sleep_unless_stopped(delay)
 
                 self.log('reviving {label}'.format(label=label))
                 self.revive_mds(name)
@@ -334,7 +326,7 @@ class MDSThrasher(Thrasher, Greenlet):
                         break
                     self.log(
                         'waiting till mds map indicates {label} is in active, standby or standby-replay'.format(label=label))
-                    sleep(2)
+                    self.sleep_unless_stopped(2)
 
         for stat in stats:
             self.log("stat['{key}'] = {value}".format(key = stat, value = stats[stat]))
