@@ -20,14 +20,19 @@ set -ex
 
 : ${R_BIN:=R}
 source_dir=${1}/r
-with_docs=${2:-false}
+build_dir=${2}
+
+: ${BUILD_DOCS_R:=OFF}
 
 pushd ${source_dir}
 
-${R_BIN} CMD INSTALL ${INSTALL_ARGS} .
+# build first so that any stray compiled files in r/src are ignored
+${R_BIN} CMD build .
+${R_BIN} CMD INSTALL ${INSTALL_ARGS} arrow*.tar.gz
 
-if [ "${with_docs}" == "true" ]; then
+if [ "${BUILD_DOCS_R}" == "ON" ]; then
   ${R_BIN} -e "pkgdown::build_site(install = FALSE)"
+  rsync -a ${source_dir}/docs/ ${build_dir}/docs/r
 fi
 
-popd
\ No newline at end of file
+popd
