@@ -20,7 +20,7 @@
 #include <limits>
 
 #include "arrow/array/builder_time.h"
-#include "arrow/compute/kernels/common.h"
+#include "arrow/compute/kernels/common_internal.h"
 #include "arrow/compute/kernels/scalar_cast_internal.h"
 #include "arrow/compute/kernels/temporal_internal.h"
 #include "arrow/util/bitmap_reader.h"
@@ -29,7 +29,8 @@
 
 namespace arrow {
 
-using internal::ParseValue;
+using internal::ParseTimestampISO8601;
+using internal::ParseYYYY_MM_DD;
 
 namespace compute {
 namespace internal {
@@ -41,10 +42,10 @@ constexpr int64_t kMillisecondsInDay = 86400000;
 
 template <typename in_type, typename out_type>
 Status ShiftTime(KernelContext* ctx, const util::DivideOrMultiply factor_op,
-                 const int64_t factor, const ArrayData& input, ArrayData* output) {
+                 const int64_t factor, const ArraySpan& input, ArraySpan* output) {
   const CastOptions& options = checked_cast<const CastState&>(*ctx->state()).options;
-  auto in_data = input.GetValues<in_type>(1);
-  auto out_data = output->GetMutableValues<out_type>(1);
+  const in_type* in_data = input.GetValues<in_type>(1);
+  out_type* out_data = output->GetValues<out_type>(1);
 
   if (factor == 1) {
     for (int64_t i = 0; i < input.length; i++) {
@@ -63,8 +64,8 @@ Status ShiftTime(KernelContext* ctx, const util::DivideOrMultiply factor_op,
 
       int64_t max_val = std::numeric_limits<int64_t>::max() / factor;
       int64_t min_val = std::numeric_limits<int64_t>::min() / factor;
-      if (input.null_count != 0) {
-        BitmapReader bit_reader(input.buffers[0]->data(), input.offset, input.length);
+      if (input.null_count != 0 && input.buffers[0].data != nullptr) {
+        BitmapReader bit_reader(input.buffers[0].data, input.offset, input.length);
         for (int64_t i = 0; i < input.length; i++) {
           if (bit_reader.IsSet() && (in_data[i] < min_val || in_data[i] > max_val)) {
             RAISE_OVERFLOW_CAST(in_data[i]);
@@ -93,8 +94,8 @@ Status ShiftTime(KernelContext* ctx, const util::DivideOrMultiply factor_op,
   return Status::Invalid("Casting from ", input.type->ToString(), " to ", \
                          output->type->ToString(), " would lose data: ", VAL);
 
-      if (input.null_count != 0) {
-        BitmapReader bit_reader(input.buffers[0]->data(), input.offset, input.length);
+      if (input.null_count != 0 && input.buffers[0].data != nullptr) {
+        BitmapReader bit_reader(input.buffers[0].data, input.offset, input.length);
         for (int64_t i = 0; i < input.length; i++) {
           out_data[i] = static_cast<out_type>(in_data[i] / factor);
           if (bit_reader.IsSet() && (out_data[i] * factor != in_data[i])) {
@@ -119,7 +120,7 @@ Status ShiftTime(KernelContext* ctx, const util::DivideOrMultiply factor_op,
 }
 
 template <template <typename...> class Op, typename OutType, typename... Args>
-Status ExtractTemporal(KernelContext* ctx, const ExecBatch& batch, Datum* out,
+Status ExtractTemporal(KernelContext* ctx, const ExecSpan& batch, ExecResult* out,
                        Args... args) {
   const auto& ty = checked_cast<const TimestampType&>(*batch[0].type());
 
@@ -146,22 +147,25 @@ struct CastFunctor<
     O, I,
     enable_if_t<(is_timestamp_type<O>::value && is_timestamp_type<I>::value) ||
                 (is_duration_type<O>::value && is_duration_type<I>::value)>> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
-    const ArrayData& input = *batch[0].array();
-    ArrayData* output = out->mutable_array();
-
-    // If units are the same, zero copy, otherwise convert
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& in_type = checked_cast<const I&>(*batch[0].type());
-    const auto& out_type = checked_cast<const O&>(*output->type);
+    const auto& out_type = checked_cast<const O&>(*out->type());
 
-    // The units may be equal if the time zones are different. We might go to
-    // lengths to make this zero copy in the future but we leave it for now
+    if (in_type.unit() == out_type.unit()) {
+      return ZeroCopyCastExec(ctx, batch, out);
+    }
 
+    ArrayData* out_arr = out->array_data().get();
+    DCHECK_EQ(0, out_arr->offset);
+    int value_size = batch[0].type()->byte_width();
+    DCHECK_OK(ctx->Allocate(out_arr->length * value_size).Value(&out_arr->buffers[1]));
+
+    ArraySpan output_span;
+    output_span.SetMembers(*out_arr);
+    const ArraySpan& input = batch[0].array;
     auto conversion = util::GetTimestampConversion(in_type.unit(), out_type.unit());
     return ShiftTime<int64_t, int64_t>(ctx, conversion.first, conversion.second, input,
-                                       output);
+                                       &output_span);
   }
 };
 
@@ -186,8 +190,7 @@ struct CastFunctor<Date32Type, TimestampType> {
     Localizer localizer_;
   };
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return ExtractTemporal<Date32, Date32Type>(ctx, batch, out);
   }
 };
@@ -213,8 +216,7 @@ struct CastFunctor<Date64Type, TimestampType> {
     Localizer localizer_;
   };
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return ExtractTemporal<Date64, Date64Type>(ctx, batch, out);
   }
 };
@@ -281,8 +283,7 @@ struct ExtractTimeDownscaledUnchecked {
 
 template <>
 struct CastFunctor<Time32Type, TimestampType> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& in_type = checked_cast<const TimestampType&>(*batch[0].type());
     const auto& out_type = checked_cast<const Time32Type&>(*out->type());
     const CastOptions& options = checked_cast<const CastState&>(*ctx->state()).options;
@@ -310,8 +311,7 @@ struct CastFunctor<Time32Type, TimestampType> {
 
 template <>
 struct CastFunctor<Time64Type, TimestampType> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& in_type = checked_cast<const TimestampType&>(*batch[0].type());
     const auto& out_type = checked_cast<const Time64Type&>(*out->type());
     const CastOptions& options = checked_cast<const CastState&>(*ctx->state()).options;
@@ -345,11 +345,9 @@ struct CastFunctor<O, I, enable_if_t<is_time_type<I>::value && is_time_type<O>::
   using in_t = typename I::c_type;
   using out_t = typename O::c_type;
 
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
-    const ArrayData& input = *batch[0].array();
-    ArrayData* output = out->mutable_array();
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
+    const ArraySpan& input = batch[0].array;
+    ArraySpan* output = out->array_span_mutable();
 
     // If units are the same, zero copy, otherwise convert
     const auto& in_type = checked_cast<const I&>(*input.type);
@@ -366,21 +364,17 @@ struct CastFunctor<O, I, enable_if_t<is_time_type<I>::value && is_time_type<O>::
 
 template <>
 struct CastFunctor<Date64Type, Date32Type> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return ShiftTime<int32_t, int64_t>(ctx, util::MULTIPLY, kMillisecondsInDay,
-                                       *batch[0].array(), out->mutable_array());
+                                       batch[0].array, out->array_span_mutable());
   }
 };
 
 template <>
 struct CastFunctor<Date32Type, Date64Type> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     return ShiftTime<int64_t, int32_t>(ctx, util::DIVIDE, kMillisecondsInDay,
-                                       *batch[0].array(), out->mutable_array());
+                                       batch[0].array, out->array_span_mutable());
   }
 };
 
@@ -389,9 +383,7 @@ struct CastFunctor<Date32Type, Date64Type> {
 
 template <>
 struct CastFunctor<TimestampType, Date32Type> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& out_type = checked_cast<const TimestampType&>(*out->type());
     // get conversion SECOND -> unit
     auto conversion = util::GetTimestampConversion(TimeUnit::SECOND, out_type.unit());
@@ -400,21 +392,19 @@ struct CastFunctor<TimestampType, Date32Type> {
     // multiply to achieve days -> unit
     conversion.second *= kMillisecondsInDay / 1000;
     return ShiftTime<int32_t, int64_t>(ctx, util::MULTIPLY, conversion.second,
-                                       *batch[0].array(), out->mutable_array());
+                                       batch[0].array, out->array_span_mutable());
   }
 };
 
 template <>
 struct CastFunctor<TimestampType, Date64Type> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
-    DCHECK_EQ(batch[0].kind(), Datum::ARRAY);
-
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& out_type = checked_cast<const TimestampType&>(*out->type());
 
     // date64 is ms since epoch
     auto conversion = util::GetTimestampConversion(TimeUnit::MILLI, out_type.unit());
     return ShiftTime<int64_t, int64_t>(ctx, conversion.first, conversion.second,
-                                       *batch[0].array(), out->mutable_array());
+                                       batch[0].array, out->array_span_mutable());
   }
 };
 
@@ -422,22 +412,39 @@ struct CastFunctor<TimestampType, Date64Type> {
 // String to Timestamp
 
 struct ParseTimestamp {
+  explicit ParseTimestamp(const TimestampType& type)
+      : type(type), expect_timezone(!type.timezone().empty()) {}
   template <typename OutValue, typename Arg0Value>
   OutValue Call(KernelContext*, Arg0Value val, Status* st) const {
     OutValue result = 0;
-    if (ARROW_PREDICT_FALSE(!ParseValue(type, val.data(), val.size(), &result))) {
+    bool zone_offset_present = false;
+    if (ARROW_PREDICT_FALSE(!ParseTimestampISO8601(val.data(), val.size(), type.unit(),
+                                                   &result, &zone_offset_present))) {
       *st = Status::Invalid("Failed to parse string: '", val, "' as a scalar of type ",
                             type.ToString());
     }
+    if (zone_offset_present != expect_timezone) {
+      if (expect_timezone) {
+        *st = Status::Invalid(
+            "Failed to parse string: '", val, "' as a scalar of type ", type.ToString(),
+            ": expected a zone offset. If these timestamps "
+            "are in local time, cast to timestamp without timezone, then "
+            "call assume_timezone.");
+      } else {
+        *st = Status::Invalid("Failed to parse string: '", val, "' as a scalar of type ",
+                              type.ToString(), ": expected no zone offset.");
+      }
+    }
     return result;
   }
 
   const TimestampType& type;
+  bool expect_timezone;
 };
 
 template <typename I>
 struct CastFunctor<TimestampType, I, enable_if_t<is_base_binary_type<I>::value>> {
-  static Status Exec(KernelContext* ctx, const ExecBatch& batch, Datum* out) {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
     const auto& out_type = checked_cast<const TimestampType&>(*out->type());
     applicator::ScalarUnaryNotNullStateful<TimestampType, I, ParseTimestamp> kernel(
         ParseTimestamp{out_type});
@@ -445,10 +452,58 @@ struct CastFunctor<TimestampType, I, enable_if_t<is_base_binary_type<I>::value>>
   }
 };
 
+template <typename DateType>
+struct ParseDate {
+  using value_type = typename DateType::c_type;
+
+  using duration_type =
+      typename std::conditional<std::is_same<DateType, Date32Type>::value,
+                                arrow_vendored::date::days,
+                                std::chrono::milliseconds>::type;
+
+  template <typename OutValue, typename Arg0Value>
+  OutValue Call(KernelContext* ctx, Arg0Value val, Status* st) const {
+    OutValue result = OutValue(0);
+
+    if (ARROW_PREDICT_FALSE(val.size() != 10)) {
+      *st = Status::Invalid("Failed to parse string: '", val, "' as a scalar of type ",
+                            TypeTraits<DateType>::type_singleton()->ToString());
+      return result;
+    }
+
+    duration_type since_epoch;
+    if (ARROW_PREDICT_FALSE(!ParseYYYY_MM_DD(val.data(), &since_epoch))) {
+      *st = Status::Invalid("Failed to parse string: '", val, "' as a scalar of type ",
+                            TypeTraits<DateType>::type_singleton()->ToString());
+    } else {
+      result = static_cast<value_type>(since_epoch.count());
+    }
+    return result;
+  }
+};
+
+template <typename O, typename I>
+struct CastFunctor<O, I,
+                   enable_if_t<(is_date_type<O>::value && is_string_type<I>::value)>> {
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
+    return applicator::ScalarUnaryNotNull<O, I, ParseDate<O>>::Exec(ctx, batch, out);
+  }
+};
+
 template <typename Type>
 void AddCrossUnitCast(CastFunction* func) {
   ScalarKernel kernel;
-  kernel.exec = TrivialScalarUnaryAsArraysExec(CastFunctor<Type, Type>::Exec);
+  kernel.exec = CastFunctor<Type, Type>::Exec;
+  kernel.signature = KernelSignature::Make({InputType(Type::type_id)}, kOutputTargetType);
+  DCHECK_OK(func->AddKernel(Type::type_id, std::move(kernel)));
+}
+
+template <typename Type>
+void AddCrossUnitCastNoPreallocate(CastFunction* func) {
+  ScalarKernel kernel;
+  kernel.exec = CastFunctor<Type, Type>::Exec;
+  kernel.null_handling = NullHandling::INTERSECTION;
+  kernel.mem_allocation = MemAllocation::NO_PREALLOCATE;
   kernel.signature = KernelSignature::Make({InputType(Type::type_id)}, kOutputTargetType);
   DCHECK_OK(func->AddKernel(Type::type_id, std::move(kernel)));
 }
@@ -467,6 +522,11 @@ std::shared_ptr<CastFunction> GetDate32Cast() {
   // timestamp -> date32
   AddSimpleCast<TimestampType, Date32Type>(InputType(Type::TIMESTAMP), date32(),
                                            func.get());
+
+  // string -> date32
+  AddSimpleCast<StringType, Date32Type>(utf8(), date32(), func.get());
+  AddSimpleCast<LargeStringType, Date32Type>(large_utf8(), date32(), func.get());
+
   return func;
 }
 
@@ -484,6 +544,11 @@ std::shared_ptr<CastFunction> GetDate64Cast() {
   // timestamp -> date64
   AddSimpleCast<TimestampType, Date64Type>(InputType(Type::TIMESTAMP), date64(),
                                            func.get());
+
+  // string -> date64
+  AddSimpleCast<StringType, Date64Type>(utf8(), date64(), func.get());
+  AddSimpleCast<LargeStringType, Date64Type>(large_utf8(), date64(), func.get());
+
   return func;
 }
 
@@ -500,7 +565,7 @@ std::shared_ptr<CastFunction> GetDurationCast() {
   AddZeroCopyCast(Type::INT64, /*in_type=*/int64(), kOutputTargetType, func.get());
 
   // Between durations
-  AddCrossUnitCast<DurationType>(func.get());
+  AddCrossUnitCastNoPreallocate<DurationType>(func.get());
 
   return func;
 }
@@ -575,7 +640,7 @@ std::shared_ptr<CastFunction> GetTimestampCast() {
                                                 func.get());
 
   // From one timestamp to another
-  AddCrossUnitCast<TimestampType>(func.get());
+  AddCrossUnitCastNoPreallocate<TimestampType>(func.get());
 
   return func;
 }
