@@ -15,7 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-// Internal header.  Platform-specific definitions for gRPC.
+// Internal header. Platform-specific definitions for Flight.
 
 #pragma once
 
@@ -28,5 +28,4 @@
 
 #endif  // _MSC_VER
 
-#include "arrow/util/config.h"                 // IWYU pragma: keep
-#include "arrow/util/windows_compatibility.h"  // IWYU pragma: keep
+#include "arrow/util/config.h"  // IWYU pragma: keep
