@@ -17,19 +17,24 @@
 package encoding_test
 
 import (
+	"bufio"
 	"fmt"
+	"os"
+	"path"
 	"reflect"
+	"strconv"
 	"testing"
 	"unsafe"
 
-	"github.com/apache/arrow/go/v6/arrow"
-	"github.com/apache/arrow/go/v6/arrow/bitutil"
-	"github.com/apache/arrow/go/v6/arrow/memory"
-	"github.com/apache/arrow/go/v6/parquet"
-	"github.com/apache/arrow/go/v6/parquet/internal/encoding"
-	"github.com/apache/arrow/go/v6/parquet/internal/testutils"
-	"github.com/apache/arrow/go/v6/parquet/schema"
+	"github.com/apache/arrow/go/v15/arrow"
+	"github.com/apache/arrow/go/v15/arrow/bitutil"
+	"github.com/apache/arrow/go/v15/arrow/memory"
+	"github.com/apache/arrow/go/v15/parquet"
+	"github.com/apache/arrow/go/v15/parquet/internal/encoding"
+	"github.com/apache/arrow/go/v15/parquet/internal/testutils"
+	"github.com/apache/arrow/go/v15/parquet/schema"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
 )
 
@@ -358,6 +363,16 @@ func (b *BaseEncodingTestSuite) TestBasicRoundTrip() {
 	b.checkRoundTrip(parquet.Encodings.Plain)
 }
 
+func (b *BaseEncodingTestSuite) TestRleBooleanEncodingRoundTrip() {
+	switch b.typ {
+	case reflect.TypeOf(true):
+		b.initData(2000, 200)
+		b.checkRoundTrip(parquet.Encodings.RLE)
+	default:
+		b.T().SkipNow()
+	}
+}
+
 func (b *BaseEncodingTestSuite) TestDeltaEncodingRoundTrip() {
 	b.initData(10000, 1)
 
@@ -403,6 +418,8 @@ func (b *BaseEncodingTestSuite) TestSpacedRoundTrip() {
 			if validBits != nil {
 				b.checkRoundTripSpaced(parquet.Encodings.Plain, validBits, validBitsOffset)
 				switch b.typ {
+				case reflect.TypeOf(false):
+					b.checkRoundTripSpaced(parquet.Encodings.RLE, validBits, validBitsOffset)
 				case reflect.TypeOf(int32(0)), reflect.TypeOf(int64(0)):
 					b.checkRoundTripSpaced(parquet.Encodings.DeltaBinaryPacked, validBits, validBitsOffset)
 				case reflect.TypeOf(parquet.ByteArray{}):
@@ -589,6 +606,28 @@ func TestWriteDeltaBitPackedInt32(t *testing.T) {
 			assert.Equalf(t, values[i:j], valueBuf, "indexes %d:%d", i, j)
 		}
 	})
+
+	t.Run("test decoding multiple pages", func(t *testing.T) {
+		values := make([]int32, 1000)
+		testutils.FillRandomInt32(0, values)
+
+		enc := encoding.NewEncoder(parquet.Types.Int32, parquet.Encodings.DeltaBinaryPacked, false, column, memory.DefaultAllocator)
+		enc.(encoding.Int32Encoder).Put(values)
+		buf, _ := enc.FlushValues()
+		defer buf.Release()
+
+		// Using same Decoder to decode the data.
+		dec := encoding.NewDecoder(parquet.Types.Int32, parquet.Encodings.DeltaBinaryPacked, column, memory.DefaultAllocator)
+		for i := 0; i < 5; i += 1 {
+			dec.(encoding.Int32Decoder).SetData(len(values), buf.Bytes())
+
+			valueBuf := make([]int32, 100)
+			for i, j := 0, len(valueBuf); j <= len(values); i, j = i+len(valueBuf), j+len(valueBuf) {
+				dec.(encoding.Int32Decoder).Decode(valueBuf)
+				assert.Equalf(t, values[i:j], valueBuf, "indexes %d:%d", i, j)
+			}
+		}
+	})
 }
 
 func TestWriteDeltaBitPackedInt64(t *testing.T) {
@@ -641,6 +680,52 @@ func TestWriteDeltaBitPackedInt64(t *testing.T) {
 			assert.Equalf(t, values[i:j], valueBuf, "indexes %d:%d", i, j)
 		}
 	})
+
+	t.Run("GH-37102", func(t *testing.T) {
+		values := []int64{
+			0, 3000000000000000000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 3000000000000000000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 3000000000000000000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 3000000000000000000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 0,
+		}
+
+		enc := encoding.NewEncoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, false, column, memory.DefaultAllocator)
+		enc.(encoding.Int64Encoder).Put(values)
+		buf, _ := enc.FlushValues()
+		defer buf.Release()
+
+		dec := encoding.NewDecoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, column, memory.DefaultAllocator)
+		dec.(encoding.Int64Decoder).SetData(len(values), buf.Bytes())
+
+		valueBuf := make([]int64, len(values))
+
+		decoded, _ := dec.(encoding.Int64Decoder).Decode(valueBuf)
+		assert.Equal(t, len(valueBuf), decoded)
+		assert.Equal(t, values, valueBuf)
+	})
+
+	t.Run("test decoding multiple pages", func(t *testing.T) {
+		values := make([]int64, 1000)
+		testutils.FillRandomInt64(0, values)
+
+		enc := encoding.NewEncoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, false, column, memory.DefaultAllocator)
+		enc.(encoding.Int64Encoder).Put(values)
+		buf, _ := enc.FlushValues()
+		defer buf.Release()
+
+		// Using same Decoder to decode the data.
+		dec := encoding.NewDecoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, column, memory.DefaultAllocator)
+		for i := 0; i < 5; i += 1 {
+			dec.(encoding.Int64Decoder).SetData(len(values), buf.Bytes())
+
+			valueBuf := make([]int64, 100)
+			for i, j := 0, len(valueBuf); j <= len(values); i, j = i+len(valueBuf), j+len(valueBuf) {
+				dec.(encoding.Int64Decoder).Decode(valueBuf)
+				assert.Equalf(t, values[i:j], valueBuf, "indexes %d:%d", i, j)
+			}
+		}
+	})
 }
 
 func TestDeltaLengthByteArrayEncoding(t *testing.T) {
@@ -682,3 +767,96 @@ func TestDeltaByteArrayEncoding(t *testing.T) {
 	assert.Equal(t, len(test), decoded)
 	assert.Equal(t, test, out)
 }
+
+func TestDeltaBitPacking(t *testing.T) {
+	datadir := os.Getenv("ARROW_TEST_DATA")
+	if datadir == "" {
+		return
+	}
+
+	fname := path.Join(datadir, "parquet/timestamp.data")
+	require.FileExists(t, fname)
+	f, err := os.Open(fname)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer f.Close()
+
+	values := make([]int64, 0)
+
+	scanner := bufio.NewScanner(f)
+	for scanner.Scan() {
+		v, err := strconv.ParseInt(scanner.Text(), 10, 64)
+		if err != nil {
+			t.Fatal(err)
+		}
+		values = append(values, v)
+	}
+
+	if err := scanner.Err(); err != nil {
+		t.Fatal(err)
+	}
+
+	col := schema.NewColumn(schema.MustPrimitive(schema.NewPrimitiveNode("foo", parquet.Repetitions.Required,
+		parquet.Types.Int64, -1, -1)), 0, 0)
+	enc := encoding.NewEncoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, false, col, memory.DefaultAllocator).(encoding.Int64Encoder)
+
+	enc.Put(values)
+	buf, err := enc.FlushValues()
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer buf.Release()
+
+	dec := encoding.NewDecoder(parquet.Types.Int64, parquet.Encodings.DeltaBinaryPacked, col, memory.DefaultAllocator).(encoding.Int64Decoder)
+	dec.SetData(len(values), buf.Bytes())
+
+	ll := len(values)
+	for i := 0; i < ll; i += 1024 {
+		out := make([]int64, 1024)
+		n, err := dec.Decode(out)
+		if err != nil {
+			t.Fatal(err)
+		}
+		assert.Equal(t, values[:n], out[:n])
+		values = values[n:]
+	}
+	assert.Equal(t, dec.ValuesLeft(), 0)
+}
+
+func TestBooleanPlainDecoderAfterFlushing(t *testing.T) {
+	descr := schema.NewColumn(schema.NewBooleanNode("bool", parquet.Repetitions.Optional, -1), 0, 0)
+	enc := encoding.NewEncoder(parquet.Types.Boolean, parquet.Encodings.Plain, false, descr, memory.DefaultAllocator)
+	benc := enc.(encoding.BooleanEncoder)
+
+	dec := encoding.NewDecoder(parquet.Types.Boolean, parquet.Encodings.Plain, descr, memory.DefaultAllocator)
+	decSlice := make([]bool, 1)
+	bdec := dec.(encoding.BooleanDecoder)
+
+	// Write and extract two different values
+	// This is validating that `FlushValues` wholly
+	// resets the encoder state.
+	benc.Put([]bool{true})
+	buf1, err := benc.FlushValues()
+	assert.NoError(t, err)
+
+	benc.Put([]bool{false})
+	buf2, err := benc.FlushValues()
+	assert.NoError(t, err)
+
+	// Decode buf1, expect true
+	err = bdec.SetData(1, buf1.Buf())
+	assert.NoError(t, err)
+	n, err := bdec.Decode(decSlice)
+	assert.NoError(t, err)
+	assert.Equal(t, n, 1)
+	assert.Equal(t, decSlice[0], true)
+
+	// Decode buf2, expect false
+	err = bdec.SetData(1, buf2.Buf())
+	assert.NoError(t, err)
+	n, err = bdec.Decode(decSlice)
+	assert.NoError(t, err)
+	assert.Equal(t, n, 1)
+	assert.Equal(t, decSlice[0], false)
+}
