@@ -17,12 +17,14 @@
 package schema
 
 import (
+	"fmt"
 	"reflect"
 	"strconv"
 	"strings"
 
-	"github.com/apache/arrow/go/v6/parquet"
-	format "github.com/apache/arrow/go/v6/parquet/internal/gen-go/parquet"
+	"github.com/apache/arrow/go/v15/arrow/float16"
+	"github.com/apache/arrow/go/v15/parquet"
+	format "github.com/apache/arrow/go/v15/parquet/internal/gen-go/parquet"
 	"golang.org/x/xerrors"
 )
 
@@ -63,6 +65,8 @@ type taggedInfo struct {
 	LogicalType      LogicalType
 	KeyLogicalType   LogicalType
 	ValueLogicalType LogicalType
+
+	Exclude bool
 }
 
 func (t *taggedInfo) CopyForKey() (ret taggedInfo) {
@@ -156,8 +160,10 @@ func (t *taggedInfo) UpdateLogicalTypes() {
 			return BSONLogicalType{}
 		case "uuid":
 			return UUIDLogicalType{}
+		case "float16":
+			return Float16LogicalType{}
 		default:
-			panic(xerrors.Errorf("invalid logical type specified: %s", t))
+			panic(fmt.Errorf("invalid logical type specified: %s", t))
 		}
 	}
 
@@ -185,6 +191,7 @@ func newTaggedInfo() taggedInfo {
 		LogicalType:        NoLogicalType{},
 		KeyLogicalType:     NoLogicalType{},
 		ValueLogicalType:   NoLogicalType{},
+		Exclude:            false,
 	}
 }
 
@@ -208,7 +215,7 @@ func infoFromTags(f reflect.StructTag) *taggedInfo {
 	typeFromStr := func(v string) parquet.Type {
 		t, err := format.TypeFromString(strings.ToUpper(v))
 		if err != nil {
-			panic(xerrors.Errorf("invalid type specified: %s", v))
+			panic(fmt.Errorf("invalid type specified: %s", v))
 		}
 		return parquet.Type(t)
 	}
@@ -231,6 +238,10 @@ func infoFromTags(f reflect.StructTag) *taggedInfo {
 
 	if ptags, ok := f.Lookup("parquet"); ok {
 		info := newTaggedInfo()
+		if ptags == "-" {
+			info.Exclude = true
+			return &info
+		}
 		for _, tag := range strings.Split(strings.Replace(ptags, "\t", "", -1), ",") {
 			tag = strings.TrimSpace(tag)
 			kv := strings.SplitN(tag, "=", 2)
@@ -303,7 +314,7 @@ func infoFromTags(f reflect.StructTag) *taggedInfo {
 	return nil
 }
 
-// typeToNode recurseively converts a physical type and the tag info into parquet Nodes
+// typeToNode recursively converts a physical type and the tag info into parquet Nodes
 //
 // to avoid having to propagate errors up potentially high numbers of recursive calls
 // we use panics and then recover in the public function NewSchemaFromStruct so that a
@@ -365,12 +376,17 @@ func typeToNode(name string, typ reflect.Type, repType parquet.Repetition, info
 		}
 		return Must(MapOf(name, key, value, repType, fieldID))
 	case reflect.Struct:
+		if typ == reflect.TypeOf(float16.Num{}) {
+			return MustPrimitive(NewPrimitiveNodeLogical(name, repType, Float16LogicalType{}, parquet.Types.FixedLenByteArray, 2, fieldID))
+		}
 		// structs are Group nodes
 		fields := make(FieldList, 0)
 		for i := 0; i < typ.NumField(); i++ {
 			f := typ.Field(i)
-
-			fields = append(fields, typeToNode(f.Name, f.Type, parquet.Repetitions.Required, infoFromTags(f.Tag)))
+			tags := infoFromTags(f.Tag)
+			if tags == nil || !tags.Exclude {
+				fields = append(fields, typeToNode(f.Name, f.Type, parquet.Repetitions.Required, tags))
+			}
 		}
 		// group nodes don't have a physical type
 		if physical != parquet.Types.Undefined {
@@ -623,7 +639,7 @@ func typeFromNode(n Node) reflect.Type {
 	switch n.Type() {
 	case Primitive:
 		typ := parquetTypeToReflect[n.(*PrimitiveNode).PhysicalType()]
-		// if a bytearray field is annoted as a String logical type or a UTF8 converted type
+		// if a bytearray field is annotated as a String logical type or a UTF8 converted type
 		// then use a string instead of parquet.ByteArray / parquet.FixedLenByteArray which are []byte
 		if n.LogicalType().Equals(StringLogicalType{}) || n.ConvertedType() == ConvertedTypes.UTF8 {
 			typ = reflect.TypeOf(string(""))
