@@ -2,7 +2,7 @@
 // detail/string_view.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2017 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -17,31 +17,33 @@
 
 #include <boost/asio/detail/config.hpp>
 
-#if defined(BOOST_ASIO_HAS_STD_STRING_VIEW)
+#if defined(BOOST_ASIO_HAS_STRING_VIEW)
 
-#if defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
-# include <experimental/string_view>
-#else // defined(BOOST_ASIO_HAS_EXPERIMENTAL_STRING_VIEW)
+#if defined(BOOST_ASIO_HAS_STD_STRING_VIEW)
 # include <string_view>
-#endif // defined(BOOST_ASIO_HAS_EXPERIMENTAL_STRING_VIEW)
+#elif defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
+# include <experimental/string_view>
+#else // defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
+# error BOOST_ASIO_HAS_STRING_VIEW is set but no string_view is available
+#endif // defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
 
 namespace boost {
 namespace asio {
 
-#if defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
-using std::experimental::basic_string_view;
-using std::experimental::string_view;
-#else // defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
+#if defined(BOOST_ASIO_HAS_STD_STRING_VIEW)
 using std::basic_string_view;
 using std::string_view;
+#elif defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
+using std::experimental::basic_string_view;
+using std::experimental::string_view;
 #endif // defined(BOOST_ASIO_HAS_STD_EXPERIMENTAL_STRING_VIEW)
 
 } // namespace asio
 } // namespace boost
 
 # define BOOST_ASIO_STRING_VIEW_PARAM boost::asio::string_view
-#else // defined(BOOST_ASIO_HAS_STD_STRING_VIEW)
+#else // defined(BOOST_ASIO_HAS_STRING_VIEW)
 # define BOOST_ASIO_STRING_VIEW_PARAM const std::string&
-#endif // defined(BOOST_ASIO_HAS_STD_STRING_VIEW)
+#endif // defined(BOOST_ASIO_HAS_STRING_VIEW)
 
 #endif // BOOST_ASIO_DETAIL_STRING_VIEW_HPP
