@@ -3,9 +3,9 @@
 // Copyright (c) 2007-2012 Barend Gehrels, Amsterdam, the Netherlands.
 // Copyright (c) 2017 Adam Wulkiewicz, Lodz, Poland.
 
-// This file was modified by Oracle on 2017-2020.
-// Modifications copyright (c) 2017-2020 Oracle and/or its affiliates.
-
+// This file was modified by Oracle on 2017-2024.
+// Modifications copyright (c) 2017-2024 Oracle and/or its affiliates.
+// Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Use, modification and distribution is subject to the Boost Software License,
@@ -25,7 +25,6 @@
 #include <boost/geometry/algorithms/detail/overlay/get_turns.hpp>
 #include <boost/geometry/algorithms/detail/sections/section_box_policies.hpp>
 
-#include <boost/geometry/core/access.hpp>
 #include <boost/geometry/core/coordinate_dimension.hpp>
 #include <boost/geometry/core/point_order.hpp>
 #include <boost/geometry/core/tags.hpp>
@@ -33,8 +32,10 @@
 #include <boost/geometry/geometries/box.hpp>
 #include <boost/geometry/geometries/concepts/check.hpp>
 
-#include <boost/geometry/util/condition.hpp>
+#include <boost/geometry/strategies/detail.hpp>
+#include <boost/geometry/strategies/relate/services.hpp>
 
+#include <boost/geometry/util/condition.hpp>
 
 namespace boost { namespace geometry
 {
@@ -63,30 +64,26 @@ template
     typename Geometry,
     typename Turns,
     typename TurnPolicy,
-    typename IntersectionStrategy,
-    typename RobustPolicy,
+    typename Strategy,
     typename InterruptPolicy
 >
 struct self_section_visitor
 {
     Geometry const& m_geometry;
-    IntersectionStrategy const& m_intersection_strategy;
-    RobustPolicy const& m_rescale_policy;
+    Strategy const& m_strategy;
     Turns& m_turns;
     InterruptPolicy& m_interrupt_policy;
     int m_source_index;
     bool m_skip_adjacent;
 
     inline self_section_visitor(Geometry const& g,
-                                IntersectionStrategy const& is,
-                                RobustPolicy const& rp,
+                                Strategy const& s,
                                 Turns& turns,
                                 InterruptPolicy& ip,
                                 int source_index,
                                 bool skip_adjacent)
         : m_geometry(g)
-        , m_intersection_strategy(is)
-        , m_rescale_policy(rp)
+        , m_strategy(s)
         , m_turns(turns)
         , m_interrupt_policy(ip)
         , m_source_index(source_index)
@@ -98,7 +95,7 @@ struct self_section_visitor
     {
         if (! detail::disjoint::disjoint_box_box(sec1.bounding_box,
                                                  sec2.bounding_box,
-                                                 m_intersection_strategy.get_disjoint_box_box_strategy())
+                                                 m_strategy)
                 && ! sec1.duplicate
                 && ! sec2.duplicate)
         {
@@ -112,8 +109,7 @@ struct self_section_visitor
                     >::apply(m_source_index, m_geometry, sec1,
                              m_source_index, m_geometry, sec2,
                              false, m_skip_adjacent,
-                             m_intersection_strategy,
-                             m_rescale_policy,
+                             m_strategy,
                              m_turns, m_interrupt_policy);
         }
 
@@ -127,23 +123,15 @@ struct self_section_visitor
 template <bool Reverse, typename TurnPolicy>
 struct get_turns
 {
-    template <typename Geometry, typename IntersectionStrategy, typename RobustPolicy, typename Turns, typename InterruptPolicy>
+    template <typename Geometry, typename Strategy, typename Turns, typename InterruptPolicy>
     static inline bool apply(
             Geometry const& geometry,
-            IntersectionStrategy const& intersection_strategy,
-            RobustPolicy const& robust_policy,
+            Strategy const& strategy,
             Turns& turns,
             InterruptPolicy& interrupt_policy,
             int source_index, bool skip_adjacent)
     {
-        typedef model::box
-            <
-                typename geometry::robust_point_type
-                <
-                    typename geometry::point_type<Geometry>::type,
-                    RobustPolicy
-                >::type
-            > box_type;
+        typedef model::box<typename geometry::point_type<Geometry>::type> box_type;
 
         // sectionalize in two dimensions to detect
         // all potential spikes correctly
@@ -152,32 +140,23 @@ struct get_turns
         typedef std::integer_sequence<std::size_t, 0, 1> dimensions;
 
         sections_type sec;
-        geometry::sectionalize<Reverse, dimensions>(geometry, robust_policy, sec,
-                                                    intersection_strategy.get_envelope_strategy(),
-                                                    intersection_strategy.get_expand_strategy());
+        geometry::sectionalize<Reverse, dimensions>(geometry,
+                                                    sec, strategy);
 
         self_section_visitor
             <
                 Reverse, Geometry,
-                Turns, TurnPolicy, IntersectionStrategy, RobustPolicy, InterruptPolicy
-            > visitor(geometry, intersection_strategy, robust_policy, turns, interrupt_policy, source_index, skip_adjacent);
-
-        typedef detail::section::get_section_box
-            <
-                typename IntersectionStrategy::expand_box_strategy_type
-            > get_section_box_type;
-        typedef detail::section::overlaps_section_box
-            <
-                typename IntersectionStrategy::disjoint_box_box_strategy_type
-            > overlaps_section_box_type;
+                Turns, TurnPolicy, Strategy, InterruptPolicy
+            > visitor(geometry, strategy, turns, interrupt_policy,
+                      source_index, skip_adjacent);
 
         // false if interrupted
         geometry::partition
             <
                 box_type
             >::apply(sec, visitor,
-                     get_section_box_type(),
-                     overlaps_section_box_type());
+                     detail::section::get_section_box<Strategy>(strategy),
+                     detail::section::overlaps_section_box<Strategy>(strategy));
 
         return ! interrupt_policy.has_intersections;
     }
@@ -231,11 +210,10 @@ struct self_get_turn_points
         TurnPolicy
     >
 {
-    template <typename Strategy, typename RobustPolicy, typename Turns, typename InterruptPolicy>
+    template <typename Strategy, typename Turns, typename InterruptPolicy>
     static inline bool apply(
             Box const& ,
             Strategy const& ,
-            RobustPolicy const& ,
             Turns& ,
             InterruptPolicy& ,
             int /*source_index*/,
@@ -280,25 +258,97 @@ struct self_get_turn_points
 #endif // DOXYGEN_NO_DISPATCH
 
 
+namespace resolve_strategy
+{
+
+template
+<
+    bool Reverse,
+    typename AssignPolicy,
+    typename Strategies,
+    bool IsUmbrella = strategies::detail::is_umbrella_strategy<Strategies>::value
+>
+struct self_get_turn_points
+{
+    template
+    <
+        typename Geometry,
+        typename Turns,
+        typename InterruptPolicy
+    >
+    static inline void apply(Geometry const& geometry,
+                             Strategies const& strategies,
+                             Turns& turns,
+                             InterruptPolicy& interrupt_policy,
+                             int source_index,
+                             bool skip_adjacent)
+    {
+        using turn_policy = detail::overlay::get_turn_info<AssignPolicy>;
+
+        dispatch::self_get_turn_points
+                <
+                    Reverse,
+                    typename tag<Geometry>::type,
+                    Geometry,
+                    turn_policy
+                >::apply(geometry, strategies, turns, interrupt_policy,
+                         source_index, skip_adjacent);
+    }
+};
+
+template <bool Reverse, typename AssignPolicy, typename Strategy>
+struct self_get_turn_points<Reverse, AssignPolicy, Strategy, false>
+{
+    template
+    <
+        typename Geometry,
+        typename Turns,
+        typename InterruptPolicy
+    >
+    static inline void apply(Geometry const& geometry,
+                             Strategy const& strategy,
+                             Turns& turns,
+                             InterruptPolicy& interrupt_policy,
+                             int source_index,
+                             bool skip_adjacent)
+    {
+        using strategies::relate::services::strategy_converter;
+
+        self_get_turn_points
+            <
+                Reverse,
+                AssignPolicy,
+                decltype(strategy_converter<Strategy>::get(strategy))
+            >::apply(geometry,
+                     strategy_converter<Strategy>::get(strategy),
+                     turns,
+                     interrupt_policy,
+                     source_index,
+                     skip_adjacent);
+    }
+};
+
+
+} // namespace resolve_strategy
+
+
 #ifndef DOXYGEN_NO_DETAIL
 namespace detail { namespace self_get_turn_points
 {
 
-// Version where Reverse can be specified manually. TODO: 
+// Version where Reverse can be specified manually. TODO:
 // can most probably be merged with self_get_turn_points::get_turn
 template
 <
     bool Reverse,
     typename AssignPolicy,
     typename Geometry,
-    typename IntersectionStrategy,
-    typename RobustPolicy,
+    typename Strategy,
     typename Turns,
     typename InterruptPolicy
 >
 inline void self_turns(Geometry const& geometry,
-                       IntersectionStrategy const& strategy,
-                       RobustPolicy const& robust_policy,
+                       Strategy const& strategy,
                        Turns& turns,
                        InterruptPolicy& interrupt_policy,
                        int source_index = 0,
@@ -306,15 +356,10 @@ inline void self_turns(Geometry const& geometry,
 {
     concepts::check<Geometry const>();
 
-    typedef detail::overlay::get_turn_info<detail::overlay::assign_null_policy> turn_policy;
-
-    dispatch::self_get_turn_points
+    resolve_strategy::self_get_turn_points
             <
-                Reverse,
-                typename tag<Geometry>::type,
-                Geometry,
-                turn_policy
-            >::apply(geometry, strategy, robust_policy, turns, interrupt_policy,
+                Reverse, AssignPolicy, Strategy
+            >::apply(geometry, strategy, turns, interrupt_policy,
                      source_index, skip_adjacent);
 }
 
@@ -329,7 +374,6 @@ inline void self_turns(Geometry const& geometry,
                 (e.g. vector of "intersection/turn point"'s)
     \param geometry geometry
     \param strategy strategy to be used
-    \param robust_policy policy to handle robustness issues
     \param turns container which will contain intersection points
     \param interrupt_policy policy determining if process is stopped
         when intersection is found
@@ -340,14 +384,12 @@ template
 <
     typename AssignPolicy,
     typename Geometry,
-    typename IntersectionStrategy,
-    typename RobustPolicy,
+    typename Strategy,
     typename Turns,
     typename InterruptPolicy
 >
 inline void self_turns(Geometry const& geometry,
-                       IntersectionStrategy const& strategy,
-                       RobustPolicy const& robust_policy,
+                       Strategy const& strategy,
                        Turns& turns,
                        InterruptPolicy& interrupt_policy,
                        int source_index = 0,
@@ -360,12 +402,11 @@ inline void self_turns(Geometry const& geometry,
             geometry::point_order<Geometry>::value
         >::value;
 
-    detail::self_get_turn_points::self_turns
+    resolve_strategy::self_get_turn_points
             <
-                reverse,
-                AssignPolicy
-            >(geometry, strategy, robust_policy, turns, interrupt_policy,
-              source_index, skip_adjacent);
+                reverse, AssignPolicy, Strategy
+            >::apply(geometry, strategy, turns, interrupt_policy,
+                     source_index, skip_adjacent);
 }
 
 
