@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2023 Antony Polukhin.
+// Copyright (c) 2019-2024 Antony Polukhin.
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -9,7 +9,11 @@
 
 #include <boost/pfr/detail/config.hpp>
 
+#ifdef BOOST_PFR_HAS_STD_MODULE
+import std;
+#else
 #include <type_traits>
+#endif
 
 namespace boost { namespace pfr { namespace detail {
 
