@@ -2,7 +2,7 @@
 // generic/seq_packet_protocol.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -79,18 +79,14 @@ void test()
     spp::socket socket4(ioc, spp(af_inet, 0), native_socket1);
 #endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     spp::socket socket5(std::move(socket4));
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
     // basic_seq_packet_socket operators.
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     socket1 = spp::socket(ioc);
     socket1 = std::move(socket2);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
-    // basic_io_object functions.
+    // I/O object functions.
 
     spp::socket::executor_type ex = socket1.get_executor();
     (void)ex;
