@@ -2,7 +2,7 @@
 // tcp.cpp
 // ~~~~~~~
 //
-// Copyright (c) 2003-2017 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -20,12 +20,12 @@
 #include <boost/asio/ip/tcp.hpp>
 
 #include <cstring>
+#include <functional>
 #include <boost/asio/io_context.hpp>
 #include <boost/asio/read.hpp>
 #include <boost/asio/write.hpp>
 #include "../unit_test.hpp"
 #include "../archetypes/async_result.hpp"
-#include "../archetypes/deprecated_async_result.hpp"
 #include "../archetypes/gettable_socket_option.hpp"
 #include "../archetypes/io_control_command.hpp"
 #include "../archetypes/settable_socket_option.hpp"
@@ -36,12 +36,6 @@
 # include <array>
 #endif // defined(BOOST_ASIO_HAS_BOOST_ARRAY)
 
-#if defined(BOOST_ASIO_HAS_BOOST_BIND)
-# include <boost/bind.hpp>
-#else // defined(BOOST_ASIO_HAS_BOOST_BIND)
-# include <functional>
-#endif // defined(BOOST_ASIO_HAS_BOOST_BIND)
-
 //------------------------------------------------------------------------------
 
 // ip_tcp_compile test
@@ -71,6 +65,9 @@ void test()
     (void)static_cast<bool>(no_delay1);
     (void)static_cast<bool>(!no_delay1);
     (void)static_cast<bool>(no_delay1.value());
+
+    ip::tcp::endpoint ep;
+    (void)static_cast<std::size_t>(std::hash<ip::tcp::endpoint>()(ep));
   }
   catch (std::exception&)
   {
@@ -142,66 +139,54 @@ struct connect_handler
 {
   connect_handler() {}
   void operator()(const boost::system::error_code&) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   connect_handler(connect_handler&&) {}
 private:
   connect_handler(const connect_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct wait_handler
 {
   wait_handler() {}
   void operator()(const boost::system::error_code&) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   wait_handler(wait_handler&&) {}
 private:
   wait_handler(const wait_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct send_handler
 {
   send_handler() {}
   void operator()(const boost::system::error_code&, std::size_t) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   send_handler(send_handler&&) {}
 private:
   send_handler(const send_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct receive_handler
 {
   receive_handler() {}
   void operator()(const boost::system::error_code&, std::size_t) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   receive_handler(receive_handler&&) {}
 private:
   receive_handler(const receive_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct write_some_handler
 {
   write_some_handler() {}
   void operator()(const boost::system::error_code&, std::size_t) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   write_some_handler(write_some_handler&&) {}
 private:
   write_some_handler(const write_some_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct read_some_handler
 {
   read_some_handler() {}
   void operator()(const boost::system::error_code&, std::size_t) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   read_some_handler(read_some_handler&&) {}
 private:
   read_some_handler(const read_some_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 void test()
@@ -218,6 +203,7 @@ void test()
   try
   {
     io_context ioc;
+    const io_context::executor_type ioc_ex = ioc.get_executor();
     char mutable_char_buffer[128] = "";
     const char const_char_buffer[128] = "";
     array<boost::asio::mutable_buffer, 2> mutable_buffers = {{
@@ -234,10 +220,8 @@ void test()
     archetypes::gettable_socket_option<int> gettable_socket_option2;
     archetypes::gettable_socket_option<double> gettable_socket_option3;
     archetypes::io_control_command io_control_command;
+    archetypes::immediate_handler immediate;
     archetypes::lazy_handler lazy;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    archetypes::deprecated_lazy_handler dlazy;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
     boost::system::error_code ec;
 
     // basic_stream_socket constructors.
@@ -253,23 +237,25 @@ void test()
     ip::tcp::socket socket6(ioc, ip::tcp::v4(), native_socket1);
 #endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
 
-#if defined(BOOST_ASIO_HAS_MOVE)
-    ip::tcp::socket socket7(std::move(socket5));
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    ip::tcp::socket socket7(ioc_ex);
+    ip::tcp::socket socket8(ioc_ex, ip::tcp::v4());
+    ip::tcp::socket socket9(ioc_ex, ip::tcp::v6());
+    ip::tcp::socket socket10(ioc_ex, ip::tcp::endpoint(ip::tcp::v4(), 0));
+    ip::tcp::socket socket11(ioc_ex, ip::tcp::endpoint(ip::tcp::v6(), 0));
+#if !defined(BOOST_ASIO_WINDOWS_RUNTIME)
+    ip::tcp::socket::native_handle_type native_socket2
+      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+    ip::tcp::socket socket12(ioc_ex, ip::tcp::v4(), native_socket2);
+#endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
+
+    ip::tcp::socket socket13(std::move(socket5));
 
     // basic_stream_socket operators.
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     socket1 = ip::tcp::socket(ioc);
     socket1 = std::move(socket2);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
-    // basic_io_object functions.
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    io_context& ioc_ref = socket1.get_io_context();
-    (void)ioc_ref;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    // I/O object functions.
 
     ip::tcp::socket::executor_type ex = socket1.get_executor();
     (void)ex;
@@ -279,9 +265,9 @@ void test()
     ip::tcp::socket::lowest_layer_type& lowest_layer = socket1.lowest_layer();
     (void)lowest_layer;
 
-    const ip::tcp::socket& socket8 = socket1;
+    const ip::tcp::socket& socket14 = socket1;
     const ip::tcp::socket::lowest_layer_type& lowest_layer2
-      = socket8.lowest_layer();
+      = socket14.lowest_layer();
     (void)lowest_layer2;
 
     socket1.open(ip::tcp::v4());
@@ -290,12 +276,12 @@ void test()
     socket1.open(ip::tcp::v6(), ec);
 
 #if !defined(BOOST_ASIO_WINDOWS_RUNTIME)
-    ip::tcp::socket::native_handle_type native_socket2
-      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    socket1.assign(ip::tcp::v4(), native_socket2);
     ip::tcp::socket::native_handle_type native_socket3
       = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    socket1.assign(ip::tcp::v4(), native_socket3, ec);
+    socket1.assign(ip::tcp::v4(), native_socket3);
+    ip::tcp::socket::native_handle_type native_socket4
+      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+    socket1.assign(ip::tcp::v4(), native_socket4, ec);
 #endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
 
     bool is_open = socket1.is_open();
@@ -307,9 +293,9 @@ void test()
     socket1.release();
     socket1.release(ec);
 
-    ip::tcp::socket::native_handle_type native_socket4
+    ip::tcp::socket::native_handle_type native_socket5
       = socket1.native_handle();
-    (void)native_socket4;
+    (void)native_socket5;
 
     socket1.cancel();
     socket1.cancel(ec);
@@ -338,18 +324,12 @@ void test()
         connect_handler());
     socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0),
         connect_handler());
+    socket1.async_connect(ip::tcp::endpoint(ip::tcp::v4(), 0), immediate);
+    socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0), immediate);
     int i1 = socket1.async_connect(ip::tcp::endpoint(ip::tcp::v4(), 0), lazy);
     (void)i1;
     int i2 = socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0), lazy);
     (void)i2;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d1 = socket1.async_connect(
-        ip::tcp::endpoint(ip::tcp::v4(), 0), dlazy);
-    (void)d1;
-    double d2 = socket1.async_connect(
-        ip::tcp::endpoint(ip::tcp::v6(), 0), dlazy);
-    (void)d2;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     socket1.set_option(settable_socket_option1);
     socket1.set_option(settable_socket_option1, ec);
@@ -379,10 +359,14 @@ void test()
     socket1.native_non_blocking(false, ec);
 
     ip::tcp::endpoint endpoint1 = socket1.local_endpoint();
+    (void)endpoint1;
     ip::tcp::endpoint endpoint2 = socket1.local_endpoint(ec);
+    (void)endpoint2;
 
     ip::tcp::endpoint endpoint3 = socket1.remote_endpoint();
+    (void)endpoint3;
     ip::tcp::endpoint endpoint4 = socket1.remote_endpoint(ec);
+    (void)endpoint4;
 
     socket1.shutdown(socket_base::shutdown_both);
     socket1.shutdown(socket_base::shutdown_both, ec);
@@ -391,12 +375,9 @@ void test()
     socket1.wait(socket_base::wait_write, ec);
 
     socket1.async_wait(socket_base::wait_read, wait_handler());
+    socket1.async_wait(socket_base::wait_read, immediate);
     int i3 = socket1.async_wait(socket_base::wait_write, lazy);
     (void)i3;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d3 = socket1.async_wait(socket_base::wait_write, dlazy);
-    (void)d3;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     // basic_stream_socket functions.
 
@@ -426,6 +407,16 @@ void test()
     socket1.async_send(mutable_buffers, in_flags, send_handler());
     socket1.async_send(const_buffers, in_flags, send_handler());
     socket1.async_send(null_buffers(), in_flags, send_handler());
+    socket1.async_send(buffer(mutable_char_buffer), immediate);
+    socket1.async_send(buffer(const_char_buffer), immediate);
+    socket1.async_send(mutable_buffers, immediate);
+    socket1.async_send(const_buffers, immediate);
+    socket1.async_send(null_buffers(), immediate);
+    socket1.async_send(buffer(mutable_char_buffer), in_flags, immediate);
+    socket1.async_send(buffer(const_char_buffer), in_flags, immediate);
+    socket1.async_send(mutable_buffers, in_flags, immediate);
+    socket1.async_send(const_buffers, in_flags, immediate);
+    socket1.async_send(null_buffers(), in_flags, immediate);
     int i4 = socket1.async_send(buffer(mutable_char_buffer), lazy);
     (void)i4;
     int i5 = socket1.async_send(buffer(const_char_buffer), lazy);
@@ -446,29 +437,6 @@ void test()
     (void)i12;
     int i13 = socket1.async_send(null_buffers(), in_flags, lazy);
     (void)i13;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d4 = socket1.async_send(buffer(mutable_char_buffer), dlazy);
-    (void)d4;
-    double d5 = socket1.async_send(buffer(const_char_buffer), dlazy);
-    (void)d5;
-    double d6 = socket1.async_send(mutable_buffers, dlazy);
-    (void)d6;
-    double d7 = socket1.async_send(const_buffers, dlazy);
-    (void)d7;
-    double d8 = socket1.async_send(null_buffers(), dlazy);
-    (void)d8;
-    double d9 = socket1.async_send(
-        buffer(mutable_char_buffer), in_flags, dlazy);
-    (void)d9;
-    double d10 = socket1.async_send(buffer(const_char_buffer), in_flags, dlazy);
-    (void)d10;
-    double d11 = socket1.async_send(mutable_buffers, in_flags, dlazy);
-    (void)d11;
-    double d12 = socket1.async_send(const_buffers, in_flags, dlazy);
-    (void)d12;
-    double d13 = socket1.async_send(null_buffers(), in_flags, dlazy);
-    (void)d13;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     socket1.receive(buffer(mutable_char_buffer));
     socket1.receive(mutable_buffers);
@@ -487,6 +455,12 @@ void test()
         receive_handler());
     socket1.async_receive(mutable_buffers, in_flags, receive_handler());
     socket1.async_receive(null_buffers(), in_flags, receive_handler());
+    socket1.async_receive(buffer(mutable_char_buffer), immediate);
+    socket1.async_receive(mutable_buffers, immediate);
+    socket1.async_receive(null_buffers(), immediate);
+    socket1.async_receive(buffer(mutable_char_buffer), in_flags, immediate);
+    socket1.async_receive(mutable_buffers, in_flags, immediate);
+    socket1.async_receive(null_buffers(), in_flags, immediate);
     int i14 = socket1.async_receive(buffer(mutable_char_buffer), lazy);
     (void)i14;
     int i15 = socket1.async_receive(mutable_buffers, lazy);
@@ -500,21 +474,6 @@ void test()
     (void)i18;
     int i19 = socket1.async_receive(null_buffers(), in_flags, lazy);
     (void)i19;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d14 = socket1.async_receive(buffer(mutable_char_buffer), dlazy);
-    (void)d14;
-    double d15 = socket1.async_receive(mutable_buffers, dlazy);
-    (void)d15;
-    double d16 = socket1.async_receive(null_buffers(), dlazy);
-    (void)d16;
-    double d17 = socket1.async_receive(buffer(mutable_char_buffer), in_flags,
-        dlazy);
-    (void)d17;
-    double d18 = socket1.async_receive(mutable_buffers, in_flags, dlazy);
-    (void)d18;
-    double d19 = socket1.async_receive(null_buffers(), in_flags, dlazy);
-    (void)d19;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     socket1.write_some(buffer(mutable_char_buffer));
     socket1.write_some(buffer(const_char_buffer));
@@ -532,6 +491,11 @@ void test()
     socket1.async_write_some(mutable_buffers, write_some_handler());
     socket1.async_write_some(const_buffers, write_some_handler());
     socket1.async_write_some(null_buffers(), write_some_handler());
+    socket1.async_write_some(buffer(mutable_char_buffer), immediate);
+    socket1.async_write_some(buffer(const_char_buffer), immediate);
+    socket1.async_write_some(mutable_buffers, immediate);
+    socket1.async_write_some(const_buffers, immediate);
+    socket1.async_write_some(null_buffers(), immediate);
     int i20 = socket1.async_write_some(buffer(mutable_char_buffer), lazy);
     (void)i20;
     int i21 = socket1.async_write_some(buffer(const_char_buffer), lazy);
@@ -542,18 +506,6 @@ void test()
     (void)i23;
     int i24 = socket1.async_write_some(null_buffers(), lazy);
     (void)i24;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d20 = socket1.async_write_some(buffer(mutable_char_buffer), dlazy);
-    (void)d20;
-    double d21 = socket1.async_write_some(buffer(const_char_buffer), dlazy);
-    (void)d21;
-    double d22 = socket1.async_write_some(mutable_buffers, dlazy);
-    (void)d22;
-    double d23 = socket1.async_write_some(const_buffers, dlazy);
-    (void)d23;
-    double d24 = socket1.async_write_some(null_buffers(), dlazy);
-    (void)d24;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
     socket1.read_some(buffer(mutable_char_buffer));
     socket1.read_some(mutable_buffers);
@@ -565,20 +517,15 @@ void test()
     socket1.async_read_some(buffer(mutable_char_buffer), read_some_handler());
     socket1.async_read_some(mutable_buffers, read_some_handler());
     socket1.async_read_some(null_buffers(), read_some_handler());
+    socket1.async_read_some(buffer(mutable_char_buffer), immediate);
+    socket1.async_read_some(mutable_buffers, immediate);
+    socket1.async_read_some(null_buffers(), immediate);
     int i25 = socket1.async_read_some(buffer(mutable_char_buffer), lazy);
     (void)i25;
     int i26 = socket1.async_read_some(mutable_buffers, lazy);
     (void)i26;
     int i27 = socket1.async_read_some(null_buffers(), lazy);
     (void)i27;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d25 = socket1.async_read_some(buffer(mutable_char_buffer), dlazy);
-    (void)d25;
-    double d26 = socket1.async_read_some(mutable_buffers, dlazy);
-    (void)d26;
-    double d27 = socket1.async_read_some(null_buffers(), dlazy);
-    (void)d27;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
   }
   catch (std::exception&)
   {
@@ -652,13 +599,9 @@ void test()
   using namespace boost::asio;
   namespace ip = boost::asio::ip;
 
-#if defined(BOOST_ASIO_HAS_BOOST_BIND)
-  namespace bindns = boost;
-#else // defined(BOOST_ASIO_HAS_BOOST_BIND)
   namespace bindns = std;
-  using std::placeholders::_1;
-  using std::placeholders::_2;
-#endif // defined(BOOST_ASIO_HAS_BOOST_BIND)
+  using bindns::placeholders::_1;
+  using bindns::placeholders::_2;
 
   io_context ioc;
 
@@ -764,25 +707,20 @@ struct wait_handler
 {
   wait_handler() {}
   void operator()(const boost::system::error_code&) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   wait_handler(wait_handler&&) {}
 private:
   wait_handler(const wait_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 struct accept_handler
 {
   accept_handler() {}
   void operator()(const boost::system::error_code&) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   accept_handler(accept_handler&&) {}
 private:
   accept_handler(const accept_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
-#if defined(BOOST_ASIO_HAS_MOVE)
 struct move_accept_handler
 {
   move_accept_handler() {}
@@ -792,7 +730,17 @@ struct move_accept_handler
 private:
   move_accept_handler(const move_accept_handler&) {}
 };
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+
+struct move_accept_ioc_handler
+{
+  move_accept_ioc_handler() {}
+  void operator()(const boost::system::error_code&,
+      boost::asio::basic_stream_socket<boost::asio::ip::tcp,
+        boost::asio::io_context::executor_type>) {}
+  move_accept_ioc_handler(move_accept_handler&&) {}
+private:
+  move_accept_ioc_handler(const move_accept_handler&) {}
+};
 
 void test()
 {
@@ -802,7 +750,10 @@ void test()
   try
   {
     io_context ioc;
-    ip::tcp::socket peer_socket(ioc);
+    const io_context::executor_type ioc_ex = ioc.get_executor();
+    ip::tcp::socket peer_socket1(ioc);
+    boost::asio::basic_stream_socket<ip::tcp,
+        io_context::executor_type> peer_socket2(ioc);
     ip::tcp::endpoint peer_endpoint;
     archetypes::settable_socket_option<void> settable_socket_option1;
     archetypes::settable_socket_option<int> settable_socket_option2;
@@ -811,10 +762,8 @@ void test()
     archetypes::gettable_socket_option<int> gettable_socket_option2;
     archetypes::gettable_socket_option<double> gettable_socket_option3;
     archetypes::io_control_command io_control_command;
+    archetypes::immediate_handler immediate;
     archetypes::lazy_handler lazy;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    archetypes::deprecated_lazy_handler dlazy;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
     boost::system::error_code ec;
 
     // basic_socket_acceptor constructors.
@@ -830,23 +779,25 @@ void test()
     ip::tcp::acceptor acceptor6(ioc, ip::tcp::v4(), native_acceptor1);
 #endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
 
-#if defined(BOOST_ASIO_HAS_MOVE)
-    ip::tcp::acceptor acceptor7(std::move(acceptor5));
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    ip::tcp::acceptor acceptor7(ioc_ex);
+    ip::tcp::acceptor acceptor8(ioc_ex, ip::tcp::v4());
+    ip::tcp::acceptor acceptor9(ioc_ex, ip::tcp::v6());
+    ip::tcp::acceptor acceptor10(ioc_ex, ip::tcp::endpoint(ip::tcp::v4(), 0));
+    ip::tcp::acceptor acceptor11(ioc_ex, ip::tcp::endpoint(ip::tcp::v6(), 0));
+#if !defined(BOOST_ASIO_WINDOWS_RUNTIME)
+    ip::tcp::acceptor::native_handle_type native_acceptor2
+      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+    ip::tcp::acceptor acceptor12(ioc_ex, ip::tcp::v4(), native_acceptor2);
+#endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
+
+    ip::tcp::acceptor acceptor13(std::move(acceptor5));
 
     // basic_socket_acceptor operators.
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     acceptor1 = ip::tcp::acceptor(ioc);
     acceptor1 = std::move(acceptor2);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
-    // basic_io_object functions.
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    io_context& ioc_ref = acceptor1.get_io_context();
-    (void)ioc_ref;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    // I/O object functions.
 
     ip::tcp::acceptor::executor_type ex = acceptor1.get_executor();
     (void)ex;
@@ -859,12 +810,12 @@ void test()
     acceptor1.open(ip::tcp::v6(), ec);
 
 #if !defined(BOOST_ASIO_WINDOWS_RUNTIME)
-    ip::tcp::acceptor::native_handle_type native_acceptor2
-      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    acceptor1.assign(ip::tcp::v4(), native_acceptor2);
     ip::tcp::acceptor::native_handle_type native_acceptor3
       = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    acceptor1.assign(ip::tcp::v4(), native_acceptor3, ec);
+    acceptor1.assign(ip::tcp::v4(), native_acceptor3);
+    ip::tcp::acceptor::native_handle_type native_acceptor4
+      = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+    acceptor1.assign(ip::tcp::v4(), native_acceptor4, ec);
 #endif // !defined(BOOST_ASIO_WINDOWS_RUNTIME)
 
     bool is_open = acceptor1.is_open();
@@ -876,9 +827,9 @@ void test()
     acceptor1.release();
     acceptor1.release(ec);
 
-    ip::tcp::acceptor::native_handle_type native_acceptor4
+    ip::tcp::acceptor::native_handle_type native_acceptor5
       = acceptor1.native_handle();
-    (void)native_acceptor4;
+    (void)native_acceptor5;
 
     acceptor1.cancel();
     acceptor1.cancel(ec);
@@ -916,51 +867,74 @@ void test()
     acceptor1.native_non_blocking(false, ec);
 
     ip::tcp::endpoint endpoint1 = acceptor1.local_endpoint();
+    (void)endpoint1;
     ip::tcp::endpoint endpoint2 = acceptor1.local_endpoint(ec);
+    (void)endpoint2;
 
     acceptor1.wait(socket_base::wait_read);
     acceptor1.wait(socket_base::wait_write, ec);
 
     acceptor1.async_wait(socket_base::wait_read, wait_handler());
+    acceptor1.async_wait(socket_base::wait_read, immediate);
     int i1 = acceptor1.async_wait(socket_base::wait_write, lazy);
     (void)i1;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d1 = acceptor1.async_wait(socket_base::wait_write, dlazy);
-    (void)d1;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
-    acceptor1.accept(peer_socket);
-    acceptor1.accept(peer_socket, ec);
-    acceptor1.accept(peer_socket, peer_endpoint);
-    acceptor1.accept(peer_socket, peer_endpoint, ec);
+    acceptor1.accept(peer_socket1);
+    acceptor1.accept(peer_socket1, ec);
+    acceptor1.accept(peer_socket1, peer_endpoint);
+    acceptor1.accept(peer_socket1, peer_endpoint, ec);
 
-#if defined(BOOST_ASIO_HAS_MOVE)
-    peer_socket = acceptor1.accept();
-    peer_socket = acceptor1.accept(ioc);
-    peer_socket = acceptor1.accept(peer_endpoint);
-    peer_socket = acceptor1.accept(ioc, peer_endpoint);
-    (void)peer_socket;
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    acceptor1.accept(peer_socket2);
+    acceptor1.accept(peer_socket2, ec);
+    acceptor1.accept(peer_socket2, peer_endpoint);
+    acceptor1.accept(peer_socket2, peer_endpoint, ec);
 
-    acceptor1.async_accept(peer_socket, accept_handler());
-    acceptor1.async_accept(peer_socket, peer_endpoint, accept_handler());
-    int i2 = acceptor1.async_accept(peer_socket, lazy);
+    peer_socket1 = acceptor1.accept();
+    peer_socket1 = acceptor1.accept(ioc);
+    peer_socket1 = acceptor1.accept(ioc_ex);
+    peer_socket1 = acceptor1.accept(peer_endpoint);
+    peer_socket1 = acceptor1.accept(ioc, peer_endpoint);
+    peer_socket1 = acceptor1.accept(ioc_ex, peer_endpoint);
+    (void)peer_socket1;
+
+    peer_socket2 = acceptor1.accept(ioc);
+    peer_socket2 = acceptor1.accept(ioc_ex);
+    peer_socket2 = acceptor1.accept(ioc, peer_endpoint);
+    peer_socket2 = acceptor1.accept(ioc_ex, peer_endpoint);
+    (void)peer_socket2;
+
+    acceptor1.async_accept(peer_socket1, accept_handler());
+    acceptor1.async_accept(peer_socket1, peer_endpoint, accept_handler());
+    acceptor1.async_accept(peer_socket1, immediate);
+    acceptor1.async_accept(peer_socket1, peer_endpoint, immediate);
+    int i2 = acceptor1.async_accept(peer_socket1, lazy);
     (void)i2;
-    int i3 = acceptor1.async_accept(peer_socket, peer_endpoint, lazy);
+    int i3 = acceptor1.async_accept(peer_socket1, peer_endpoint, lazy);
     (void)i3;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d2 = acceptor1.async_accept(peer_socket, dlazy);
-    (void)d2;
-    double d3 = acceptor1.async_accept(peer_socket, peer_endpoint, dlazy);
-    (void)d3;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
-#if defined(BOOST_ASIO_HAS_MOVE)
+    acceptor1.async_accept(peer_socket2, accept_handler());
+    acceptor1.async_accept(peer_socket2, peer_endpoint, accept_handler());
+    acceptor1.async_accept(peer_socket2, immediate);
+    acceptor1.async_accept(peer_socket2, peer_endpoint, immediate);
+    int i4 = acceptor1.async_accept(peer_socket2, lazy);
+    (void)i4;
+    int i5 = acceptor1.async_accept(peer_socket2, peer_endpoint, lazy);
+    (void)i5;
+
     acceptor1.async_accept(move_accept_handler());
     acceptor1.async_accept(ioc, move_accept_handler());
+    acceptor1.async_accept(ioc_ex, move_accept_handler());
+    acceptor1.async_accept(ioc_ex, move_accept_ioc_handler());
     acceptor1.async_accept(peer_endpoint, move_accept_handler());
     acceptor1.async_accept(ioc, peer_endpoint, move_accept_handler());
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    acceptor1.async_accept(ioc_ex, peer_endpoint, move_accept_handler());
+    acceptor1.async_accept(ioc_ex, peer_endpoint, move_accept_ioc_handler());
+    acceptor1.async_accept(immediate);
+    acceptor1.async_accept(ioc, immediate);
+    acceptor1.async_accept(ioc_ex, immediate);
+    acceptor1.async_accept(peer_endpoint, immediate);
+    acceptor1.async_accept(ioc, peer_endpoint, immediate);
+    acceptor1.async_accept(ioc_ex, peer_endpoint, immediate);
   }
   catch (std::exception&)
   {
@@ -1062,23 +1036,9 @@ struct resolve_handler
   resolve_handler() {}
   void operator()(const boost::system::error_code&,
       boost::asio::ip::tcp::resolver::results_type) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
   resolve_handler(resolve_handler&&) {}
 private:
   resolve_handler(const resolve_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
-};
-
-struct legacy_resolve_handler
-{
-  legacy_resolve_handler() {}
-  void operator()(const boost::system::error_code&,
-      boost::asio::ip::tcp::resolver::iterator) {}
-#if defined(BOOST_ASIO_HAS_MOVE)
-  legacy_resolve_handler(legacy_resolve_handler&&) {}
-private:
-  legacy_resolve_handler(const legacy_resolve_handler&);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 };
 
 void test()
@@ -1089,37 +1049,24 @@ void test()
   try
   {
     io_context ioc;
+    const io_context::executor_type ioc_ex = ioc.get_executor();
     archetypes::lazy_handler lazy;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    archetypes::deprecated_lazy_handler dlazy;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
     boost::system::error_code ec;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    ip::tcp::resolver::query q(ip::tcp::v4(), "localhost", "0");
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
     ip::tcp::endpoint e(ip::address_v4::loopback(), 0);
 
     // basic_resolver constructors.
 
     ip::tcp::resolver resolver(ioc);
+    ip::tcp::resolver resolver2(ioc_ex);
 
-#if defined(BOOST_ASIO_HAS_MOVE)
-    ip::tcp::resolver resolver2(std::move(resolver));
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    ip::tcp::resolver resolver3(std::move(resolver));
 
     // basic_resolver operators.
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     resolver = ip::tcp::resolver(ioc);
-    resolver = std::move(resolver2);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    resolver = std::move(resolver3);
 
-    // basic_io_object functions.
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    io_context& ioc_ref = resolver.get_io_context();
-    (void)ioc_ref;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    // I/O object functions.
 
     ip::tcp::resolver::executor_type ex = resolver.get_executor();
     (void)ex;
@@ -1128,111 +1075,65 @@ void test()
 
     resolver.cancel();
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    ip::tcp::resolver::results_type results1 = resolver.resolve(q);
+    ip::tcp::resolver::results_type results1 = resolver.resolve("", "");
     (void)results1;
 
-    ip::tcp::resolver::results_type results2 = resolver.resolve(q, ec);
+    ip::tcp::resolver::results_type results2 = resolver.resolve("", "", ec);
     (void)results2;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
 
-    ip::tcp::resolver::results_type results3 = resolver.resolve("", "");
+    ip::tcp::resolver::results_type results3 =
+      resolver.resolve("", "", ip::tcp::resolver::flags());
     (void)results3;
 
-    ip::tcp::resolver::results_type results4 = resolver.resolve("", "", ec);
+    ip::tcp::resolver::results_type results4 =
+      resolver.resolve("", "", ip::tcp::resolver::flags(), ec);
     (void)results4;
 
     ip::tcp::resolver::results_type results5 =
-      resolver.resolve("", "", ip::tcp::resolver::flags());
+      resolver.resolve(ip::tcp::v4(), "", "");
     (void)results5;
 
     ip::tcp::resolver::results_type results6 =
-      resolver.resolve("", "", ip::tcp::resolver::flags(), ec);
+      resolver.resolve(ip::tcp::v4(), "", "", ec);
     (void)results6;
 
     ip::tcp::resolver::results_type results7 =
-      resolver.resolve(ip::tcp::v4(), "", "");
+      resolver.resolve(ip::tcp::v4(), "", "", ip::tcp::resolver::flags());
     (void)results7;
 
     ip::tcp::resolver::results_type results8 =
-      resolver.resolve(ip::tcp::v4(), "", "", ec);
+      resolver.resolve(ip::tcp::v4(), "", "", ip::tcp::resolver::flags(), ec);
     (void)results8;
 
-    ip::tcp::resolver::results_type results9 =
-      resolver.resolve(ip::tcp::v4(), "", "", ip::tcp::resolver::flags());
+    ip::tcp::resolver::results_type results9 = resolver.resolve(e);
     (void)results9;
 
-    ip::tcp::resolver::results_type results10 =
-      resolver.resolve(ip::tcp::v4(), "", "", ip::tcp::resolver::flags(), ec);
+    ip::tcp::resolver::results_type results10 = resolver.resolve(e, ec);
     (void)results10;
 
-    ip::tcp::resolver::results_type results11 = resolver.resolve(e);
-    (void)results11;
-
-    ip::tcp::resolver::results_type results12 = resolver.resolve(e, ec);
-    (void)results12;
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    resolver.async_resolve(q, resolve_handler());
-    resolver.async_resolve(q, legacy_resolve_handler());
-    int i1 = resolver.async_resolve(q, lazy);
-    (void)i1;
-    double d1 = resolver.async_resolve(q, dlazy);
-    (void)d1;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
     resolver.async_resolve("", "", resolve_handler());
-    resolver.async_resolve("", "", legacy_resolve_handler());
-    int i2 = resolver.async_resolve("", "", lazy);
-    (void)i2;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d2 = resolver.async_resolve("", "", dlazy);
-    (void)d2;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    int i1 = resolver.async_resolve("", "", lazy);
+    (void)i1;
 
     resolver.async_resolve("", "",
         ip::tcp::resolver::flags(), resolve_handler());
-    resolver.async_resolve("", "",
-        ip::tcp::resolver::flags(), legacy_resolve_handler());
-    int i3 = resolver.async_resolve("", "",
+    int i2 = resolver.async_resolve("", "",
         ip::tcp::resolver::flags(), lazy);
-    (void)i3;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d3 = resolver.async_resolve("", "",
-        ip::tcp::resolver::flags(), dlazy);
-    (void)d3;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    (void)i2;
 
     resolver.async_resolve(ip::tcp::v4(), "", "", resolve_handler());
-    resolver.async_resolve(ip::tcp::v4(), "", "", legacy_resolve_handler());
-    int i4 = resolver.async_resolve(ip::tcp::v4(), "", "", lazy);
-    (void)i4;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d4 = resolver.async_resolve(ip::tcp::v4(), "", "", dlazy);
-    (void)d4;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    int i3 = resolver.async_resolve(ip::tcp::v4(), "", "", lazy);
+    (void)i3;
 
     resolver.async_resolve(ip::tcp::v4(),
         "", "", ip::tcp::resolver::flags(), resolve_handler());
-    resolver.async_resolve(ip::tcp::v4(),
-        "", "", ip::tcp::resolver::flags(), legacy_resolve_handler());
-    int i5 = resolver.async_resolve(ip::tcp::v4(),
+    int i4 = resolver.async_resolve(ip::tcp::v4(),
         "", "", ip::tcp::resolver::flags(), lazy);
-    (void)i5;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d5 = resolver.async_resolve(ip::tcp::v4(),
-        "", "", ip::tcp::resolver::flags(), dlazy);
-    (void)d5;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    (void)i4;
 
     resolver.async_resolve(e, resolve_handler());
-    resolver.async_resolve(e, legacy_resolve_handler());
-    int i6 = resolver.async_resolve(e, lazy);
-    (void)i6;
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-    double d6 = resolver.async_resolve(e, dlazy);
-    (void)d6;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
+    int i5 = resolver.async_resolve(e, lazy);
+    (void)i5;
   }
   catch (std::exception&)
   {
@@ -1267,9 +1168,7 @@ void test()
     const ip::basic_resolver_entry<ip::tcp> entry1;
     ip::basic_resolver_entry<ip::tcp> entry2(endpoint, host_name, service_name);
     ip::basic_resolver_entry<ip::tcp> entry3(entry1);
-#if defined(BOOST_ASIO_HAS_MOVE)
     ip::basic_resolver_entry<ip::tcp> entry4(std::move(entry2));
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
     // basic_resolver_entry functions.
 
@@ -1330,19 +1229,15 @@ void test()
 
   ip::tcp::iostream ios1;
 
-#if defined(BOOST_ASIO_HAS_STD_IOSTREAM_MOVE)
   ip::tcp::iostream ios2(std::move(sock));
-#endif // defined(BOOST_ASIO_HAS_STD_IOSTREAM_MOVE)
 
   ip::tcp::iostream ios3("hostname", "service");
 
   // basic_socket_iostream operators.
 
-#if defined(BOOST_ASIO_HAS_STD_IOSTREAM_MOVE)
   ios1 = ip::tcp::iostream();
 
   ios2 = std::move(ios1);
-#endif // defined(BOOST_ASIO_HAS_STD_IOSTREAM_MOVE)
 
   // basic_socket_iostream members.
 
@@ -1352,11 +1247,7 @@ void test()
 
   (void)static_cast<std::streambuf*>(ios1.rdbuf());
 
-#if defined(BOOST_ASIO_ENABLE_OLD_SERVICES)
-  basic_socket<ip::tcp, stream_socket_service<ip::tcp> >& sref = ios1.socket();
-#else // defined(BOOST_ASIO_ENABLE_OLD_SERVICES)
   basic_socket<ip::tcp>& sref = ios1.socket();
-#endif // defined(BOOST_ASIO_ENABLE_OLD_SERVICES)
   (void)sref;
 
   boost::system::error_code ec = ios1.error();
@@ -1367,7 +1258,7 @@ void test()
 
   ios1.expires_at(tp);
 
-  ip::tcp::iostream::duration d;
+  ip::tcp::iostream::duration d = ip::tcp::iostream::duration();
   ios1.expires_after(d);
 
   // iostream operators.
@@ -1385,14 +1276,14 @@ void test()
 BOOST_ASIO_TEST_SUITE
 (
   "ip/tcp",
-  BOOST_ASIO_TEST_CASE(ip_tcp_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_compile::test)
   BOOST_ASIO_TEST_CASE(ip_tcp_runtime::test)
-  BOOST_ASIO_TEST_CASE(ip_tcp_socket_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_socket_compile::test)
   BOOST_ASIO_TEST_CASE(ip_tcp_socket_runtime::test)
-  BOOST_ASIO_TEST_CASE(ip_tcp_acceptor_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_acceptor_compile::test)
   BOOST_ASIO_TEST_CASE(ip_tcp_acceptor_runtime::test)
-  BOOST_ASIO_TEST_CASE(ip_tcp_resolver_compile::test)
-  BOOST_ASIO_TEST_CASE(ip_tcp_resolver_entry_compile::test)
-  BOOST_ASIO_TEST_CASE(ip_tcp_resolver_entry_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_resolver_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_resolver_entry_compile::test)
+  BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_resolver_entry_compile::test)
   BOOST_ASIO_COMPILE_TEST_CASE(ip_tcp_iostream_compile::test)
 )
