@@ -3,8 +3,9 @@
 
 // Copyright (c) 2015 Barend Gehrels, Amsterdam, the Netherlands.
 
-// This file was modified by Oracle on 2017-2021.
-// Modifications copyright (c) 2017-2021, Oracle and/or its affiliates.
+// This file was modified by Oracle on 2017-2024.
+// Modifications copyright (c) 2017-2024, Oracle and/or its affiliates.
+// Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Use, modification and distribution is subject to the Boost Software License,
@@ -318,9 +319,9 @@ struct map_visitor
         // Create a rounded off point
         std::pair<int, int> p
             = std::make_pair(
-                boost::numeric_cast<int>(half
+                util::numeric_cast<int>(half
                     + ten * bg::get<0>(turn.point)),
-                boost::numeric_cast<int>(half
+                util::numeric_cast<int>(half
                     + ten * bg::get<1>(turn.point))
                 );
         m_mapper.text(turn.point, text, style, margin, m_offsets[p], lineheight);
@@ -363,9 +364,6 @@ void test_overlay(std::string const& caseid,
         << "_" << string_from_type<typename bg::coordinate_type<Geometry>::type>::name()
         << (ccw ? "_ccw" : "")
         << (open ? "_open" : "")
-#if defined(BOOST_GEOMETRY_USE_RESCALING)
-        << "_rescaled"
-#endif
         << ".svg";
 
     std::ofstream svg(filename.str().c_str());
@@ -404,15 +402,6 @@ void test_overlay(std::string const& caseid,
 
     strategy_type strategy;
 
-    using rescale_policy_type = typename bg::rescale_overlay_policy_type
-    <
-        Geometry,
-        Geometry
-    >::type;
-
-    rescale_policy_type robust_policy
-        = bg::get_rescale_policy<rescale_policy_type>(g1, g2);
-
 #if defined(TEST_WITH_SVG)
     map_visitor<svg_mapper> visitor(mapper);
 #else
@@ -420,7 +409,7 @@ void test_overlay(std::string const& caseid,
 #endif
 
     Geometry result;
-    overlay::apply(g1, g2, robust_policy, std::back_inserter(result),
+    overlay::apply(g1, g2, std::back_inserter(result),
                    strategy, visitor);
 
     std::string message;
