@@ -42,25 +42,26 @@ Inode::~Inode()
   }
 }
 
-ostream& operator<<(ostream &out, const Inode &in)
+void Inode::print(std::ostream& out) const
 {
-  out << in.vino() << "("
-      << "faked_ino=" << in.faked_ino
-      << " nref=" << in.get_nref()
-      << " ll_ref=" << in.ll_ref
-      << " cap_refs=" << in.cap_refs
-      << " open=" << in.open_by_mode
-      << " mode=" << oct << in.mode << dec
-      << " size=" << in.size << "/" << in.max_size
-      << " nlink=" << in.nlink
-      << " btime=" << in.btime
-      << " mtime=" << in.mtime
-      << " ctime=" << in.ctime
-      << " caps=" << ccap_string(in.caps_issued());
-  if (!in.caps.empty()) {
+  out << vino() << "("
+      << "faked_ino=" << faked_ino
+      << " nref=" << get_nref()
+      << " ll_ref=" << ll_ref
+      << " cap_refs=" << cap_refs
+      << " open=" << open_by_mode
+      << " mode=" << oct << mode << dec
+      << " size=" << size << "/" << max_size
+      << " nlink=" << nlink
+      << " btime=" << btime
+      << " mtime=" << mtime
+      << " ctime=" << ctime
+      << " change_attr=" << change_attr
+      << " caps=" << ccap_string(caps_issued());
+  if (!caps.empty()) {
     out << "(";
     bool first = true;
-    for (const auto &pair : in.caps) {
+    for (const auto &pair : caps) {
       if (!first)
         out << ',';
       out << pair.first << '=' << ccap_string(pair.second.issued);
@@ -68,28 +69,31 @@ ostream& operator<<(ostream &out, const Inode &in)
     }
     out << ")";
   }
-  if (in.dirty_caps)
-    out << " dirty_caps=" << ccap_string(in.dirty_caps);
-  if (in.flushing_caps)
-    out << " flushing_caps=" << ccap_string(in.flushing_caps);
+  if (dirty_caps)
+    out << " dirty_caps=" << ccap_string(dirty_caps);
+  if (flushing_caps)
+    out << " flushing_caps=" << ccap_string(flushing_caps);
 
-  if (in.flags & I_COMPLETE)
+  if (flags & I_COMPLETE)
     out << " COMPLETE";
 
-  if (in.is_file())
-    out << " " << in.oset;
+  if (is_file())
+    out << " " << oset;
 
-  if (!in.dentries.empty())
-    out << " parents=" << in.dentries;
+  if (!dentries.empty())
+    out << " parents=" << dentries;
 
-  if (in.is_dir() && in.has_dir_layout())
+  if (is_dir() && has_dir_layout())
     out << " has_dir_layout";
 
-  if (in.quota.is_enable())
-    out << " " << in.quota;
+  if (quota.is_enabled())
+    out << " " << quota;
 
-  out << ' ' << &in << ")";
-  return out;
+  if (optmetadata.size() > 0) {
+    out << " " << optmetadata;
+  }
+
+  out << ' ' << this << ")";
 }
 
 
@@ -118,6 +122,22 @@ void Inode::make_short_path(filepath& p)
     p = filepath(ino);
 }
 
+/*
+ * make a filepath suitable for mds auth access check:
+ */
+bool Inode::make_path_string(std::string& s)
+{
+  if (client->_get_root_ino(false) == ino) {
+    return true;
+  } else if (!dentries.empty()) {
+    Dentry *dn = get_first_parent();
+    ceph_assert(dn->dir && dn->dir->parent_inode);
+    return dn->make_path_string(s);
+  }
+
+  return false;
+}
+
 /*
  * make a filepath suitable for an mds request:
  *  - if we are non-snapped/live, the ino is sufficient, e.g. #1234
@@ -687,13 +707,13 @@ int Inode::set_deleg(Fh *fh, unsigned type, ceph_deleg_cb_t cb, void *priv)
    * allow it, with an unusual error to make it clear.
    */
   if (!client->get_deleg_timeout())
-    return -CEPHFS_ETIME;
+    return -ETIME;
 
   // Just say no if we have any recalled delegs still outstanding
   if (has_recalled_deleg()) {
     lsubdout(client->cct, client, 10) << __func__ <<
 	  ": has_recalled_deleg" << dendl;
-    return -CEPHFS_EAGAIN;
+    return -EAGAIN;
   }
 
   // check vs. currently open files on this inode
@@ -702,17 +722,17 @@ int Inode::set_deleg(Fh *fh, unsigned type, ceph_deleg_cb_t cb, void *priv)
     if (open_count_for_write()) {
       lsubdout(client->cct, client, 10) << __func__ <<
 	    ": open for write" << dendl;
-      return -CEPHFS_EAGAIN;
+      return -EAGAIN;
     }
     break;
   case CEPH_DELEGATION_WR:
     if (open_count() > 1) {
       lsubdout(client->cct, client, 10) << __func__ << ": open" << dendl;
-      return -CEPHFS_EAGAIN;
+      return -EAGAIN;
     }
     break;
   default:
-    return -CEPHFS_EINVAL;
+    return -EINVAL;
   }
 
   /*
@@ -733,7 +753,7 @@ int Inode::set_deleg(Fh *fh, unsigned type, ceph_deleg_cb_t cb, void *priv)
   if (!caps_issued_mask(need)) {
     lsubdout(client->cct, client, 10) << __func__ << ": cap mismatch, have="
       << ccap_string(caps_issued()) << " need=" << ccap_string(need) << dendl;
-    return -CEPHFS_EAGAIN;
+    return -EAGAIN;
   }
 
   for (list<Delegation>::iterator d = delegations.begin();
