@@ -34,7 +34,7 @@
 #include "common/cmdparse.h"
 #include "common/code_environment.h"
 #include "msg/msg_types.h"
-#if defined(WITH_SEASTAR) && !defined(WITH_ALIEN)
+#ifdef WITH_CRIMSON
 #include "crimson/common/config_proxy.h"
 #include "crimson/common/perf_counters_collection.h"
 #else
@@ -47,6 +47,7 @@
 #include "crush/CrushLocation.h"
 
 class AdminSocket;
+class AdminSocketHook;
 class CryptoHandler;
 class CryptoRandom;
 class MonMap;
@@ -66,7 +67,7 @@ namespace ceph {
   }
 }
 
-#if defined(WITH_SEASTAR) && !defined(WITH_ALIEN)
+#ifdef WITH_CRIMSON
 namespace crimson::common {
 class CephContext {
 public:
@@ -282,10 +283,18 @@ public:
   void set_mon_addrs(const MonMap& mm);
   void set_mon_addrs(const std::vector<entity_addrvec_t>& in) {
     auto ptr = std::make_shared<std::vector<entity_addrvec_t>>(in);
+#ifdef __cpp_lib_atomic_shared_ptr
+    _mon_addrs.store(std::move(ptr), std::memory_order_relaxed);
+#else
     atomic_store_explicit(&_mon_addrs, std::move(ptr), std::memory_order_relaxed);
+#endif
   }
   std::shared_ptr<std::vector<entity_addrvec_t>> get_mon_addrs() const {
+#ifdef __cpp_lib_atomic_shared_ptr
+    auto ptr = _mon_addrs.load(std::memory_order_relaxed);
+#else
     auto ptr = atomic_load_explicit(&_mon_addrs, std::memory_order_relaxed);
+#endif
     return ptr;
   }
 
@@ -306,7 +315,11 @@ private:
 
   int _crypto_inited;
 
+#ifdef __cpp_lib_atomic_shared_ptr
+  std::atomic<std::shared_ptr<std::vector<entity_addrvec_t>>> _mon_addrs;
+#else
   std::shared_ptr<std::vector<entity_addrvec_t>> _mon_addrs;
+#endif
 
   /* libcommon service thread.
    * SIGHUP wakes this thread, which then reopens logfiles */
@@ -366,8 +379,13 @@ private:
 #ifdef CEPH_DEBUG_MUTEX
   md_config_obs_t *_lockdep_obs;
 #endif
+
+  std::unique_ptr<AdminSocketHook> _msgr_hook;
+  ceph::mutex _msgr_hook_lock = ceph::make_mutex("CephContext::msgr_hook");
 public:
   TOPNSPC::crush::CrushLocation crush_location;
+  void modify_msgr_hook(std::function<AdminSocketHook*(void)> create,
+			std::function<void(AdminSocketHook*)> add);
 private:
 
   enum {
@@ -406,9 +424,9 @@ private:
 #ifdef __cplusplus
 }
 #endif
-#endif	// WITH_SEASTAR
+#endif	// WITH_CRIMSON
 
-#if !(defined(WITH_SEASTAR) && !defined(WITH_ALIEN)) && defined(__cplusplus)
+#if !defined(WITH_CRIMSON) && defined(__cplusplus)
 namespace ceph::common {
 inline void intrusive_ptr_add_ref(CephContext* cct)
 {
@@ -420,5 +438,5 @@ inline void intrusive_ptr_release(CephContext* cct)
   cct->put();
 }
 }
-#endif // !(defined(WITH_SEASTAR) && !defined(WITH_ALIEN)) && defined(__cplusplus)
+#endif // !defined(WITH_CRIMSON) && defined(__cplusplus)
 #endif
