@@ -83,9 +83,9 @@ class Client : public crimson::net::Dispatcher,
   using command_result_t =
     seastar::future<std::tuple<std::int32_t, std::string, ceph::bufferlist>>;
   struct mon_command_t {
-    ceph::ref_t<MMonCommand> req;
+    MURef<MMonCommand> req;
     typename command_result_t::promise_type result;
-    mon_command_t(ceph::ref_t<MMonCommand> req);
+    mon_command_t(MURef<MMonCommand> req);
   };
   std::vector<mon_command_t> mon_commands;
 
@@ -126,11 +126,12 @@ private:
 			 const std::vector<uint32_t>& preferred_modes) final;
   AuthAuthorizeHandler* get_auth_authorize_handler(int peer_type,
 						   int auth_method) final;
-  int handle_auth_request(crimson::net::ConnectionRef conn,
-			  AuthConnectionMetaRef auth_meta,
+  int handle_auth_request(crimson::net::Connection &conn,
+			  AuthConnectionMeta &auth_meta,
 			  bool more,
 			  uint32_t auth_method,
 			  const ceph::bufferlist& payload,
+			  uint64_t *p_peer_global_id,
 			  ceph::bufferlist *reply) final;
 
   crimson::common::CephContext cct; // for auth_registry
@@ -139,24 +140,24 @@ private:
 
   // AuthClient methods
   crimson::auth::AuthClient::auth_request_t
-  get_auth_request(crimson::net::ConnectionRef conn,
-		   AuthConnectionMetaRef auth_meta) final;
+  get_auth_request(crimson::net::Connection &conn,
+		   AuthConnectionMeta &auth_meta) final;
 
    // Handle server's request to continue the handshake
-  ceph::bufferlist handle_auth_reply_more(crimson::net::ConnectionRef conn,
-					  AuthConnectionMetaRef auth_meta,
+  ceph::bufferlist handle_auth_reply_more(crimson::net::Connection &conn,
+					  AuthConnectionMeta &auth_meta,
 					  const bufferlist& bl) final;
 
    // Handle server's indication that authentication succeeded
-  int handle_auth_done(crimson::net::ConnectionRef conn,
-		       AuthConnectionMetaRef auth_meta,
+  int handle_auth_done(crimson::net::Connection &conn,
+		       AuthConnectionMeta &auth_meta,
 		       uint64_t global_id,
 		       uint32_t con_mode,
 		       const bufferlist& bl) final;
 
    // Handle server's indication that the previous auth attempt failed
-  int handle_auth_bad_method(crimson::net::ConnectionRef conn,
-			     AuthConnectionMetaRef auth_meta,
+  int handle_auth_bad_method(crimson::net::Connection &conn,
+			     AuthConnectionMeta &auth_meta,
 			     uint32_t old_auth_method,
 			     int result,
 			     const std::vector<uint32_t>& allowed_methods,
@@ -169,9 +170,9 @@ private:
                                                MessageRef m) override;
   void ms_handle_reset(crimson::net::ConnectionRef conn, bool is_replace) override;
 
-  seastar::future<> handle_monmap(crimson::net::ConnectionRef conn,
+  seastar::future<> handle_monmap(crimson::net::Connection &conn,
 				  Ref<MMonMap> m);
-  seastar::future<> handle_auth_reply(crimson::net::ConnectionRef conn,
+  seastar::future<> handle_auth_reply(crimson::net::Connection &conn,
 				      Ref<MAuthReply> m);
   seastar::future<> handle_subscribe_ack(Ref<MMonSubscribeAck> m);
   seastar::future<> handle_get_version_reply(Ref<MMonGetVersionReply> m);
@@ -193,7 +194,7 @@ private:
   std::vector<unsigned> get_random_mons(unsigned n) const;
   seastar::future<> _add_conn(unsigned rank, uint64_t global_id);
   void _finish_auth(const entity_addr_t& peer);
-  crimson::common::Gated gate;
+  crimson::common::gate_per_shard gates;
 
   // messages that are waiting for the active_con to be available
   struct pending_msg_t {
@@ -211,3 +212,7 @@ inline std::ostream& operator<<(std::ostream& out, const Client& client) {
 }
 
 } // namespace crimson::mon
+
+#if FMT_VERSION >= 90000
+template <> struct fmt::formatter<crimson::mon::Client> : fmt::ostream_formatter {};
+#endif
