@@ -49,7 +49,7 @@ CircularJournalSpace::roll_ertr::future<> CircularJournalSpace::roll() {
   return roll_ertr::now();
 }
 
-CircularJournalSpace::write_ret
+CircularJournalSpace::write_ertr::future<>
 CircularJournalSpace::write(ceph::bufferlist&& to_write) {
   LOG_PREFIX(CircularJournalSpace::write);
   assert(get_written_to().segment_seq != NULL_SEG_SEQ);
@@ -60,7 +60,6 @@ CircularJournalSpace::write(ceph::bufferlist&& to_write) {
   assert(encoded_size + get_rbm_addr(get_written_to())
 	 < get_journal_end());
 
-  journal_seq_t j_seq = get_written_to();
   auto target = get_rbm_addr(get_written_to());
   auto new_written_to = target + encoded_size;
   assert(new_written_to < get_journal_end());
@@ -69,22 +68,12 @@ CircularJournalSpace::write(ceph::bufferlist&& to_write) {
     get_device_id());
   set_written_to(
     journal_seq_t{get_written_to().segment_seq, paddr});
-  DEBUG("{}, target {}", to_write.length(), target);
+  DEBUG("length {}, commit target {}, used_size {}",
+        encoded_size, target, get_records_used_size());
 
-  auto write_result = write_result_t{
-    j_seq,
-    encoded_size
-  };
   return device_write_bl(target, to_write
-  ).safe_then([this, target,
-    length=encoded_size,
-    write_result,
-    FNAME] {
-    DEBUG("commit target {} used_size {} written length {}",
-          target, get_records_used_size(), length);
-    return write_result;
-  }).handle_error(
-    base_ertr::pass_further{},
+  ).handle_error(
+    write_ertr::pass_further{},
     crimson::ct_error::assert_all{ "Invalid error" }
   );
 }
@@ -167,7 +156,8 @@ ceph::bufferlist CircularJournalSpace::encode_header()
   return bl;
 }
 
-CircularJournalSpace::write_ertr::future<> CircularJournalSpace::device_write_bl(
+CircularJournalSpace::submit_ertr::future<>
+CircularJournalSpace::device_write_bl(
     rbm_abs_addr offset, bufferlist &bl)
 {
   LOG_PREFIX(CircularJournalSpace::device_write_bl);
@@ -181,7 +171,7 @@ CircularJournalSpace::write_ertr::future<> CircularJournalSpace::device_write_bl
     length);
   return device->writev(offset, bl
   ).handle_error(
-    write_ertr::pass_further{},
+    submit_ertr::pass_further{},
     crimson::ct_error::assert_all{ "Invalid error device->write" }
   );
 }
@@ -229,7 +219,7 @@ CircularJournalSpace::read_header()
   });
 }
 
-CircularJournalSpace::write_ertr::future<>
+CircularJournalSpace::submit_ertr::future<>
 CircularJournalSpace::write_header()
 {
   LOG_PREFIX(CircularJournalSpace::write_header);
@@ -245,7 +235,7 @@ CircularJournalSpace::write_header()
   iter.copy(bl.length(), bp.c_str());
   return device->write(device->get_shard_journal_start(), std::move(bp)
   ).handle_error(
-    write_ertr::pass_further{},
+    submit_ertr::pass_further{},
     crimson::ct_error::assert_all{ "Invalid error device->write" }
   );
 }
