@@ -3,7 +3,9 @@
 
 #include <fmt/format.h>
 
+#include "crimson/common/coroutine.h"
 #include "crimson/common/exception.h"
+#include "crimson/common/log.h"
 #include "crimson/osd/recovery_backend.h"
 #include "crimson/osd/pg.h"
 #include "crimson/osd/pg_backend.h"
@@ -12,23 +14,20 @@
 #include "messages/MOSDFastDispatchOp.h"
 #include "osd/osd_types.h"
 
-namespace {
-  seastar::logger& logger() {
-    return crimson::get_logger(ceph_subsys_osd);
-  }
-}
+SET_SUBSYS(osd);
 
 hobject_t RecoveryBackend::get_temp_recovery_object(
   const hobject_t& target,
   eversion_t version) const
 {
+  LOG_PREFIX(RecoveryBackend::get_temp_recovery_object);
   hobject_t hoid =
     target.make_temp_hobject(fmt::format("temp_recovering_{}_{}_{}_{}",
                                          pg.get_info().pgid,
                                          version,
                                          pg.get_info().history.same_interval_since,
                                          target.snap));
-  logger().debug("{} {}", __func__, hoid);
+  DEBUGDPP("{}", pg, hoid);
   return hoid;
 }
 
@@ -43,7 +42,7 @@ void RecoveryBackend::clear_temp_obj(const hobject_t &oid)
 }
 
 void RecoveryBackend::clean_up(ceph::os::Transaction& t,
-			       std::string_view why)
+			       interrupt_cause_t why)
 {
   for_each_temp_obj([&](auto &soid) {
     t.remove(pg.get_collection_ref()->get_cid(),
@@ -51,6 +50,8 @@ void RecoveryBackend::clean_up(ceph::os::Transaction& t,
   });
   clear_temp_objs();
 
+  replica_push_targets.clear();
+
   for (auto& [soid, recovery_waiter] : recovering) {
     if ((recovery_waiter->pull_info
          && recovery_waiter->pull_info->is_complete())
@@ -65,6 +66,36 @@ void RecoveryBackend::clean_up(ceph::os::Transaction& t,
   recovering.clear();
 }
 
+void RecoveryBackend::WaitForObjectRecovery::interrupt(interrupt_cause_t why) {
+  switch(why) {
+  case interrupt_cause_t::INTERVAL_CHANGE:
+    if (readable) {
+      readable->set_exception(
+	crimson::common::actingset_changed(pg.is_primary()));
+      readable.reset();
+    }
+    if (recovered) {
+      recovered->set_exception(
+	crimson::common::actingset_changed(pg.is_primary()));
+      recovered.reset();
+    }
+    if (pulled) {
+      pulled->set_exception(
+	crimson::common::actingset_changed(pg.is_primary()));
+      pulled.reset();
+    }
+    for (auto& [pg_shard, pr] : pushes) {
+      pr.set_exception(
+	crimson::common::actingset_changed(pg.is_primary()));
+    }
+    pushes.clear();
+    break;
+  default:
+    ceph_abort("impossible");
+    break;
+  }
+}
+
 void RecoveryBackend::WaitForObjectRecovery::stop() {
   if (readable) {
     readable->set_exception(
@@ -92,7 +123,8 @@ void RecoveryBackend::handle_backfill_finish(
   MOSDPGBackfill& m,
   crimson::net::ConnectionXcoreRef conn)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_backfill_finish);
+  DEBUGDPP("", pg);
   ceph_assert(!pg.is_primary());
   ceph_assert(crimson::common::local_conf()->osd_kill_backfill_at != 1);
   auto reply = crimson::make_message<MOSDPGBackfill>(
@@ -115,7 +147,8 @@ RecoveryBackend::interruptible_future<>
 RecoveryBackend::handle_backfill_progress(
   MOSDPGBackfill& m)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_backfill_progress);
+  DEBUGDPP("", pg);
   ceph_assert(!pg.is_primary());
   ceph_assert(crimson::common::local_conf()->osd_kill_backfill_at != 2);
 
@@ -125,7 +158,7 @@ RecoveryBackend::handle_backfill_progress(
     m.stats,
     m.op == MOSDPGBackfill::OP_BACKFILL_PROGRESS,
     t);
-  logger().debug("RecoveryBackend::handle_backfill_progress: do_transaction...");
+  DEBUGDPP("submitting transaction", pg);
   return shard_services.get_store().do_transaction(
     pg.get_collection_ref(), std::move(t)).or_terminate();
 }
@@ -134,11 +167,12 @@ RecoveryBackend::interruptible_future<>
 RecoveryBackend::handle_backfill_finish_ack(
   MOSDPGBackfill& m)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_backfill_finish_ack);
+  DEBUGDPP("", pg);
   ceph_assert(pg.is_primary());
   ceph_assert(crimson::common::local_conf()->osd_kill_backfill_at != 3);
-  // TODO:
-  // finish_recovery_op(hobject_t::get_max());
+  auto recovery_handler = pg.get_recovery_handler();
+  recovery_handler->backfill_target_finished();
   return seastar::now();
 }
 
@@ -147,9 +181,10 @@ RecoveryBackend::handle_backfill(
   MOSDPGBackfill& m,
   crimson::net::ConnectionXcoreRef conn)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_backfill);
+  DEBUGDPP("", pg);
   if (pg.old_peering_msg(m.map_epoch, m.query_epoch)) {
-    logger().debug("{}: discarding {}", __func__, m);
+    DEBUGDPP("discarding {}", pg, m);
     return seastar::now();
   }
   switch (m.op) {
@@ -170,75 +205,155 @@ RecoveryBackend::interruptible_future<>
 RecoveryBackend::handle_backfill_remove(
   MOSDPGBackfillRemove& m)
 {
-  logger().debug("{} m.ls={}", __func__, m.ls);
+  LOG_PREFIX(RecoveryBackend::handle_backfill_remove);
+  DEBUGDPP("m.ls={}", pg, m.ls);
   assert(m.get_type() == MSG_OSD_PG_BACKFILL_REMOVE);
 
   ObjectStore::Transaction t;
   for ([[maybe_unused]] const auto& [soid, ver] : m.ls) {
     // TODO: the reserved space management. PG::try_reserve_recovery_space().
-    t.remove(pg.get_collection_ref()->get_cid(),
-	      ghobject_t(soid, ghobject_t::NO_GEN, pg.get_pg_whoami().shard));
+    co_await interruptor::async([this, soid=soid, &t] {
+      pg.remove_maybe_snapmapped_object(t, soid);
+    });
   }
-  logger().debug("RecoveryBackend::handle_backfill_remove: do_transaction...");
-  return shard_services.get_store().do_transaction(
-    pg.get_collection_ref(), std::move(t)).or_terminate();
+  DEBUGDPP("submitting transaction", pg);
+  co_await interruptor::make_interruptible(
+    shard_services.get_store().do_transaction(
+      pg.get_collection_ref(), std::move(t)).or_terminate());
 }
 
-RecoveryBackend::interruptible_future<BackfillInterval>
-RecoveryBackend::scan_for_backfill(
-  const hobject_t& start,
+RecoveryBackend::interruptible_future<PrimaryBackfillInterval>
+RecoveryBackend::scan_for_backfill_primary(
+  const hobject_t start,
+  [[maybe_unused]] const std::int64_t min,
+  const std::int64_t max,
+  const std::set<pg_shard_t> &backfill_targets)
+{
+  LOG_PREFIX(RecoveryBackend::scan_for_backfill_primary);
+  DEBUGDPP("starting from {}", pg, start);
+  auto version_map = seastar::make_lw_shared<std::multimap<hobject_t,
+		       std::pair<shard_id_t,eversion_t>>>();
+  auto&& [objects, next] = co_await backend->list_objects(start, max);
+  co_await interruptor::parallel_for_each(objects,
+    seastar::coroutine::lambda([FNAME, this, version_map, backfill_targets]
+    (const hobject_t& object) -> interruptible_future<> {
+    DEBUGDPP("querying obj:{}", pg, object);
+    auto obc_manager = pg.obc_loader.get_obc_manager(
+      object, /* resolve_clone = */ false);
+
+    auto found = co_await pg.obc_loader.load_and_lock(
+      obc_manager, RWState::RWREAD
+    ).si_then([] {
+      return true;
+    }).handle_error_interruptible(
+      crimson::ct_error::enoent::handle([](auto) {
+	return false;
+      }),
+      crimson::ct_error::assert_all(fmt::format("{} {} error when loading obc", pg, FNAME).c_str())
+    );
+    if (!found) {
+      // if the object does not exist here, it must have been removed
+      // between the collection_list_partial and here.  This can happen
+      // for the first item in the range, which is usually last_backfill.
+      co_return;
+    }
+
+    if (obc_manager.get_obc()->obs.exists) {
+      auto version = obc_manager.get_obc()->obs.oi.version;
+      auto shard_versions = obc_manager.get_obc()->obs.oi.shard_versions;
+      if (shard_versions.empty()) {
+	version_map->emplace(object, std::make_pair(shard_id_t::NO_SHARD,
+						    version));
+      } else {
+	bool added_default = false;
+	for (auto & shard: backfill_targets) {
+	  if (shard_versions.contains(shard.shard)) {
+	    auto shard_version = shard_versions.at(shard.shard);
+	    version_map->emplace(object, std::make_pair(shard.shard,
+							shard_version));
+	  } else if (!added_default) {
+	    version_map->emplace(object, std::make_pair(shard_id_t::NO_SHARD,
+							version));
+	    added_default = true;
+	  }
+	}
+      }
+      DEBUGDPP("found: {}  {}", pg,
+               object, version);
+      co_return;
+    } else {
+      // if the object does not exist here, it must have been removed
+      // between the collection_list_partial and here.  This can happen
+      // for the first item in the range, which is usually last_backfill.
+      co_return;
+    }
+  }));
+  PrimaryBackfillInterval bi;
+  bi.begin = std::move(start);
+  bi.end = std::move(next);
+  bi.objects = std::move(*version_map);
+  DEBUGDPP("{} PrimaryBackfillInterval filled, leaving, {}",
+           "scan_for_backfill_primary",
+           pg, bi);
+  co_return std::move(bi);
+}
+
+RecoveryBackend::interruptible_future<ReplicaBackfillInterval>
+RecoveryBackend::scan_for_backfill_replica(
+  const hobject_t start,
   [[maybe_unused]] const std::int64_t min,
   const std::int64_t max)
 {
-  logger().debug("{} starting from {}", __func__, start);
-  auto version_map = seastar::make_lw_shared<std::map<hobject_t, eversion_t>>();
-  return backend->list_objects(start, max).then_interruptible(
-    [this, start, version_map] (auto&& ret) {
-    auto&& [objects, next] = std::move(ret);
-    return seastar::do_with(
-      std::move(objects),
-      [this, version_map](auto &objects) {
-      return interruptor::parallel_for_each(objects,
-	[this, version_map] (const hobject_t& object)
-	-> interruptible_future<> {
-	crimson::osd::ObjectContextRef obc;
-	if (pg.is_primary()) {
-	  obc = pg.obc_registry.maybe_get_cached_obc(object);
-	}
-	if (obc) {
-	  if (obc->obs.exists) {
-	    logger().debug("scan_for_backfill found (primary): {}  {}",
-			   object, obc->obs.oi.version);
-	    version_map->emplace(object, obc->obs.oi.version);
-	  } else {
-	    // if the object does not exist here, it must have been removed
-	    // between the collection_list_partial and here.  This can happen
-	    // for the first item in the range, which is usually last_backfill.
-	  }
-	  return seastar::now();
-	} else {
-	  return backend->load_metadata(object).safe_then_interruptible(
-	    [version_map, object] (auto md) {
-	    if (md->os.exists) {
-	      logger().debug("scan_for_backfill found: {}  {}",
-			     object, md->os.oi.version);
-	      version_map->emplace(object, md->os.oi.version);
-	    }
-	    return seastar::now();
-	  }, PGBackend::load_metadata_ertr::assert_all{});
-	}
-      });
-    }).then_interruptible([version_map, start=std::move(start), next=std::move(next), this] {
-      BackfillInterval bi;
-      bi.begin = std::move(start);
-      bi.end = std::move(next);
-      bi.version = pg.get_info().last_update;
-      bi.objects = std::move(*version_map);
-      logger().debug("{} BackfillInterval filled, leaving",
-                     "scan_for_backfill");
-      return seastar::make_ready_future<BackfillInterval>(std::move(bi));
-    });
-  });
+  LOG_PREFIX(RecoveryBackend::scan_for_backfill_replica);
+  DEBUGDPP("starting from {}", pg, start);
+  auto version_map = seastar::make_lw_shared<std::map<hobject_t,
+						      eversion_t>>();
+  auto&& [objects, next] = co_await backend->list_objects(start, max);
+  co_await interruptor::parallel_for_each(objects,
+    seastar::coroutine::lambda([FNAME, this, version_map]
+    (const hobject_t& object) -> interruptible_future<> {
+    DEBUGDPP("querying obj:{}", pg, object);
+    auto obc_manager = pg.obc_loader.get_obc_manager(
+      object, /* resolve_clone = */ false);
+
+    auto found = co_await pg.obc_loader.load_and_lock(
+      obc_manager, RWState::RWREAD
+    ).si_then([] {
+      return true;
+    }).handle_error_interruptible(
+      crimson::ct_error::enoent::handle([](auto) {
+	return false;
+      }),
+      crimson::ct_error::assert_all(fmt::format("{} {} error when loading obc", pg, FNAME).c_str())
+    );
+    if (!found) {
+      // if the object does not exist here, it must have been removed
+      // between the collection_list_partial and here.  This can happen
+      // for the first item in the range, which is usually last_backfill.
+      co_return;
+    }
+
+    if (obc_manager.get_obc()->obs.exists) {
+      auto version = obc_manager.get_obc()->obs.oi.version;
+      version_map->emplace(object, version);
+      DEBUGDPP("found: {}  {}", pg,
+               object, version);
+      co_return;
+    } else {
+      // if the object does not exist here, it must have been removed
+      // between the collection_list_partial and here.  This can happen
+      // for the first item in the range, which is usually last_backfill.
+      co_return;
+    }
+  }));
+  ReplicaBackfillInterval bi;
+  bi.begin = std::move(start);
+  bi.end = std::move(next);
+  bi.objects = std::move(*version_map);
+  DEBUGDPP("{} ReplicaBackfillInterval filled, leaving, {}",
+           "scan_for_backfill_replica",
+           pg, bi);
+  co_return std::move(bi);
 }
 
 RecoveryBackend::interruptible_future<>
@@ -246,7 +361,8 @@ RecoveryBackend::handle_scan_get_digest(
   MOSDPGScan& m,
   crimson::net::ConnectionXcoreRef conn)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_scan_get_digest);
+  DEBUGDPP("", pg);
   if (false /* FIXME: check for backfill too full */) {
     std::ignore = shard_services.start_operation<crimson::osd::LocalPeeringEvent>(
       // TODO: abstract start_background_recovery
@@ -258,7 +374,7 @@ RecoveryBackend::handle_scan_get_digest(
       PeeringState::BackfillTooFull());
     return seastar::now();
   }
-  return scan_for_backfill(
+  return scan_for_backfill_replica(
     std::move(m.begin),
     crimson::common::local_conf().get_val<std::int64_t>("osd_backfill_scan_min"),
     crimson::common::local_conf().get_val<std::int64_t>("osd_backfill_scan_max")
@@ -282,11 +398,12 @@ RecoveryBackend::interruptible_future<>
 RecoveryBackend::handle_scan_digest(
   MOSDPGScan& m)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_scan_digest);
+  DEBUGDPP("", pg);
   // Check that from is in backfill_targets vector
   ceph_assert(pg.is_backfill_target(m.from));
 
-  BackfillInterval bi;
+  ReplicaBackfillInterval bi;
   bi.begin = m.begin;
   bi.end = m.end;
   {
@@ -295,11 +412,10 @@ RecoveryBackend::handle_scan_digest(
     bi.clear_objects();
     ::decode_noclear(bi.objects, p);
   }
-  shard_services.start_operation<crimson::osd::BackfillRecovery>(
-    static_cast<crimson::osd::PG*>(&pg),
-    shard_services,
-    pg.get_osdmap_epoch(),
-    crimson::osd::BackfillState::ReplicaScanned{ m.from, std::move(bi) });
+  auto recovery_handler = pg.get_recovery_handler();
+  recovery_handler->dispatch_backfill_event(
+    crimson::osd::BackfillState::ReplicaScanned{
+      m.from, std::move(bi) }.intrusive_from_this());
   return seastar::now();
 }
 
@@ -308,9 +424,10 @@ RecoveryBackend::handle_scan(
   MOSDPGScan& m,
   crimson::net::ConnectionXcoreRef conn)
 {
-  logger().debug("{}", __func__);
+  LOG_PREFIX(RecoveryBackend::handle_scan);
+  DEBUGDPP("", pg);
   if (pg.old_peering_msg(m.map_epoch, m.query_epoch)) {
-    logger().debug("{}: discarding {}", __func__, m);
+    DEBUGDPP("discarding {}", pg, m);
     return seastar::now();
   }
   switch (m.op) {
@@ -326,7 +443,7 @@ RecoveryBackend::handle_scan(
 }
 
 RecoveryBackend::interruptible_future<>
-RecoveryBackend::handle_recovery_op(
+RecoveryBackend::handle_backfill_op(
   Ref<MOSDFastDispatchOp> m,
   crimson::net::ConnectionXcoreRef conn)
 {
@@ -340,6 +457,6 @@ RecoveryBackend::handle_recovery_op(
   default:
     return seastar::make_exception_future<>(
 	std::invalid_argument(fmt::format("invalid request type: {}",
-					  m->get_header().type)));
+					  (uint16_t)m->get_header().type)));
   }
 }
