@@ -91,29 +91,50 @@ public:
     const std::string &svc_type, const std::string &svc_id);
   PyObject *get_daemon_status_python(
     const std::string &svc_type, const std::string &svc_id);
-  PyObject *get_counter_python(
+  PyObject *get_unlabeled_counter_python(
     const std::string &svc_type,
     const std::string &svc_id,
     const std::string &path);
+  PyObject *get_latest_unlabeled_counter_python(
+      const std::string &svc_type,
+      const std::string &svc_id,
+      const std::string &path);
   PyObject *get_latest_counter_python(
-    const std::string &svc_type,
-    const std::string &svc_id,
-    const std::string &path);
+      const std::string &svc_type,
+      const std::string &svc_id,
+      std::string_view counter_name,
+      std::string_view sub_counter_name,
+      const std::vector<std::pair<std::string_view, std::string_view>> &labels);
+  PyObject *get_unlabeled_perf_schema_python(
+      const std::string &svc_type,
+      const std::string &svc_id);
   PyObject *get_perf_schema_python(
-     const std::string &svc_type,
-     const std::string &svc_id);
+      const std::string &svc_type,
+      const std::string &svc_id);
   PyObject *get_rocksdb_version();
   PyObject *get_context();
   PyObject *get_osdmap();
   /// @note @c fct is not allowed to acquire locks when holding GIL
-  PyObject *with_perf_counters(
+  PyObject *with_unlabled_perf_counters(
       std::function<void(
-        PerfCounterInstance& counter_instance,
-        PerfCounterType& counter_type,
-        PyFormatter& f)> fct,
+	  PerfCounterInstance &counter_instance,
+	  PerfCounterType &counter_type,
+	  PyFormatter &f)> fct,
       const std::string &svc_name,
       const std::string &svc_id,
       const std::string &path) const;
+  /// @note @c fct is not allowed to acquire locks when holding GIL
+  PyObject *with_perf_counters(
+      std::function<void(
+	  PerfCounterInstance &counter_instance,
+	  PerfCounterType &counter_type,
+	  PyFormatter &f)> fct,
+      const std::string &svc_name,
+      const std::string &svc_id,
+      std::string_view counter_name,
+      std::string_view sub_counter_name,
+      const std::vector<std::pair<std::string_view, std::string_view>> &labels)
+      const;
 
   MetricQueryID add_osd_perf_query(
       const OSDPerfMetricQuery &query,
