@@ -4,6 +4,7 @@
 #define CEPH_OS_BLUESTORE_BLUEFS_TYPES_H
 
 #include <optional>
+#include <ostream>
 
 #include "bluestore_types.h"
 #include "include/utime.h"
@@ -33,6 +34,14 @@ public:
 };
 WRITE_CLASS_DENC(bluefs_extent_t)
 
+enum bluefs_node_encoding {
+  PLAIN = 0,        ///< Normal; legacy mode.
+  ENVELOPE = 1,     ///< Data flushed to file is wrapped in envelope - no size update needed.
+                    ///  Without shutdown, range [fnode.size ... fnode.allocated) may contain envelopes.
+  ENVELOPE_FIN = 2, ///< Same as envelope but file orderly closed. Fnode.size reflects actual end.
+  ENCODING_MAX = 3
+};
+
 std::ostream& operator<<(std::ostream& out, const bluefs_extent_t& e);
 
 struct bluefs_locked_extents_t {
@@ -73,16 +82,54 @@ struct bluefs_fnode_delta_t {
   uint64_t offset; // Contains offset in file of extents.
                    // Equal to 'allocated' when created.
                    // Used for consistency checking.
+
+  uint8_t encoding = PLAIN;
+  // For envelope mode only.
+  uint64_t content_size; // The size of payload in the file; size = wal_size + n * envelope_size
+
   mempool::bluefs::vector<bluefs_extent_t> extents;
 
-  DENC(bluefs_fnode_delta_t, v, p) {
-    DENC_START(1, 1, p);
+  DENC_HELPERS
+
+  void bound_encode(size_t& p) const {
+    uint8_t version = 1, compat = 1;
+    if (encoding == ENVELOPE  || encoding == ENVELOPE_FIN) {
+      version = 2;
+      compat = 2;
+    }
+    DENC_START_UNSAFE(version, compat, p);
+    _denc_friend(*this, p, version);
+    DENC_FINISH(p);
+  }
+  void encode(ceph::buffer::list::contiguous_appender& p) const {
+    DENC_DUMP_PRE(bluefs_fnode_t);
+    uint8_t version = 1, compat = 1;
+    if (encoding == ENVELOPE  || encoding == ENVELOPE_FIN) {
+      version = 2;
+      compat = 2;
+    }
+    DENC_START_UNSAFE(version, compat, p);
+    _denc_friend(*this, p, version);
+    DENC_FINISH(p);
+  }
+  void decode(ceph::buffer::ptr::const_iterator& p) {
+    DENC_START_UNSAFE(2, !"value unused in decode", p);
+    _denc_friend(*this, p, struct_v);
+    DENC_FINISH(p);
+  }
+
+  template<typename T, typename P>
+  friend std::enable_if_t<std::is_same_v<bluefs_fnode_delta_t, std::remove_const_t<T>>>
+  _denc_friend(T& v, P& p, __u8& struct_v) {
     denc_varint(v.ino, p);
     denc_varint(v.size, p);
     denc(v.mtime, p);
     denc(v.offset, p);
     denc(v.extents, p);
-    DENC_FINISH(p);
+    if (struct_v >= 2) {
+      denc_varint(v.encoding, p);
+      denc_varint(v.content_size, p);
+    }
   }
 };
 WRITE_CLASS_DENC(bluefs_fnode_delta_t)
@@ -102,14 +149,19 @@ struct bluefs_fnode_t {
 
   uint64_t allocated;
   uint64_t allocated_commited;
-
-  bluefs_fnode_t() : ino(0), size(0), allocated(0), allocated_commited(0) {}
+  uint8_t encoding = PLAIN;
+  // envelope mode only
+  uint64_t content_size; ///< Payload bytes inside envelopes.
+                         ///  When encoding == ENVELOPE indexing might update the value.
+  bluefs_fnode_t() : ino(0), size(0), allocated(0), allocated_commited(0), content_size(0) {}
   bluefs_fnode_t(uint64_t _ino, uint64_t _size, utime_t _mtime) :
-    ino(_ino), size(_size), mtime(_mtime), allocated(0), allocated_commited(0) {}
+    ino(_ino), size(_size), mtime(_mtime), allocated(0), allocated_commited(0), content_size(0) {}
   bluefs_fnode_t(const bluefs_fnode_t& other) :
     ino(other.ino), size(other.size), mtime(other.mtime),
     allocated(other.allocated),
-    allocated_commited(other.allocated_commited) {
+    allocated_commited(other.allocated_commited),
+    encoding(other.encoding),
+    content_size(other.content_size) {
     clone_extents(other);
   }
 
@@ -130,26 +182,46 @@ struct bluefs_fnode_t {
 
   DENC_HELPERS
   void bound_encode(size_t& p) const {
-    _denc_friend(*this, p);
+    uint8_t version = 1, compat = 1;
+    if (encoding == ENVELOPE || encoding == ENVELOPE_FIN) {
+      version = 2;
+      compat = 2;
+    }
+    DENC_START_UNSAFE(version, compat, p);
+    _denc_friend(*this, p, version);
+    DENC_FINISH(p);
   }
   void encode(ceph::buffer::list::contiguous_appender& p) const {
     DENC_DUMP_PRE(bluefs_fnode_t);
-    _denc_friend(*this, p);
+    uint8_t version = 1, compat = 1;
+    if (encoding == ENVELOPE || encoding == ENVELOPE_FIN) {
+      version = 2;
+      compat = 2;
+    }
+    DENC_START_UNSAFE(version, compat, p);
+    _denc_friend(*this, p, version);
+    DENC_FINISH(p);
   }
+
   void decode(ceph::buffer::ptr::const_iterator& p) {
-    _denc_friend(*this, p);
+    DENC_START_UNSAFE(2, !"value unused in decode", p);
+    _denc_friend(*this, p, struct_v);
+    DENC_FINISH(p);
     recalc_allocated();
   }
+
   template<typename T, typename P>
   friend std::enable_if_t<std::is_same_v<bluefs_fnode_t, std::remove_const_t<T>>>
-  _denc_friend(T& v, P& p) {
-    DENC_START(1, 1, p);
+  _denc_friend(T& v, P& p, __u8& struct_v) {
     denc_varint(v.ino, p);
     denc_varint(v.size, p);
     denc(v.mtime, p);
     denc(v.__unused__, p);
     denc(v.extents, p);
-    DENC_FINISH(p);
+    if (struct_v >= 2) {
+      denc_varint(v.encoding, p);
+      denc_varint(v.content_size, p);
+    }
   }
   void reset_delta() {
     allocated_commited = allocated;
@@ -242,6 +314,11 @@ struct bluefs_layout_t {
 WRITE_CLASS_ENCODER(bluefs_layout_t)
 
 struct bluefs_super_t {
+  static constexpr uint8_t BASELINE = 2;
+  static constexpr uint8_t ENVELOPE_MODE_ENABLED = 3;
+
+  uint8_t _version = BASELINE; ///< usually we hide encoding version,
+                               ///  but we need to tie features to it
   uuid_d uuid;      ///< unique to this bluefs instance
   uuid_d osd_uuid;  ///< matches the osd that owns us
   uint64_t seq;     ///< sequence counter
@@ -287,7 +364,6 @@ struct bluefs_transaction_t {
   uuid_d uuid;          ///< fs uuid
   uint64_t seq;         ///< sequence number
   ceph::buffer::list op_bl;     ///< encoded transaction ops
-
   bluefs_transaction_t() : seq(0) {}
 
   void clear() {
@@ -369,4 +445,5 @@ struct bluefs_transaction_t {
 WRITE_CLASS_ENCODER(bluefs_transaction_t)
 
 std::ostream& operator<<(std::ostream& out, const bluefs_transaction_t& t);
+
 #endif
