@@ -16,13 +16,14 @@
 #define SNAPMAPPER_H
 
 #include <cstring>
+#include <map>
 #include <set>
 #include <string>
 #include <utility>
 
 #include "common/hobject.h"
 #include "common/map_cacher.hpp"
-#ifdef WITH_SEASTAR
+#ifdef WITH_CRIMSON
 #  include "crimson/os/futurized_store.h"
 #  include "crimson/os/futurized_collection.h"
 #endif
@@ -34,7 +35,7 @@
 #include "osd/SnapMapReaderI.h"
 
 class OSDriver : public MapCacher::StoreDriver<std::string, ceph::buffer::list> {
-#ifdef WITH_SEASTAR
+#ifdef WITH_CRIMSON
   using ObjectStoreT = crimson::os::FuturizedStore::Shard;
   using CollectionHandleT = ObjectStoreT::CollectionRef;
 #else
@@ -77,7 +78,7 @@ public:
     return OSTransaction(ch->get_cid(), hoid, t);
   }
 
-#ifndef WITH_SEASTAR
+#ifndef WITH_CRIMSON
   OSDriver(ObjectStoreT *os, const coll_t& cid, const ghobject_t &hoid) :
     OSDriver(os, os->open_collection(cid), hoid) {}
 #endif
@@ -122,6 +123,8 @@ public:
 class SnapMapper : public Scrub::SnapMapReaderI {
   friend class MapperVerifier; // unit-test support
   friend class DirectMapper; // unit-test support
+  friend std::ostream& operator<<(std::ostream &lhs, const SnapMapper &sm);
+
 public:
   CephContext* cct;
   struct object_snaps {
@@ -172,7 +175,7 @@ public:
   static const char *PURGED_SNAP_EPOCH_PREFIX;
   static const char *PURGED_SNAP_PREFIX;
 
-#ifndef WITH_SEASTAR
+#ifndef WITH_CRIMSON
   struct Scrubber {
     CephContext *cct;
     ObjectStore *store;
@@ -180,17 +183,15 @@ public:
     ghobject_t mapping_hoid;
     ghobject_t purged_snaps_hoid;
 
-    ObjectMap::ObjectMapIterator psit;
     int64_t pool;
     snapid_t begin, end;
 
-    bool _parse_p();   ///< advance the purged_snaps pointer
+    bool _parse_p(std::string_view key, std::string_view value);
 
-    ObjectMap::ObjectMapIterator mapit;
     Mapping mapping;
     shard_id_t shard;
 
-    bool _parse_m();   ///< advance the (object) mapper pointer
+    bool _parse_m(std::string_view key, std::string_view value);
 
     std::vector<std::tuple<int64_t, snapid_t, uint32_t, shard_id_t>> stray;
 
@@ -208,17 +209,6 @@ public:
 
     void run();
   };
-
-  static std::string convert_legacy_key(
-    const std::string& old_key,
-    const bufferlist& value);
-
-  static int convert_legacy(
-    CephContext *cct,
-    ObjectStore *store,
-    ObjectStore::CollectionHandle& ch,
-    ghobject_t hoid,
-    unsigned max);
 #endif
 
   static void record_purged_snaps(
@@ -242,11 +232,6 @@ private:
   // functions.
   mutable MapCacher::MapCacher<std::string, ceph::buffer::list> backend;
 
-  static std::string get_legacy_prefix(snapid_t snap);
-  std::string to_legacy_raw_key(
-    const std::pair<snapid_t, hobject_t> &to_map);
-  static bool is_legacy_mapping(const std::string &to_test);
-
   static std::string get_prefix(int64_t pool, snapid_t snap);
   std::string to_raw_key(
     const std::pair<snapid_t, hobject_t> &to_map) const;
@@ -291,11 +276,10 @@ private:
   tl::expected<object_snaps, SnapMapReaderI::result_t> get_snaps_common(
     const hobject_t &hoid) const;
 
-  /// file @out vector with the first objects with @snap as a snap
-  void get_objects_by_prefixes(
+  /// \returns vector with the first objects with @snap as a snap
+  std::vector<hobject_t> get_objects_by_prefixes(
     snapid_t snap,
-    unsigned max,
-    std::vector<hobject_t> *out);
+    unsigned max);
 
   std::set<std::string>           prefixes;
   // maintain a current active prefix
@@ -330,28 +314,12 @@ private:
     uint32_t bits,   ///< [in] current split bits
     int64_t pool,    ///< [in] pool
     shard_id_t shard ///< [in] shard
-    )
-    : cct(cct), backend(driver), mask_bits(bits), match(match), pool(pool),
-      shard(shard), shard_prefix(make_shard_prefix(shard)) {
-    update_bits(mask_bits);
-  }
+    );
 
   /// Update bits in case of pg split or merge
   void update_bits(
     uint32_t new_bits  ///< [in] new split bits
-    ) {
-    mask_bits = new_bits;
-    std::set<std::string> _prefixes = hobject_t::get_prefixes(
-      mask_bits,
-      match,
-      pool);
-    prefixes.clear();
-    for (auto i = _prefixes.begin(); i != _prefixes.end(); ++i) {
-      prefixes.insert(shard_prefix + *i);
-    }
-
-    reset_prefix_itr(CEPH_NOSNAP, "update_bits");
-  }
+    );
 
   const std::set<std::string>::iterator get_prefix_itr() {
     return prefix_itr;
@@ -378,11 +346,10 @@ private:
     );
 
   /// Returns first object with snap as a snap
-  int get_next_objects_to_trim(
+  std::optional<std::vector<hobject_t>> get_next_objects_to_trim(
     snapid_t snap,              ///< [in] snap to check
-    unsigned max,               ///< [in] max to get
-    std::vector<hobject_t> *out      ///< [out] next objects to trim (must be empty)
-    );  ///< @return error, -ENOENT if no more objects
+    unsigned max                ///< [in] max to get
+    );  ///< @return nullopt if no more objects
 
   /// Remove mapping for oid
   int remove_oid(
@@ -396,6 +363,10 @@ private:
     std::set<snapid_t> *snaps ///< [out] snaps
     ) const; ///< @return error, -ENOENT if oid is not recorded
 
+  void update_snap_map(
+    const pg_log_entry_t& i,
+    MapCacher::Transaction<std::string, ceph::buffer::list> *t);
+
   /// Get snaps for oid - alternative interface
   tl::expected<std::set<snapid_t>, SnapMapReaderI::result_t> get_snaps(
     const hobject_t &hoid) const final;
@@ -412,4 +383,14 @@ private:
 WRITE_CLASS_ENCODER(SnapMapper::object_snaps)
 WRITE_CLASS_ENCODER(SnapMapper::Mapping)
 
+inline std::ostream& operator<<(std::ostream& os, const SnapMapper& sm)
+{
+  os << fmt::format(" [pg_id:{:x}, match:{}, mask_bits:{}, "
+                    "last_key_checked:{}, pool:{}, shard:{}, "
+                    "shard_prefix: {}, prefixes: {}] ",
+                    sm.match, sm.match, sm.mask_bits, sm.last_key_checked,
+                    sm.pool, sm.shard.id, sm.shard_prefix, sm.prefixes);
+  return os;
+}
+
 #endif
