@@ -10,7 +10,11 @@ describe('Navigation accessibility', { retries: 0 }, () => {
 
   it('top-nav should have no accessibility violations', () => {
     cy.injectAxe();
-    cy.checkAccessibility('.cd-navbar-top');
+    cy.checkAccessibility('cds-header', {
+      rules: {
+        'nested-interactive': { enabled: false }
+      }
+    });
   });
 
   it('sidebar should have no accessibility violations', () => {
