@@ -22,17 +22,27 @@ And('I should see a button to {string}', (button: string) => {
 });
 
 When('I click on {string} button', (button: string) => {
-  cy.get(`[aria-label="${button}"]`).first().click();
+  cy.get(`[aria-label="${button}"]`).first().click({ force: true });
 });
 
 Then('I should see the modal', () => {
   cy.get('cd-modal').should('exist');
 });
 
+// @TODO: Replace with the existing (above one)
+// once carbon migration is completed
+Then('I should see the carbon modal', () => {
+  cy.get('cds-modal').should('exist');
+});
+
 Then('I should not see the modal', () => {
   cy.get('cd-modal').should('not.exist');
 });
 
+Then('I should not see the carbon modal', () => {
+  cy.get('cds-modal').should('not.exist');
+});
+
 And('I go to the {string} tab', (names: string) => {
   for (const name of names.split(', ')) {
     cy.contains('.nav.nav-tabs a', name).click();
