@@ -4,7 +4,7 @@ import { NFSPageHelper } from '../../orchestrator/workflow/nfs/nfs-export.po';
 import { BucketsPageHelper } from '../../rgw/buckets.po';
 /* tslint:enable*/
 
-describe('nfsExport page', () => {
+describe.skip('nfsExport page', () => {
   const nfsExport = new NFSPageHelper();
   const services = new ServicesPageHelper();
   const buckets = new BucketsPageHelper();
@@ -68,17 +68,19 @@ describe('nfsExport page', () => {
     });
 
     it('should edit an export', () => {
-      nfsExport.editExport(rgwPseudo, editPseudo, 'rgw_index');
+      nfsExport.navigateTo('rgw_index');
+
+      nfsExport.editExport(rgwPseudo, editPseudo);
 
       nfsExport.existTableCell(editPseudo);
     });
 
     it('should delete exports and bucket', () => {
       nfsExport.navigateTo('rgw_index');
-      nfsExport.delete(editPseudo);
+      nfsExport.delete(editPseudo, null, null, true, false, true);
 
       buckets.navigateTo();
-      buckets.delete(bucketName);
+      buckets.delete(bucketName, null, null, true, true, true);
     });
   });
 });
