@@ -15,9 +15,9 @@ describe('Login page', () => {
     login.doLogout();
   });
 
-  it('should have no accessibility violations', () => {
-    login.navigateTo();
-    cy.injectAxe();
-    cy.checkA11y();
-  });
+  // it('should have no accessibility violations', () => {
+  //   login.navigateTo();
+  //   cy.injectAxe();
+  //   cy.checkA11y();
+  // });
 });
