@@ -1,8 +1,9 @@
+import _ from 'lodash';
 import { Component, OnInit } from '@angular/core';
 import { UntypedFormControl, Validators } from '@angular/forms';
 import { ActivatedRoute, Router } from '@angular/router';
 import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
-import { ListenerRequest, NvmeofService } from '~/app/shared/api/nvmeof.service';
+import { GatewayGroup, ListenerRequest, NvmeofService } from '~/app/shared/api/nvmeof.service';
 import { ActionLabelsI18n, URLVerbs } from '~/app/shared/constants/app.constants';
 import { CdFormGroup } from '~/app/shared/forms/cd-form-group';
 import { FinishedTask } from '~/app/shared/models/finished-task';
@@ -13,9 +14,9 @@ import { FormatterService } from '~/app/shared/services/formatter.service';
 import { CdValidators } from '~/app/shared/forms/cd-validators';
 import { DimlessBinaryPipe } from '~/app/shared/pipes/dimless-binary.pipe';
 import { HostService } from '~/app/shared/api/host.service';
-import { DaemonService } from '~/app/shared/api/daemon.service';
 import { map } from 'rxjs/operators';
 import { forkJoin } from 'rxjs';
+import { Host } from '~/app/shared/models/host.interface';
 
 @Component({
   selector: 'cd-nvmeof-listeners-form',
@@ -31,6 +32,7 @@ export class NvmeofListenersFormComponent implements OnInit {
   listenerForm: CdFormGroup;
   subsystemNQN: string;
   hosts: Array<object> = null;
+  group: string;
 
   constructor(
     public actionLabels: ActionLabelsI18n,
@@ -42,8 +44,7 @@ export class NvmeofListenersFormComponent implements OnInit {
     private route: ActivatedRoute,
     public activeModal: NgbActiveModal,
     public formatterService: FormatterService,
-    public dimlessBinaryPipe: DimlessBinaryPipe,
-    private daemonService: DaemonService
+    public dimlessBinaryPipe: DimlessBinaryPipe
   ) {
     this.permission = this.authStorageService.getPermissions().nvmeof;
     this.hostPermission = this.authStorageService.getPermissions().hosts;
@@ -51,19 +52,42 @@ export class NvmeofListenersFormComponent implements OnInit {
     this.pageURL = 'block/nvmeof/subsystems';
   }
 
+  filterHostsByLabel(allHosts: Host[], gwNodesLabel: string | string[]) {
+    return allHosts.filter((host: Host) => {
+      const hostLabels: string[] = host?.labels;
+      if (typeof gwNodesLabel === 'string') {
+        return hostLabels.includes(gwNodesLabel);
+      }
+      return hostLabels?.length === gwNodesLabel?.length && _.isEqual(hostLabels, gwNodesLabel);
+    });
+  }
+
+  filterHostsByHostname(allHosts: Host[], gwNodes: string[]) {
+    return allHosts.filter((host: Host) => gwNodes.includes(host.hostname));
+  }
+
+  getGwGroupPlacement(gwGroups: GatewayGroup[][]) {
+    return (
+      gwGroups?.[0]?.find((gwGroup: GatewayGroup) => gwGroup?.spec?.group === this.group)
+        ?.placement || { hosts: [], label: [] }
+    );
+  }
+
   setHosts() {
     forkJoin({
-      daemons: this.daemonService.list(['nvmeof']),
-      hosts: this.hostService.getAllHosts()
+      gwGroups: this.nvmeofService.listGatewayGroups(),
+      allHosts: this.hostService.getAllHosts()
     })
       .pipe(
-        map(({ daemons, hosts }) => {
-          const hostNamesFromDaemon = daemons.map((daemon: any) => daemon.hostname);
-          return hosts.filter((host: any) => hostNamesFromDaemon.includes(host.hostname));
+        map(({ gwGroups, allHosts }) => {
+          const { hosts, label } = this.getGwGroupPlacement(gwGroups);
+          if (hosts?.length) return this.filterHostsByHostname(allHosts, hosts);
+          else if (label?.length) return this.filterHostsByLabel(allHosts, label);
+          return [];
         })
       )
-      .subscribe((nvmeofHosts: any[]) => {
-        this.hosts = nvmeofHosts.map((h) => ({ hostname: h.hostname, addr: h.addr }));
+      .subscribe((nvmeofGwNodes: Host[]) => {
+        this.hosts = nvmeofGwNodes.map((h) => ({ hostname: h.hostname, addr: h.addr }));
       });
   }
 
@@ -71,7 +95,10 @@ export class NvmeofListenersFormComponent implements OnInit {
     this.createForm();
     this.action = this.actionLabels.CREATE;
     this.route.params.subscribe((params: { subsystem_nqn: string }) => {
-      this.subsystemNQN = params.subsystem_nqn;
+      this.subsystemNQN = params?.subsystem_nqn;
+    });
+    this.route.queryParams.subscribe((params) => {
+      this.group = params?.['group'];
     });
     this.setHosts();
   }
@@ -93,7 +120,8 @@ export class NvmeofListenersFormComponent implements OnInit {
     const host = this.listenerForm.getValue('host');
     let trsvcid = Number(this.listenerForm.getValue('trsvcid'));
     if (!trsvcid) trsvcid = 4420;
-    const request = {
+    const request: ListenerRequest = {
+      gw_group: this.group,
       host_name: host.hostname,
       traddr: host.addr,
       trsvcid
