@@ -9,7 +9,7 @@ import { Permission } from '~/app/shared/models/permissions';
 import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
 import { TaskWrapperService } from '~/app/shared/services/task-wrapper.service';
 import { FinishedTask } from '~/app/shared/models/finished-task';
-import { Router } from '@angular/router';
+import { ActivatedRoute, Router } from '@angular/router';
 import { MAX_NAMESPACE, NvmeofService } from '~/app/shared/api/nvmeof.service';
 
 @Component({
@@ -24,6 +24,7 @@ export class NvmeofSubsystemsFormComponent implements OnInit {
   resource: string;
   pageURL: string;
   defaultMaxNamespace: number = MAX_NAMESPACE;
+  group: string;
 
   constructor(
     private authStorageService: AuthStorageService,
@@ -31,7 +32,8 @@ export class NvmeofSubsystemsFormComponent implements OnInit {
     public activeModal: NgbActiveModal,
     private nvmeofService: NvmeofService,
     private taskWrapperService: TaskWrapperService,
-    private router: Router
+    private router: Router,
+    private route: ActivatedRoute
   ) {
     this.permission = this.authStorageService.getPermissions().nvmeof;
     this.resource = $localize`Subsystem`;
@@ -49,6 +51,9 @@ export class NvmeofSubsystemsFormComponent implements OnInit {
   );
 
   ngOnInit() {
+    this.route.queryParams.subscribe((params) => {
+      this.group = params?.['group'];
+    });
     this.createForm();
     this.action = this.actionLabels.CREATE;
   }
@@ -66,7 +71,13 @@ export class NvmeofSubsystemsFormComponent implements OnInit {
           )
         ],
         asyncValidators: [
-          CdValidators.unique(this.nvmeofService.isSubsystemPresent, this.nvmeofService)
+          CdValidators.unique(
+            this.nvmeofService.isSubsystemPresent,
+            this.nvmeofService,
+            null,
+            null,
+            this.group
+          )
         ]
       }),
       max_namespaces: new UntypedFormControl(this.defaultMaxNamespace, {
@@ -87,8 +98,9 @@ export class NvmeofSubsystemsFormComponent implements OnInit {
 
     const request = {
       nqn,
-      max_namespaces,
-      enable_ha: true
+      enable_ha: true,
+      gw_group: this.group,
+      max_namespaces
     };
 
     if (!max_namespaces) {
