@@ -1,11 +1,13 @@
 import { ComponentFixture, TestBed } from '@angular/core/testing';
 
 import { CephfsAuthModalComponent } from './cephfs-auth-modal.component';
-import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
+import { NgbTypeaheadModule } from '@ng-bootstrap/ng-bootstrap';
 import { HttpClientTestingModule } from '@angular/common/http/testing';
 import { ToastrModule } from 'ngx-toastr';
 import { SharedModule } from '~/app/shared/shared.module';
 import { ReactiveFormsModule } from '@angular/forms';
+import { RouterTestingModule } from '@angular/router/testing';
+import { CheckboxModule, InputModule, ModalModule } from 'carbon-components-angular';
 
 describe('CephfsAuthModalComponent', () => {
   let component: CephfsAuthModalComponent;
@@ -14,8 +16,17 @@ describe('CephfsAuthModalComponent', () => {
   beforeEach(async () => {
     await TestBed.configureTestingModule({
       declarations: [CephfsAuthModalComponent],
-      imports: [HttpClientTestingModule, SharedModule, ReactiveFormsModule, ToastrModule.forRoot()],
-      providers: [NgbActiveModal]
+      imports: [
+        HttpClientTestingModule,
+        SharedModule,
+        ReactiveFormsModule,
+        ToastrModule.forRoot(),
+        RouterTestingModule,
+        NgbTypeaheadModule,
+        ModalModule,
+        InputModule,
+        CheckboxModule
+      ]
     }).compileComponents();
 
     fixture = TestBed.createComponent(CephfsAuthModalComponent);
