@@ -1,19 +1,21 @@
-import { Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
+import { Component, OnInit, ViewChild } from '@angular/core';
 import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
+import { BaseModal } from 'carbon-components-angular';
 
 @Component({
   selector: 'cd-cephfs-mount-details',
   templateUrl: './cephfs-mount-details.component.html',
   styleUrls: ['./cephfs-mount-details.component.scss']
 })
-export class CephfsMountDetailsComponent implements OnInit, OnDestroy {
+export class CephfsMountDetailsComponent extends BaseModal implements OnInit {
   @ViewChild('mountDetailsTpl', { static: true })
   mountDetailsTpl: any;
   onCancel?: Function;
-  private canceled = false;
   private MOUNT_DIRECTORY = '<MOUNT_DIRECTORY>';
   mountData!: Record<string, any>;
-  constructor(public activeModal: NgbActiveModal) {}
+  constructor(public activeModal: NgbActiveModal) {
+    super();
+  }
   mount!: string;
   fuse!: string;
   nfs!: string;
@@ -24,14 +26,7 @@ export class CephfsMountDetailsComponent implements OnInit, OnDestroy {
     this.nfs = `sudo mount -t nfs -o port=<PORT> <IP of active_nfs daemon>:<export_name> ${this.MOUNT_DIRECTORY}`;
   }
 
-  ngOnDestroy(): void {
-    if (this.onCancel && this.canceled) {
-      this.onCancel();
-    }
-  }
-
   cancel() {
-    this.canceled = true;
-    this.activeModal.close();
+    this.closeModal();
   }
 }
