@@ -1,14 +1,13 @@
 import { HttpClientTestingModule } from '@angular/common/http/testing';
 import { DebugElement } from '@angular/core';
 import { ComponentFixture, TestBed } from '@angular/core/testing';
-import { TreeModule } from '@circlon/angular-tree-component';
 import { ToastrModule } from 'ngx-toastr';
 import { SharedModule } from '~/app/shared/shared.module';
 
 import { RgwMultisiteDetailsComponent } from './rgw-multisite-details.component';
 import { RouterTestingModule } from '@angular/router/testing';
 import { configureTestBed } from '~/testing/unit-test-helper';
-import { NgbNavModule } from '@ng-bootstrap/ng-bootstrap';
+import { NgbNavModule, NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
 
 describe('RgwMultisiteDetailsComponent', () => {
   let component: RgwMultisiteDetailsComponent;
@@ -19,12 +18,12 @@ describe('RgwMultisiteDetailsComponent', () => {
     declarations: [RgwMultisiteDetailsComponent],
     imports: [
       HttpClientTestingModule,
-      TreeModule,
       SharedModule,
       ToastrModule.forRoot(),
       RouterTestingModule,
       NgbNavModule
-    ]
+    ],
+    providers: [NgbActiveModal]
   });
 
   beforeEach(() => {
@@ -40,6 +39,6 @@ describe('RgwMultisiteDetailsComponent', () => {
 
   it('should display right title', () => {
     const span = debugElement.nativeElement.querySelector('.card-header');
-    expect(span.textContent).toBe('Topology Viewer');
+    expect(span.textContent.trim()).toBe('Topology Viewer');
   });
 });
