@@ -3,15 +3,14 @@ import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { ReactiveFormsModule } from '@angular/forms';
 import { RouterTestingModule } from '@angular/router/testing';
 
-import { NgbActiveModal, NgbModalModule, NgbModalRef } from '@ng-bootstrap/ng-bootstrap';
-
-import { ModalService } from '~/app/shared/services/modal.service';
 import { configureTestBed, FixtureHelper } from '~/testing/unit-test-helper';
 import { BackButtonComponent } from '../back-button/back-button.component';
 import { FormButtonPanelComponent } from '../form-button-panel/form-button-panel.component';
 import { ModalComponent } from '../modal/modal.component';
 import { SubmitButtonComponent } from '../submit-button/submit-button.component';
 import { ConfirmationModalComponent } from './confirmation-modal.component';
+import { ModalCdsService } from '../../services/modal-cds.service';
+import { ModalService, PlaceholderService } from 'carbon-components-angular';
 
 @NgModule({})
 export class MockModule {}
@@ -22,11 +21,11 @@ export class MockModule {}
 class MockComponent {
   @ViewChild('fillTpl', { static: true })
   fillTpl: TemplateRef<any>;
-  modalRef: NgbModalRef;
+  modalRef: any;
   returnValue: any;
 
   // Normally private, but public is needed by tests
-  constructor(public modalService: ModalService) {}
+  constructor(public modalService: ModalCdsService) {}
 
   private openModal(extendBaseState = {}) {
     this.modalRef = this.modalService.show(
@@ -63,6 +62,7 @@ describe('ConfirmationModalComponent', () => {
   let mockComponent: MockComponent;
   let mockFixture: ComponentFixture<MockComponent>;
   let fh: FixtureHelper;
+  let modalService: ModalCdsService;
 
   const expectReturnValue = (v: string) => expect(mockComponent.returnValue).toBe(v);
 
@@ -76,8 +76,25 @@ describe('ConfirmationModalComponent', () => {
       FormButtonPanelComponent
     ],
     schemas: [NO_ERRORS_SCHEMA],
-    imports: [ReactiveFormsModule, MockModule, RouterTestingModule, NgbModalModule],
-    providers: [NgbActiveModal, SubmitButtonComponent, FormButtonPanelComponent]
+    imports: [ReactiveFormsModule, MockModule, RouterTestingModule],
+    providers: [
+      SubmitButtonComponent,
+      FormButtonPanelComponent,
+      ModalService,
+      PlaceholderService,
+      {
+        provide: 'titleText',
+        useValue: 'test-title'
+      },
+      {
+        provide: 'buttonText',
+        useValue: 'test-button'
+      },
+      {
+        provide: 'onSubmit',
+        useValue: () => {}
+      }
+    ]
   });
 
   beforeEach(() => {
@@ -85,13 +102,13 @@ describe('ConfirmationModalComponent', () => {
     mockFixture = TestBed.createComponent(MockComponent);
     mockComponent = mockFixture.componentInstance;
     mockFixture.detectChanges();
+    modalService = TestBed.inject(ModalCdsService);
 
-    spyOn(TestBed.inject(ModalService), 'show').and.callFake((_modalComp, config) => {
+    spyOn(TestBed.inject(ModalCdsService), 'show').and.callFake((_modalComp, config) => {
       fixture = TestBed.createComponent(ConfirmationModalComponent);
       component = fixture.componentInstance;
       component = Object.assign(component, config);
-      component.activeModal = { close: () => true } as any;
-      spyOn(component.activeModal, 'close').and.callThrough();
+      spyOn(modalService, 'dismissAll').and.callThrough();
       fh.updateFixture(fixture);
     });
   });
@@ -153,7 +170,7 @@ describe('ConfirmationModalComponent', () => {
     });
 
     it('should show the correct title', () => {
-      expect(fh.getText('.modal-title')).toBe('Title is a must have');
+      expect(fh.getText('cds-modal-header h3')).toBe('Title is a must have');
     });
 
     it('should show the correct action name', () => {
@@ -165,21 +182,19 @@ describe('ConfirmationModalComponent', () => {
       spyOn(fh.getElementByCss('.tc_submitButton').componentInstance, 'focusButton');
       fh.clickElement('.tc_submitButton');
       expect(component.onSubmit).toHaveBeenCalledTimes(1);
-      expect(component.activeModal.close).toHaveBeenCalledTimes(0);
+      expect(modalService.dismissAll).toHaveBeenCalledTimes(0);
       expectReturnValue('The submit action has to hide manually.');
     });
 
     it('should use the default cancel action', () => {
       fh.clickElement('.tc_backButton');
       expect(component.onSubmit).toHaveBeenCalledTimes(0);
-      expect(component.activeModal.close).toHaveBeenCalledTimes(1);
+      expect(modalService.dismissAll).toHaveBeenCalledTimes(1);
       expectReturnValue(undefined);
     });
 
     it('should show the description', () => {
-      expect(fh.getText('.modal-body')).toBe(
-        'Template based description. String based description.'
-      );
+      expect(fh.getText('section')).toBe('Template based description. String based description.');
     });
   });
 });
