@@ -2,12 +2,18 @@ import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { ReactiveFormsModule, Validators } from '@angular/forms';
 import { RouterTestingModule } from '@angular/router/testing';
 
-import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
-
 import { CdValidators } from '~/app/shared/forms/cd-validators';
 import { SharedModule } from '~/app/shared/shared.module';
 import { configureTestBed, FixtureHelper, FormHelper } from '~/testing/unit-test-helper';
 import { FormModalComponent } from './form-modal.component';
+import {
+  CheckboxModule,
+  ComboBoxModule,
+  InputModule,
+  ModalModule,
+  NumberModule,
+  SelectModule
+} from 'carbon-components-angular';
 
 describe('InputModalComponent', () => {
   let component: FormModalComponent;
@@ -46,8 +52,17 @@ describe('InputModalComponent', () => {
   };
 
   configureTestBed({
-    imports: [RouterTestingModule, ReactiveFormsModule, SharedModule],
-    providers: [NgbActiveModal]
+    imports: [
+      RouterTestingModule,
+      ReactiveFormsModule,
+      SharedModule,
+      InputModule,
+      CheckboxModule,
+      SelectModule,
+      ComboBoxModule,
+      NumberModule,
+      ModalModule
+    ]
   });
 
   beforeEach(() => {
@@ -64,11 +79,11 @@ describe('InputModalComponent', () => {
   });
 
   it('has the defined title', () => {
-    fh.expectTextToBe('.modal-title', 'Some title');
+    fh.expectTextToBe('.cds--modal-header__heading', 'Some title');
   });
 
   it('has the defined description', () => {
-    fh.expectTextToBe('.modal-body > p', 'Some description');
+    fh.expectTextToBe('[id=description]', 'Some description');
   });
 
   it('should display both inputs', () => {
@@ -77,7 +92,7 @@ describe('InputModalComponent', () => {
   });
 
   it('has one defined label field', () => {
-    fh.expectTextToBe('.cd-col-form-label', 'Optional');
+    fh.expectTextToBe('cds-number .cds--label', 'Optional');
   });
 
   it('has a predefined values for requiredField', () => {
@@ -99,7 +114,7 @@ describe('InputModalComponent', () => {
 
   it('tests required field message', () => {
     formHelper.setValue('requiredField', '', true);
-    fh.expectTextToBe('.cd-requiredField-form-group .invalid-feedback', 'This field is required.');
+    fh.expectTextToBe('.cds--form-requirement', 'This field is required.');
   });
 
   it('tests custom validator on number field', () => {
@@ -109,28 +124,19 @@ describe('InputModalComponent', () => {
 
   it('tests custom validator error message', () => {
     formHelper.setValue('optionalField', -1, true);
-    fh.expectTextToBe(
-      '.cd-optionalField-form-group .invalid-feedback',
-      'Value has to be above zero!'
-    );
+    fh.expectTextToBe('.cds--form-requirement', 'Value has to be above zero!');
   });
 
   it('tests default error message', () => {
     formHelper.setValue('optionalField', 11, true);
-    fh.expectTextToBe('.cd-optionalField-form-group .invalid-feedback', 'An error occurred.');
+    fh.expectTextToBe('.cds--form-requirement', 'An error occurred.');
   });
 
   it('tests binary error messages', () => {
     formHelper.setValue('dimlessBinary', '4 K', true);
-    fh.expectTextToBe(
-      '.cd-dimlessBinary-form-group .invalid-feedback',
-      'Size has to be at most 3 KiB or less'
-    );
+    fh.expectTextToBe('.cds--form-requirement', 'Size has to be at most 3 KiB or less');
     formHelper.setValue('dimlessBinary', '0.5 K', true);
-    fh.expectTextToBe(
-      '.cd-dimlessBinary-form-group .invalid-feedback',
-      'Size has to be at least 1 KiB or more'
-    );
+    fh.expectTextToBe('.cds--form-requirement', 'Size has to be at least 1 KiB or more');
   });
 
   it('shows result of dimlessBinary pipe', () => {
