@@ -47,7 +47,15 @@ setup_teuthology() {
     ${TEUTHOLOGY_PYTHON_BIN:-/usr/bin/python3} -m venv venv
     source venv/bin/activate
     pip install -U pip 'setuptools>=12,<60'
-    pip install "git+https://github.com/ceph/teuthology@2ef0dcd#egg=teuthology[test]"
+
+    local TEUTH_REF="${TEUTH_REF:-3ae1592c30adc5875a8aeb1f50a30ed9dd04dc04}"
+    git clone https://github.com/ceph/teuthology
+    pushd teuthology
+    git checkout -q "$TEUTH_REF"
+    pip install -r requirements.txt
+    pip install -e '.[test]'
+    popd
+
     pushd $CURR_DIR
     pip install -r requirements.txt -c constraints.txt
     popd
@@ -134,7 +142,7 @@ run_teuthology_tests() {
     export CEPH_OUT_CLIENT_DIR=${LOCAL_BUILD_DIR}/out/client
     find . -iname "*${COVERAGE_FILE}*" -type f -delete
 
-    python ../qa/tasks/vstart_runner.py --ignore-missing-binaries --no-verbose $OPTIONS $(echo $TEST_CASES) ||
+    python ../qa/tasks/vstart_runner.py --ignore-missing-binaries --no-verbose --debug $OPTIONS $(echo $TEST_CASES) ||
       on_tests_error
 
     deactivate
