@@ -1,7 +1,22 @@
 import ceph_module  # noqa
 
-from typing import cast, Tuple, Any, Dict, Generic, Optional, Callable, List, \
-    Mapping, NamedTuple, Sequence, Union, Set, TYPE_CHECKING
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    Iterator,
+    List,
+    Mapping,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Set,
+    TYPE_CHECKING,
+    Tuple,
+    Union,
+    cast,
+)
 if TYPE_CHECKING:
     import sys
     if sys.version_info >= (3, 8):
@@ -18,6 +33,7 @@ import json
 import subprocess
 import threading
 from collections import defaultdict
+from contextlib import contextmanager
 from enum import IntEnum, Enum
 import os
 import rados
@@ -85,6 +101,7 @@ PG_STATES = [
 NFS_GANESHA_SUPPORTED_FSALS = ['CEPH', 'RGW']
 NFS_POOL_NAME = '.nfs'
 
+
 class CephReleases(IntEnum):
     argonaut = 1
     bobtail = 2
@@ -107,6 +124,7 @@ class CephReleases(IntEnum):
     squid = 19
     maximum = 20
 
+
 class NotifyType(str, Enum):
     mon_map = 'mon_map'
     pg_summary = 'pg_summary'
@@ -166,8 +184,19 @@ class HandleCommandResult(NamedTuple):
     stderr: str = ""            # Typically used for error messages.
 
 
-class MonCommandFailed(RuntimeError): pass
-class MgrDBNotReady(RuntimeError): pass
+class MonCommandFailed(RuntimeError):
+    pass
+
+
+class MgrDBNotReady(RuntimeError):
+    pass
+
+
+class MgrDBNotAllowed(MgrDBNotReady):
+    """A more specific subclass of MgrDBNotReady raised when mgr_pool option
+    disabled.
+    """
+    pass
 
 
 class OSDMap(ceph_module.BasePyOSDMap):
@@ -356,6 +385,7 @@ class CRUSHMap(ceph_module.BasePyCRUSH):
 
 HandlerFuncType = Callable[..., Tuple[int, str, str]]
 
+
 def _extract_target_func(
     f: HandlerFuncType
 ) -> Tuple[HandlerFuncType, Dict[str, Any]]:
@@ -443,12 +473,15 @@ class CLICommand(object):
         self.desc, self.arg_spec, self.first_default, self.args = \
             self._load_func_metadata(f)
 
-    def __call__(self, func: HandlerFuncType) -> HandlerFuncType:
+    def _register_handler(self, func: HandlerFuncType) -> HandlerFuncType:
         self.store_func_metadata(func)
         self.func = func
         self.COMMANDS[self.prefix] = self
         return self.func
 
+    def __call__(self, func: HandlerFuncType) -> HandlerFuncType:
+        return self._register_handler(func)
+
     def _get_arg_value(self, kwargs_switch: bool, key: str, val: Any) -> Tuple[bool, str, Any]:
         def start_kwargs() -> bool:
             if isinstance(val, str) and '=' in val:
@@ -532,16 +565,23 @@ def CLICheckNonemptyFileInput(desc: str) -> Callable[[HandlerFuncType], HandlerF
                 # Delete new line separator at EOF (it may have been added by a text editor).
                 kwargs['inbuf'] = kwargs['inbuf'].rstrip('\r\n').rstrip('\n')
             if not kwargs['inbuf'] or not kwargs['inbuf'].strip():
-                return -errno.EINVAL, '', f'{ERROR_MSG_EMPTY_INPUT_FILE}: Please add {desc} to '\
-                                           'the file'
+                return (
+                    -errno.EINVAL,
+                    '',
+                    f'{ERROR_MSG_EMPTY_INPUT_FILE}: Please add {desc} to '
+                    'the file'
+                )
             return func(*args, **kwargs)
         check.__signature__ = inspect.signature(func)  # type: ignore[attr-defined]
         return check
     return CheckFileInput
 
+
 # If the mgr loses its lock on the database because e.g. the pgs were
 # transiently down, then close it and allow it to be reopened.
 MAX_DBCLEANUP_RETRIES = 3
+
+
 def MgrModuleRecoverDB(func: Callable) -> Callable:
     @functools.wraps(func)
     def check(self: MgrModule, *args: Any, **kwargs: Any) -> Any:
@@ -551,16 +591,17 @@ def MgrModuleRecoverDB(func: Callable) -> Callable:
                 return func(self, *args, **kwargs)
             except sqlite3.DatabaseError as e:
                 self.log.error(f"Caught fatal database error: {e}")
-                retries = retries+1
+                retries = retries + 1
                 if retries > MAX_DBCLEANUP_RETRIES:
                     raise
-                self.log.debug(f"attempting reopen of database")
+                self.log.debug("attempting reopen of database")
                 self.close_db()
-                self.open_db();
+                self.open_db()
                 # allow retry of func(...)
     check.__signature__ = inspect.signature(func)  # type: ignore[attr-defined]
     return check
 
+
 def CLIRequiresDB(func: HandlerFuncType) -> HandlerFuncType:
     @functools.wraps(func)
     def check(self: MgrModule, *args: Any, **kwargs: Any) -> Tuple[int, str, str]:
@@ -570,6 +611,7 @@ def CLIRequiresDB(func: HandlerFuncType) -> HandlerFuncType:
     check.__signature__ = inspect.signature(func)  # type: ignore[attr-defined]
     return check
 
+
 def _get_localized_key(prefix: str, key: str) -> str:
     return '{}/{}'.format(prefix, key)
 
@@ -585,12 +627,14 @@ if TYPE_CHECKING:
 # common/options.h: value_t
 OptionValue = Optional[Union[bool, int, float, str]]
 
+
 class OptionLevel(IntEnum):
     BASIC = 0
     ADVANCED = 1
     DEV = 2
     UNKNOWN = 3
 
+
 class Option(Dict):
     """
     Helper class to declare options for MODULE_OPTIONS list.
@@ -745,9 +789,9 @@ class MgrModuleLoggingMixin(object):
         # remove existing handlers:
         rm_handlers = [
             h for h in self._root_logger.handlers
-            if (isinstance(h, CPlusPlusHandler) or
-                isinstance(h, FileHandler) or
-                isinstance(h, ClusterLogHandler))]
+            if (isinstance(h, CPlusPlusHandler)
+                or isinstance(h, FileHandler)
+                or isinstance(h, ClusterLogHandler))]
         for h in rm_handlers:
             self._root_logger.removeHandler(h)
         self.log_to_file = False
@@ -973,7 +1017,7 @@ class API:
         class DecoratorClass:
             _ATTR_TOKEN = f'__ATTR_{attr.upper()}__'
 
-            def __init__(self, value: Any=default) -> None:
+            def __init__(self, value: Any = default) -> None:
                 self.value = value
 
             def __call__(self, func: Callable) -> Any:
@@ -998,8 +1042,8 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
     MODULE_OPTION_DEFAULTS = {}  # type: Dict[str, Any]
 
     # Database Schema
-    SCHEMA = None # type: Optional[List[str]]
-    SCHEMA_VERSIONED = None # type: Optional[List[List[str]]]
+    SCHEMA = None  # type: Optional[List[str]]
+    SCHEMA_VERSIONED = None  # type: Optional[List[List[str]]]
 
     # Priority definitions for perf counters
     PRIO_CRITICAL = 10
@@ -1059,7 +1103,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         # for backwards compatibility
         self._logger = self.getLogger()
 
-        self._db = None # type: Optional[sqlite3.Connection]
+        self._db = None  # type: Optional[sqlite3.Connection]
 
         self._version = self._ceph_get_version()
 
@@ -1187,15 +1231,15 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
 
     def create_skeleton_schema(self, db: sqlite3.Connection) -> None:
         SQL = [
-        """
-        CREATE TABLE IF NOT EXISTS MgrModuleKV (
-          key TEXT PRIMARY KEY,
-          value NOT NULL
-        ) WITHOUT ROWID;
-        """,
-        """
-        INSERT OR IGNORE INTO MgrModuleKV (key, value) VALUES ('__version', 0);
-        """,
+            """
+            CREATE TABLE IF NOT EXISTS MgrModuleKV (
+              key TEXT PRIMARY KEY,
+              value NOT NULL
+            ) WITHOUT ROWID;
+            """,
+            """
+            INSERT OR IGNORE INTO MgrModuleKV (key, value) VALUES ('__version', 0);
+            """,
         ]
 
         for sql in SQL:
@@ -1247,7 +1291,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             if latest < version:
                 raise RuntimeError(f"main.db version is newer ({version}) than module ({latest})")
             for i in range(version, latest):
-                self.log.info(f"upgrading main.db for {self.module_name} from {i-1}:{i}")
+                self.log.info(f"upgrading main.db for {self.module_name} from {i - 1}:{i}")
                 for sql in self.SCHEMA_VERSIONED[i]:
                     db.execute(sql)
             if version < latest:
@@ -1292,9 +1336,10 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
     def open_db(self) -> Optional[sqlite3.Connection]:
         if not self.pool_exists(self.MGR_POOL_NAME):
             if not self.have_enough_osds():
+                self.log.warning('not enough osds to create mgr pool')
                 return None
             self.create_mgr_pool()
-        uri = f"file:///{self.MGR_POOL_NAME}:{self.module_name}/main.db?vfs=ceph";
+        uri = f"file:///{self.MGR_POOL_NAME}:{self.module_name}/main.db?vfs=ceph"
         self.log.debug(f"using uri {uri}")
         db = sqlite3.connect(uri, check_same_thread=False, uri=True, isolation_level=None)  # type: ignore[call-arg]
         # if libcephsqlite reconnects, update the addrv for blocklist
@@ -1322,12 +1367,29 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             return self._db
         db_allowed = self.get_ceph_option("mgr_pool")
         if not db_allowed:
-            raise MgrDBNotReady();
+            raise MgrDBNotAllowed()
         self._db = self.open_db()
         if self._db is None:
-            raise MgrDBNotReady();
+            raise MgrDBNotReady()
         return self._db
 
+    @contextmanager
+    def exclusive_db_access(self) -> Iterator[sqlite3.Connection]:
+        """Context manager that grants exclusive access to the manager module sqlite3
+        db connection, while establishing a new db transaction.
+        """
+        with self._db_lock, self.db:
+            yield self.db
+
+    @contextmanager
+    def exclusive_db_cursor(self) -> Iterator[sqlite3.Cursor]:
+        """Context manager that yields a db cursor after getting exclusive
+        access to the manager module sqlite3 connection and a new db
+        transaction.
+        """
+        with self.exclusive_db_access() as db:
+            yield db.cursor()
+
     @property
     def release_name(self) -> str:
         """
@@ -1439,7 +1501,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             All these structures have their own JSON representations: experiment
             or look at the C++ ``dump()`` methods to learn about them.
         """
-        obj =  self._ceph_get(data_name)
+        obj = self._ceph_get(data_name)
         if isinstance(obj, bytes):
             obj = json.loads(obj)
 
@@ -1568,6 +1630,21 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         """
         return cast(ServerInfoT, self._ceph_get_server(hostname))
 
+    @API.expose
+    def get_unlabeled_perf_schema(
+        self, svc_type: str, svc_name: str
+    ) -> Dict[str, Dict[str, Dict[str, Union[str, int]]]]:
+        """
+        Called by the plugin to fetch unlabeled perf counter schema info.
+        svc_name can be nullptr, as can svc_type, in which case
+        they are wildcards
+
+        :param str svc_type:
+        :param str svc_name:
+        :return: list of dicts describing the counters requested
+        """
+        return self._ceph_get_unlabeled_perf_schema(svc_type, svc_name)
+
     @API.expose
     def get_perf_schema(self,
                         svc_type: str,
@@ -1593,10 +1670,9 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         return self._ceph_get_rocksdb_version()
 
     @API.expose
-    def get_counter(self,
-                    svc_type: str,
-                    svc_name: str,
-                    path: str) -> Dict[str, List[Tuple[float, int]]]:
+    def get_unlabeled_counter(
+        self, svc_type: str, svc_name: str, path: str
+    ) -> Dict[str, List[Tuple[float, int]]]:
         """
         Called by the plugin to fetch the latest performance counter data for a
         particular counter on a particular service.
@@ -1609,17 +1685,15 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             of two-tuples of (timestamp, value).  This may be empty if no data is
             available.
         """
-        return self._ceph_get_counter(svc_type, svc_name, path)
+        return self._ceph_get_unlabeled_counter(svc_type, svc_name, path)
 
     @API.expose
-    def get_latest_counter(self,
-                           svc_type: str,
-                           svc_name: str,
-                           path: str) -> Dict[str, Union[Tuple[float, int],
-                                                         Tuple[float, int, int]]]:
+    def get_latest_unlabeled_counter(
+        self, svc_type: str, svc_name: str, path: str
+    ) -> Dict[str, Union[Tuple[float, int], Tuple[float, int, int]]]:
         """
-        Called by the plugin to fetch only the newest performance counter data
-        point for a particular counter on a particular service.
+        Called by the plugin to fetch only the newest performance unlabeled counter
+        data point for a particular counter on a particular service.
 
         :param str svc_type:
         :param str svc_name:
@@ -1629,7 +1703,33 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             (timestamp, value, count) is returned.  This may be empty if no
             data is available.
         """
-        return self._ceph_get_latest_counter(svc_type, svc_name, path)
+        return self._ceph_get_latest_unlabeled_counter(svc_type, svc_name, path)
+
+    @API.expose
+    def get_latest_counter(self,
+                           svc_type: str,
+                           svc_name: str,
+                           counter_name: str,
+                           sub_counter_name: str,
+                           labels: List[Tuple[str, str]]) -> Dict[str, Union[Tuple[float, int],
+                                                                             Tuple[float, int, int]]]:
+        """
+        Called by the plugin to fetch only the newest performance counter data
+        point for a particular counter on a particular service.
+
+        :param str svc_type:
+        :param str svc_name:
+        :param str counter_name: the key_name of the counter, for example
+            "osd_scrub_sh_repl"
+        :param str sub_counter_name: the counters present under the key_name,
+            for example "successful_scrubs_elapsed"
+        :param list[(str, str)] labels: the labels associated with the counter,
+            for example "[("level", "deep"), ("pooltype", "ec")]"
+        :return: A list of two-tuples of (timestamp, value) or three-tuple of
+            (timestamp, value, count) is returned.  This may be empty if no
+            data is available.
+        """
+        return self._ceph_get_latest_counter(svc_type, svc_name, counter_name, sub_counter_name, labels)
 
     @API.expose
     def list_servers(self) -> List[ServerInfoT]:
@@ -1761,11 +1861,11 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         return r
 
     def get_quiesce_leader_gid(self, fscid: str) -> Optional[int]:
-        leader_gid : Optional[int] = None
+        leader_gid: Optional[int] = None
         for fs in self.get("fs_map")['filesystems']:
             if fscid != fs["id"]:
                 continue
-            
+
             # quiesce leader is the lowest rank
             # with the highest state
             mdsmap = fs["mdsmap"]
@@ -1814,7 +1914,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             command: str,
             tag: str,
             inbuf: Optional[str] = None,
-            *, # kw-only args go below
+            *,  # kw-only args go below
             one_shot: bool = False) -> None:
         """
         Called by the plugin to send a command to the mon
@@ -2070,7 +2170,8 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
     @API.expose
     def set_localized_module_option(self, key: str, val: Optional[str]) -> None:
         """
-        Set localized configuration for this ceph-mgr instance
+        Set localized configuration for this ceph-mgr instance.
+
         :param str key:
         :param str val:
         :return: str
@@ -2135,8 +2236,8 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         return cast(OSDMap, self._ceph_get_osdmap())
 
     @API.expose
-    def get_latest(self, daemon_type: str, daemon_name: str, counter: str) -> int:
-        data = self.get_latest_counter(
+    def get_unlabeled_counter_latest(self, daemon_type: str, daemon_name: str, counter: str) -> int:
+        data = self.get_latest_unlabeled_counter(
             daemon_type, daemon_name, counter)[counter]
         if data:
             return data[1]
@@ -2144,8 +2245,18 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
             return 0
 
     @API.expose
-    def get_latest_avg(self, daemon_type: str, daemon_name: str, counter: str) -> Tuple[int, int]:
+    def get_counter_latest(self, daemon_type: str, daemon_name: str, counter_name: str,
+                           sub_counter_name: str, labels: List[Tuple[str, str]]) -> int:
         data = self.get_latest_counter(
+            daemon_type, daemon_name, counter_name, sub_counter_name, labels)[counter_name]
+        if data:
+            return data[1]
+        else:
+            return 0
+
+    @API.expose
+    def get_unlabeled_counter_latest_avg(self, daemon_type: str, daemon_name: str, counter: str) -> Tuple[int, int]:
+        data = self.get_latest_unlabeled_counter(
             daemon_type, daemon_name, counter)[counter]
         if data:
             # https://github.com/python/mypy/issues/1178
@@ -2154,12 +2265,33 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         else:
             return 0, 0
 
+    @API.expose
+    def get_counter_latest_avg(self, daemon_type: str, daemon_name: str, counter_name: str,
+                               sub_counter_name: str, labels: List[Tuple[str, str]]) -> Tuple[int, int]:
+        data = self.get_latest_counter(
+            daemon_type, daemon_name, counter_name, sub_counter_name, labels)[counter_name]
+        if data:
+            # https://github.com/python/mypy/issues/1178
+            _, value, count = cast(Tuple[float, int, int], data)
+            return value, count
+        else:
+            return 0, 0
+
     @API.expose
     @profile_method()
-    def get_unlabeled_perf_counters(self, prio_limit: int = PRIO_USEFUL,
-                              services: Sequence[str] = ("mds", "mon", "osd",
-                                                         "rbd-mirror", "cephfs-mirror", "rgw",
-                                                         "tcmu-runner")) -> Dict[str, dict]:
+    def get_unlabeled_perf_counters(
+        self,
+        prio_limit: int = PRIO_USEFUL,
+        services: Sequence[str] = (
+            "mds",
+            "mon",
+            "osd",
+            "rbd-mirror",
+            "cephfs-mirror",
+            "rgw",
+            "tcmu-runner",
+        ),
+    ) -> Dict[str, dict]:
         """
         Return the perf counters currently known to this ceph-mgr
         instance, filtered by priority equal to or greater than `prio_limit`.
@@ -2179,7 +2311,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
                 if service['type'] not in services:
                     continue
 
-                schemas = self.get_perf_schema(service['type'], service['id'])
+                schemas = self.get_unlabeled_perf_schema(service['type'], service['id'])
                 if not schemas:
                     self.log.warning("No perf counter schema for {0}.{1}".format(
                         service['type'], service['id']
@@ -2207,7 +2339,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
                     counter_info = dict(counter_schema)
                     # Also populate count for the long running avgs
                     if tp & self.PERFCOUNTER_LONGRUNAVG:
-                        v, c = self.get_latest_avg(
+                        v, c = self.get_unlabeled_counter_latest_avg(
                             service['type'],
                             service['id'],
                             counter_path
@@ -2215,7 +2347,7 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
                         counter_info['value'], counter_info['count'] = v, c
                         result[svc_full_name][counter_path] = counter_info
                     else:
-                        counter_info['value'] = self.get_latest(
+                        counter_info['value'] = self.get_unlabeled_counter_latest(
                             service['type'],
                             service['id'],
                             counter_path
@@ -2227,6 +2359,127 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
 
         return result
 
+    @API.expose
+    @profile_method()
+    def get_perf_counters(
+        self,
+        prio_limit: int = PRIO_USEFUL,
+        services: Sequence[str] = (
+            "mds",
+            "mon",
+            "osd",
+            "rbd-mirror",
+            "cephfs-mirror",
+            "rgw",
+            "tcmu-runner",
+        ),
+    ) -> Dict[str, dict]:
+        """
+        Return the perf counters currently known to this ceph-mgr
+        instance, filtered by priority equal to or greater than `prio_limit`.
+        The result is a map of string to dict, associating services
+        (like "osd.123") with their counters.  The counter
+        dict for each service maps counter paths to a counter
+        info structure, which is the information from
+        the schema, plus an additional "value" member with the latest
+        value.
+
+        The returned dictionary looks like:
+        ```
+        {
+            "mon.a": {
+                "AsyncMessenger::Worker": [
+                    {
+                        "labels": {
+                            "id": "1"
+                        },
+                        "counters": {
+                            "msgr_connection_ready_timeouts": {
+                                "type": 10,
+                                "priority": 5,
+                                "units": 1,
+                                "value": 0
+                            },
+                            "msgr_connection_idle_timeouts": {
+                                "type": 10,
+                                "priority": 5,
+                                "units": 1,
+                                "value": 0
+                            }
+                        }
+                    }
+                ]
+            }
+        }
+        ```
+
+        """
+
+        result = defaultdict(dict)   # type: Dict[str, dict]
+
+        for server in self.list_servers():
+            for service in cast(List[ServiceInfoT], server['services']):
+                if service['type'] not in services:
+                    continue
+
+                schemas = self.get_perf_schema(service['type'], service['id'])
+
+                if not schemas:
+                    self.log.warning("No perf counter schema for {0}.{1}".format(
+                        service['type'], service['id']
+                    ))
+                    continue
+
+                # Value is returned in a potentially-multi-service format,
+                # get just the service we're asking about
+                svc_full_name = "{0}.{1}".format(
+                    service['type'], service['id'])
+                labeled_schema = schemas[svc_full_name]
+
+                for counter_name, sub_counters_list in labeled_schema.items():
+                    result[svc_full_name][counter_name] = []
+                    for sub_counter in sub_counters_list:
+                        sub_counter_labels = []
+                        sub_counter_info = dict(sub_counter)
+
+                        for label_key, label_value in sub_counter["labels"].items():
+                            sub_counter_labels.append((label_key, label_value))
+
+                        for sub_counter_name, sub_counter_schema in sub_counter["counters"].items():
+                            priority = sub_counter_schema['priority']
+                            assert isinstance(priority, int)
+                            if priority < prio_limit:
+                                continue
+
+                            tp = sub_counter_schema['type']
+                            assert isinstance(tp, int)
+
+                            # Also populate count for the long running avgs
+                            if tp & self.PERFCOUNTER_LONGRUNAVG:
+                                v, c = self.get_counter_latest_avg(
+                                    service['type'],
+                                    service['id'],
+                                    counter_name,
+                                    sub_counter_name,
+                                    sub_counter_labels,
+                                )
+                                sub_counter_info['counters'][sub_counter_name]['value'] = v
+                                sub_counter_info['counters'][sub_counter_name]['count'] = c
+
+                            else:
+                                sub_counter_info['counters'][sub_counter_name]['value'] = self.get_counter_latest(
+                                    service['type'],
+                                    service['id'],
+                                    counter_name,
+                                    sub_counter_name,
+                                    sub_counter_labels
+                                )
+
+                        result[svc_full_name][counter_name].append(sub_counter_info)
+        self.log.debug("returning {0} counter".format(len(result)))
+
+        return result
+
     @API.expose
     def set_uri(self, uri: str) -> None:
         """
@@ -2428,7 +2681,6 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
         return self._ceph_remove_mds_perf_query(query_id)
 
     @API.expose
-
     def reregister_mds_perf_queries(self) -> None:
         """
         Re-register MDS perf queries.
@@ -2466,11 +2718,11 @@ class MgrModule(ceph_module.BaseMgrModule, MgrModuleLoggingMixin):
                               stdout_as_json: bool = True) -> Tuple[int, Union[str, dict], str]:
         try:
             cmd = [
-                    'radosgw-admin',
-                    '-c', str(self.get_ceph_conf_path()),
-                    '-k', str(self.get_ceph_option('keyring')),
-                    '-n', f'mgr.{self.get_mgr_id()}',
-                ] + args
+                'radosgw-admin',
+                '-c', str(self.get_ceph_conf_path()),
+                '-k', str(self.get_ceph_option('keyring')),
+                '-n', f'mgr.{self.get_mgr_id()}',
+            ] + args
             self.log.debug('Executing %s', str(cmd))
             result = subprocess.run(  # pylint: disable=subprocess-run-check
                 cmd,
