@@ -1,14 +1,21 @@
 from ceph.deployment.inventory import Devices, Device
 
+try:
+    from typing import Any, List
+except ImportError:
+    pass  # for type checking
+
 
 def _mk_device(rotational=True,
                locked=False,
-               size="394.27 GB"):
+               size="394.27 GB",
+               vendor='Vendor',
+               model='Model'):
     return [Device(
         path='??',
         sys_api={
             "rotational": '1' if rotational else '0',
-            "vendor": "Vendor",
+            "vendor": vendor,
             "human_readable_size": size,
             "partitions": {},
             "locked": int(locked),
@@ -16,7 +23,7 @@ def _mk_device(rotational=True,
             "removable": "0",
             "path": "??",
             "support_discard": "",
-            "model": "Model",
+            "model": model,
             "ro": "0",
             "nr_requests": "128",
             "size": 423347879936  # ignore coversion from human_readable_size
@@ -28,7 +35,7 @@ def _mk_device(rotational=True,
     )]
 
 
-def _mk_inventory(devices):
+def _mk_inventory(devices: Any) -> List[Device]:
     devs = []
     for dev_, name in zip(devices, map(chr, range(ord('a'), ord('z')))):
         dev = Device.from_json(dev_.to_json())
