@@ -11,12 +11,14 @@
 
 #define dout_subsys ceph_subsys_rgw
 
+using namespace std;
+
 void RGWLoadGenRequestEnv::set_date(utime_t& tm)
 {
   date_str = rgw_to_asctime(tm);
 }
 
-int RGWLoadGenRequestEnv::sign(RGWAccessKey& access_key)
+int RGWLoadGenRequestEnv::sign(const DoutPrefixProvider *dpp, RGWAccessKey& access_key)
 {
   meta_map_t meta_map;
   map<string, string> sub_resources;
@@ -24,7 +26,8 @@ int RGWLoadGenRequestEnv::sign(RGWAccessKey& access_key)
   string canonical_header;
   string digest;
 
-  rgw_create_s3_canonical_header(request_method.c_str(),
+  rgw_create_s3_canonical_header(dpp,
+                                 request_method.c_str(),
                                  nullptr, /* const char *content_md5 */
                                  content_type.c_str(),
                                  date_str.c_str(),
@@ -111,8 +114,8 @@ size_t RGWLoadGenIO::send_100_continue()
   return 0;
 }
 
-size_t RGWLoadGenIO::send_header(const boost::string_ref& name,
-                                 const boost::string_ref& value)
+size_t RGWLoadGenIO::send_header(const std::string_view& name,
+                                 const std::string_view& value)
 {
   return 0;
 }
