@@ -435,6 +435,17 @@ std::unique_ptr<Lifecycle> FilterDriver::get_lifecycle(void)
   return std::make_unique<FilterLifecycle>(std::move(lc));
 }
 
+bool FilterDriver::process_expired_objects(const DoutPrefixProvider *dpp,
+	       			           optional_yield y) {
+  return next->process_expired_objects(dpp, y);
+}
+
+std::unique_ptr<Restore> FilterDriver::get_restore()
+{
+  std::unique_ptr<Restore> restore = next->get_restore();
+  return std::make_unique<FilterRestore>(std::move(restore));
+}
+
 std::unique_ptr<Notification> FilterDriver::get_notification(rgw::sal::Object* obj,
 				rgw::sal::Object* src_obj, req_state* s,
 				rgw::notify::EventType event_type, optional_yield y,
@@ -482,6 +493,11 @@ RGWLC* FilterDriver::get_rgwlc()
   return next->get_rgwlc();
 }
 
+rgw::restore::Restore* FilterDriver::get_rgwrestore()
+{
+  return next->get_rgwrestore();
+}
+
 RGWCoroutinesManagerRegistry* FilterDriver::get_cr_registry()
 {
   return next->get_cr_registry();
@@ -839,14 +855,14 @@ int FilterBucket::load_bucket(const DoutPrefixProvider* dpp, optional_yield y)
   return next->load_bucket(dpp, y);
 }
 
-int FilterBucket::read_stats(const DoutPrefixProvider *dpp,
+int FilterBucket::read_stats(const DoutPrefixProvider *dpp, optional_yield y,
 			     const bucket_index_layout_generation& idx_layout,
 			     int shard_id, std::string* bucket_ver,
 			     std::string* master_ver,
 			     std::map<RGWObjCategory, RGWStorageStats>& stats,
 			     std::string* max_marker, bool* syncstopped)
 {
-  return next->read_stats(dpp, idx_layout, shard_id, bucket_ver, master_ver,
+  return next->read_stats(dpp, y, idx_layout, shard_id, bucket_ver, master_ver,
 			  stats, max_marker, syncstopped);
 }
 
@@ -930,21 +946,21 @@ int FilterBucket::remove_objs_from_index(const DoutPrefixProvider *dpp,
   return next->remove_objs_from_index(dpp, objs_to_unlink);
 }
 
-int FilterBucket::check_index(const DoutPrefixProvider *dpp,
+int FilterBucket::check_index(const DoutPrefixProvider *dpp, optional_yield y,
 			      std::map<RGWObjCategory, RGWStorageStats>& existing_stats,
 			      std::map<RGWObjCategory, RGWStorageStats>& calculated_stats)
 {
-  return next->check_index(dpp, existing_stats, calculated_stats);
+  return next->check_index(dpp, y, existing_stats, calculated_stats);
 }
 
-int FilterBucket::rebuild_index(const DoutPrefixProvider *dpp)
+int FilterBucket::rebuild_index(const DoutPrefixProvider *dpp, optional_yield y)
 {
-  return next->rebuild_index(dpp);
+  return next->rebuild_index(dpp, y);
 }
 
-int FilterBucket::set_tag_timeout(const DoutPrefixProvider *dpp, uint64_t timeout)
+int FilterBucket::set_tag_timeout(const DoutPrefixProvider *dpp, optional_yield y, uint64_t timeout)
 {
-  return next->set_tag_timeout(dpp, timeout);
+  return next->set_tag_timeout(dpp, y, timeout);
 }
 
 int FilterBucket::purge_instance(const DoutPrefixProvider* dpp, optional_yield y)
@@ -1046,10 +1062,20 @@ RGWAccessControlPolicy& FilterObject::get_acl()
   return next->get_acl();
 }
 
-int FilterObject::get_obj_state(const DoutPrefixProvider* dpp, RGWObjState **pstate,
-				optional_yield y, bool follow_olh)
+int FilterObject::list_parts(const DoutPrefixProvider* dpp, CephContext* cct,
+			     int max_parts, int marker, int* next_marker,
+			     bool* truncated, list_parts_each_t each_func,
+			     optional_yield y)
 {
-  return next->get_obj_state(dpp, pstate, y, follow_olh);
+  return next->list_parts(dpp, cct, max_parts, marker, next_marker,
+			  truncated,
+			  sal::Object::list_parts_each_t(each_func),
+			  y);
+}
+
+int FilterObject::load_obj_state(const DoutPrefixProvider *dpp,
+                                 optional_yield y, bool follow_olh) {
+  return next->load_obj_state(dpp, y, follow_olh);
 }
 
 int FilterObject::set_obj_attrs(const DoutPrefixProvider* dpp, Attrs* setattrs,
@@ -1065,9 +1091,9 @@ int FilterObject::get_obj_attrs(optional_yield y, const DoutPrefixProvider* dpp,
 }
 
 int FilterObject::modify_obj_attrs(const char* attr_name, bufferlist& attr_val,
-				   optional_yield y, const DoutPrefixProvider* dpp)
+				   optional_yield y, const DoutPrefixProvider* dpp, uint32_t flags)
 {
-  return next->modify_obj_attrs(attr_name, attr_val, y, dpp);
+  return next->modify_obj_attrs(attr_name, attr_val, y, dpp, flags);
 }
 
 int FilterObject::delete_obj_attrs(const DoutPrefixProvider* dpp,
@@ -1118,6 +1144,18 @@ int FilterObject::transition_to_cloud(Bucket* bucket,
 				   o, cloud_targets, cct, update_object, dpp, y);
 }
 
+int FilterObject::restore_obj_from_cloud(Bucket* bucket,
+		          rgw::sal::PlacementTier* tier,
+		          CephContext* cct,
+		          std::optional<uint64_t> days,
+			  bool& in_progress,
+		          const DoutPrefixProvider* dpp,
+		          optional_yield y)
+{
+  return next->restore_obj_from_cloud(nextBucket(bucket), nextPlacementTier(tier),
+           cct, days, in_progress, dpp, y);
+}
+
 bool FilterObject::placement_rules_match(rgw_placement_rule& r1, rgw_placement_rule& r2)
 {
   return next->placement_rules_match(r1, r2);
@@ -1194,7 +1232,12 @@ int FilterObject::FilterReadOp::prepare(optional_yield y, const DoutPrefixProvid
 {
   /* Copy params into next */
   next->params = params;
-  return next->prepare(y, dpp);
+  int ret = next->prepare(y, dpp);
+  if (ret < 0)
+    return ret;
+
+  params.parts_count = next->params.parts_count;
+  return 0;
 }
 
 int FilterObject::FilterReadOp::read(int64_t ofs, int64_t end, bufferlist& bl,
@@ -1248,6 +1291,9 @@ int FilterMultipartUpload::init(const DoutPrefixProvider *dpp, optional_yield y,
 				ACLOwner& owner, rgw_placement_rule& dest_placement,
 				rgw::sal::Attrs& attrs)
 {
+  next->obj_legal_hold = obj_legal_hold;
+  next->obj_retention = obj_retention;
+  next->cksum_type = cksum_type;
   return next->init(dpp, y, owner, dest_placement, attrs);
 }
 
@@ -1306,7 +1352,15 @@ int FilterMultipartUpload::get_info(const DoutPrefixProvider *dpp,
 				    optional_yield y, rgw_placement_rule** rule,
 				    rgw::sal::Attrs* attrs)
 {
-  return next->get_info(dpp, y, rule, attrs);
+  auto ret = next->get_info(dpp, y, rule, attrs);
+  if (ret < 0) {
+    return ret;
+  }
+
+  this->obj_legal_hold = next->obj_legal_hold;
+  this->obj_retention = next->obj_retention;
+  this->cksum_type = next->cksum_type;
+  return 0;
 }
 
 std::unique_ptr<Writer> FilterMultipartUpload::get_writer(
@@ -1337,90 +1391,50 @@ int FilterLCSerializer::try_lock(const DoutPrefixProvider *dpp, utime_t dur,
   return next->try_lock(dpp, dur, y);
 }
 
-std::unique_ptr<Lifecycle::LCEntry> FilterLifecycle::get_entry()
+int FilterLifecycle::get_entry(const DoutPrefixProvider* dpp, optional_yield y,
+                               const std::string& oid, const std::string& marker,
+			       LCEntry& entry)
 {
-  std::unique_ptr<Lifecycle::LCEntry> e = next->get_entry();
-  return std::make_unique<FilterLCEntry>(std::move(e));
+  return next->get_entry(dpp, y, oid, marker, entry);
 }
 
-int FilterLifecycle::get_entry(const std::string& oid, const std::string& marker,
-			       std::unique_ptr<LCEntry>* entry)
+int FilterLifecycle::get_next_entry(const DoutPrefixProvider* dpp, optional_yield y,
+                                    const std::string& oid, const std::string& marker,
+				    LCEntry& entry)
 {
-  std::unique_ptr<LCEntry> ne;
-  int ret;
-
-  ret = next->get_entry(oid, marker, &ne);
-  if (ret < 0)
-    return ret;
-
-  LCEntry* e = new FilterLCEntry(std::move(ne));
-  entry->reset(e);
-
-  return 0;
+  return next->get_next_entry(dpp, y, oid, marker, entry);
 }
 
-int FilterLifecycle::get_next_entry(const std::string& oid, const std::string& marker,
-				    std::unique_ptr<LCEntry>* entry)
+int FilterLifecycle::set_entry(const DoutPrefixProvider* dpp, optional_yield y,
+                               const std::string& oid, const LCEntry& entry)
 {
-  std::unique_ptr<LCEntry> ne;
-  int ret;
-
-  ret = next->get_next_entry(oid, marker, &ne);
-  if (ret < 0)
-    return ret;
-
-  LCEntry* e = new FilterLCEntry(std::move(ne));
-  entry->reset(e);
-
-  return 0;
+  return next->set_entry(dpp, y, oid, entry);
 }
 
-int FilterLifecycle::set_entry(const std::string& oid, LCEntry& entry)
-{
-  return next->set_entry(oid, entry);
-}
-
-int FilterLifecycle::list_entries(const std::string& oid, const std::string& marker,
+int FilterLifecycle::list_entries(const DoutPrefixProvider* dpp, optional_yield y,
+                                  const std::string& oid, const std::string& marker,
 				  uint32_t max_entries,
-				  std::vector<std::unique_ptr<LCEntry>>& entries)
+				  std::vector<LCEntry>& entries)
 {
-  std::vector<std::unique_ptr<LCEntry>> ne;
-  int ret;
-
-  ret = next->list_entries(oid, marker, max_entries, ne);
-  if (ret < 0)
-    return ret;
-
-  for (auto& ent : ne) {
-    entries.emplace_back(std::make_unique<FilterLCEntry>(std::move(ent)));
-  }
-
-  return 0;
+  return next->list_entries(dpp, y, oid, marker, max_entries, entries);
 }
 
-int FilterLifecycle::rm_entry(const std::string& oid, LCEntry& entry)
+int FilterLifecycle::rm_entry(const DoutPrefixProvider* dpp, optional_yield y,
+                              const std::string& oid, const LCEntry& entry)
 {
-  return next->rm_entry(oid, entry);
+  return next->rm_entry(dpp, y, oid, entry);
 }
 
-int FilterLifecycle::get_head(const std::string& oid, std::unique_ptr<LCHead>* head)
+int FilterLifecycle::get_head(const DoutPrefixProvider* dpp, optional_yield y,
+                              const std::string& oid, LCHead& head)
 {
-  std::unique_ptr<LCHead> nh;
-  int ret;
-
-  ret = next->get_head(oid, &nh);
-  if (ret < 0)
-    return ret;
-
-  LCHead* h = new FilterLCHead(std::move(nh));
-  head->reset(h);
-
-  return 0;
+  return next->get_head(dpp, y, oid, head);
 }
 
-int FilterLifecycle::put_head(const std::string& oid, LCHead& head)
+int FilterLifecycle::put_head(const DoutPrefixProvider* dpp, optional_yield y,
+                              const std::string& oid, const LCHead& head)
 {
-  return next->put_head(oid, *(dynamic_cast<FilterLCHead&>(head).next.get()));
+  return next->put_head(dpp, y, oid, head);
 }
 
 std::unique_ptr<LCSerializer> FilterLifecycle::get_serializer(
@@ -1434,6 +1448,46 @@ std::unique_ptr<LCSerializer> FilterLifecycle::get_serializer(
   return std::make_unique<FilterLCSerializer>(std::move(ns));
 }
 
+int FilterRestoreSerializer::try_lock(const DoutPrefixProvider *dpp, utime_t dur,
+				 optional_yield y)
+{
+  return next->try_lock(dpp, dur, y);
+}
+
+std::unique_ptr<RestoreSerializer> FilterRestore::get_serializer(const std::string& lock_name,
+						       const std::string& oid,
+						       const std::string& cookie) {
+  std::unique_ptr<RestoreSerializer> ns;
+  ns = next->get_serializer(lock_name, oid, cookie);
+  return std::make_unique<FilterRestoreSerializer>(std::move(ns));
+}
+
+int FilterRestore::initialize(const DoutPrefixProvider* dpp, optional_yield y,
+		  int n_objs, std::vector<std::string>& obj_names) {
+  return next->initialize(dpp, y, n_objs, obj_names);
+}
+
+int FilterRestore::add_entries(const DoutPrefixProvider* dpp, optional_yield y,
+	       		       int index,
+			       const std::vector<rgw::restore::RestoreEntry>& restore_entries) {
+  return next->add_entries(dpp, y, index, restore_entries);
+}
+
+/** List all known entries */
+int FilterRestore::list(const DoutPrefixProvider *dpp, optional_yield y,
+	       	   int index, const std::string& marker, std::string* out_marker,
+		   uint32_t max_entries, std::vector<rgw::restore::RestoreEntry>& entries,
+		   bool* truncated) {
+  return next->list(dpp, y, index, marker, out_marker, max_entries,
+  		    entries, truncated);
+}
+
+int FilterRestore::trim_entries(const DoutPrefixProvider *dpp, optional_yield y,
+		 	        int index, const std::string_view& marker) {
+  return next->trim_entries(dpp, y, index, marker);
+}
+
+
 int FilterNotification::publish_reserve(const DoutPrefixProvider *dpp,
 					RGWObjTags* obj_tags)
 {
@@ -1455,6 +1509,7 @@ int FilterWriter::process(bufferlist&& data, uint64_t offset)
 int FilterWriter::complete(size_t accounted_size, const std::string& etag,
                        ceph::real_time *mtime, ceph::real_time set_mtime,
                        std::map<std::string, bufferlist>& attrs,
+		       const std::optional<rgw::cksum::Cksum>& cksum,
                        ceph::real_time delete_at,
                        const char *if_match, const char *if_nomatch,
                        const std::string *user_data,
@@ -1462,7 +1517,7 @@ int FilterWriter::complete(size_t accounted_size, const std::string& etag,
                        const req_context& rctx,
                        uint32_t flags)
 {
-  return next->complete(accounted_size, etag, mtime, set_mtime, attrs,
+  return next->complete(accounted_size, etag, mtime, set_mtime, attrs, cksum,
 			delete_at, if_match, if_nomatch, user_data, zones_trace,
 			canceled, rctx, flags);
 }
