@@ -10,12 +10,14 @@
 #include <jni.h>
 #include <stdio.h>
 #include <stdlib.h>
+
 #include <memory>
 #include <string>
 #include <vector>
 
 #include "include/org_rocksdb_TtlDB.h"
 #include "rocksdb/utilities/db_ttl.h"
+#include "rocksjni/cplusplus_to_java_convert.h"
 #include "rocksjni/portal.h"
 
 /*
@@ -23,9 +25,9 @@
  * Method:    open
  * Signature: (JLjava/lang/String;IZ)J
  */
-jlong Java_org_rocksdb_TtlDB_open(
-    JNIEnv* env, jclass, jlong joptions_handle, jstring jdb_path, jint jttl,
-    jboolean jread_only) {
+jlong Java_org_rocksdb_TtlDB_open(JNIEnv* env, jclass, jlong joptions_handle,
+                                  jstring jdb_path, jint jttl,
+                                  jboolean jread_only) {
   const char* db_path = env->GetStringUTFChars(jdb_path, nullptr);
   if (db_path == nullptr) {
     // exception thrown: OutOfMemoryError
@@ -41,7 +43,7 @@ jlong Java_org_rocksdb_TtlDB_open(
   // as TTLDB extends RocksDB on the java side, we can reuse
   // the RocksDB portal here.
   if (s.ok()) {
-    return reinterpret_cast<jlong>(db);
+    return GET_CPLUSPLUS_POINTER(db);
   } else {
     ROCKSDB_NAMESPACE::RocksDBExceptionJni::ThrowNew(env, s);
     return 0;
@@ -53,10 +55,11 @@ jlong Java_org_rocksdb_TtlDB_open(
  * Method:    openCF
  * Signature: (JLjava/lang/String;[[B[J[IZ)[J
  */
-jlongArray Java_org_rocksdb_TtlDB_openCF(
-    JNIEnv* env, jclass, jlong jopt_handle, jstring jdb_path,
-    jobjectArray jcolumn_names, jlongArray jcolumn_options,
-    jintArray jttls, jboolean jread_only) {
+jlongArray Java_org_rocksdb_TtlDB_openCF(JNIEnv* env, jclass, jlong jopt_handle,
+                                         jstring jdb_path,
+                                         jobjectArray jcolumn_names,
+                                         jlongArray jcolumn_options,
+                                         jintArray jttls, jboolean jread_only) {
   const char* db_path = env->GetStringUTFChars(jdb_path, nullptr);
   if (db_path == nullptr) {
     // exception thrown: OutOfMemoryError
@@ -121,9 +124,9 @@ jlongArray Java_org_rocksdb_TtlDB_openCF(
     const jsize resultsLen = 1 + len_cols;  // db handle + column family handles
     std::unique_ptr<jlong[]> results =
         std::unique_ptr<jlong[]>(new jlong[resultsLen]);
-    results[0] = reinterpret_cast<jlong>(db);
+    results[0] = GET_CPLUSPLUS_POINTER(db);
     for (int i = 1; i <= len_cols; i++) {
-      results[i] = reinterpret_cast<jlong>(handles[i - 1]);
+      results[i] = GET_CPLUSPLUS_POINTER(handles[i - 1]);
     }
 
     jlongArray jresults = env->NewLongArray(resultsLen);
@@ -151,8 +154,7 @@ jlongArray Java_org_rocksdb_TtlDB_openCF(
  * Method:    disposeInternal
  * Signature: (J)V
  */
-void Java_org_rocksdb_TtlDB_disposeInternal(
-    JNIEnv*, jobject, jlong jhandle) {
+void Java_org_rocksdb_TtlDB_disposeInternal(JNIEnv*, jobject, jlong jhandle) {
   auto* ttl_db = reinterpret_cast<ROCKSDB_NAMESPACE::DBWithTTL*>(jhandle);
   assert(ttl_db != nullptr);
   delete ttl_db;
@@ -163,14 +165,15 @@ void Java_org_rocksdb_TtlDB_disposeInternal(
  * Method:    closeDatabase
  * Signature: (J)V
  */
-void Java_org_rocksdb_TtlDB_closeDatabase(
-    JNIEnv* /* env */, jclass, jlong /* jhandle */) {
+void Java_org_rocksdb_TtlDB_closeDatabase(JNIEnv* /* env */, jclass,
+                                          jlong /* jhandle */) {
   // auto* ttl_db = reinterpret_cast<ROCKSDB_NAMESPACE::DBWithTTL*>(jhandle);
   // assert(ttl_db != nullptr);
   // ROCKSDB_NAMESPACE::Status s = ttl_db->Close();
   // ROCKSDB_NAMESPACE::RocksDBExceptionJni::ThrowNew(env, s);
 
-  //TODO(AR) this is disabled until https://github.com/facebook/rocksdb/issues/4818 is resolved!
+  // TODO(AR) this is disabled until
+  // https://github.com/facebook/rocksdb/issues/4818 is resolved!
 }
 
 /*
@@ -178,9 +181,11 @@ void Java_org_rocksdb_TtlDB_closeDatabase(
  * Method:    createColumnFamilyWithTtl
  * Signature: (JLorg/rocksdb/ColumnFamilyDescriptor;[BJI)J;
  */
-jlong Java_org_rocksdb_TtlDB_createColumnFamilyWithTtl(
-    JNIEnv* env, jobject, jlong jdb_handle, jbyteArray jcolumn_name,
-    jlong jcolumn_options, jint jttl) {
+jlong Java_org_rocksdb_TtlDB_createColumnFamilyWithTtl(JNIEnv* env, jobject,
+                                                       jlong jdb_handle,
+                                                       jbyteArray jcolumn_name,
+                                                       jlong jcolumn_options,
+                                                       jint jttl) {
   jbyte* cfname = env->GetByteArrayElements(jcolumn_name, nullptr);
   if (cfname == nullptr) {
     // exception thrown: OutOfMemoryError
@@ -200,7 +205,7 @@ jlong Java_org_rocksdb_TtlDB_createColumnFamilyWithTtl(
   env->ReleaseByteArrayElements(jcolumn_name, cfname, JNI_ABORT);
 
   if (s.ok()) {
-    return reinterpret_cast<jlong>(handle);
+    return GET_CPLUSPLUS_POINTER(handle);
   }
   ROCKSDB_NAMESPACE::RocksDBExceptionJni::ThrowNew(env, s);
   return 0;
