@@ -3,20 +3,46 @@
 # README:
 #
 # This tool's purpose is to make it easier to merge PRs into test branches and
-# into main. Make sure you generate a Personal access token in GitHub and
-# add it your ~/.github.key.
+# into main.
 #
-# Because developers often have custom names for the ceph upstream remote
-# (https://github.com/ceph/ceph.git), You will probably want to export the
-# PTL_TOOL_BASE_PATH environment variable in your shell rc files before using
-# this script:
 #
-#     export PTL_TOOL_BASE_PATH=refs/remotes/<remotename>/
+# == Getting Started ==
 #
-# and PTL_TOOL_BASE_REMOTE as the name of your Ceph upstream remote (default: "upstream"):
+# You will probably want to setup a virtualenv for running this script:
 #
-#     export PTL_TOOL_BASE_REMOTE=<remotename>
+#    (
+#    virtualenv ~/ptl-venv
+#    source ~/ptl-venv/bin/activate
+#    pip3 install GitPython
+#    pip3 install python-redmine
+#    )
 #
+# Then run the tool with:
+#
+#    (source ~/ptl-venv/bin/activate && python3 src/script/ptl-tool.py --help)
+#
+# Important files in your $HOME:
+#
+#  ~/.redmine_key -- Your redmine API access key from right side of: https://tracker.ceph.com/my/account
+#
+#  ~/.github_token -- Your github Personal access token: https://github.com/settings/tokens
+#
+# Some important environment variables:
+#
+#  - PTL_TOOL_GITHUB_USER (your github username)
+#  - PTL_TOOL_GITHUB_TOKEN (your github Personal access token, or what is stored in ~/.github_token)
+#  - PTL_TOOL_REDMINE_USER (your redmine username)
+#  - PTL_TOOL_REDMINE_API_KEY (your redmine api key, or what is stored in ~/redmine_key)
+#  - PTL_TOOL_USER (your desired username embedded in test branch names)
+#
+#
+# You can use this tool to create a QA tracker ticket for you:
+#
+# $ python3 ptl-tool.py ... --create-qa --qa-release reef
+#
+# which will populate the ticket with all the usual information and also push a
+# tagged version of your test branch to ceph-ci for posterity.
+
 #
 # ** Here are some basic exmples to get started: **
 #
@@ -53,6 +79,23 @@
 # ...
 #
 #
+# Merging all PRs labeled 'wip-pdonnell-testing' into a new test branch but
+# NOT pushing that branch to ceph-ci repo (pushing to ceph-ci repo usually
+# happens only when we use --create-qa or --update-qa):
+#
+# $ src/script/ptl-tool.py --pr-label wip-pdonnell-testing --branch main --no-push-ci
+# Adding labeled PR #18805 to PR list
+# Adding labeled PR #18774 to PR list
+# Adding labeled PR #18600 to PR list
+# Will merge PRs: [18805, 18774, 18600]
+# Detaching HEAD onto base: main
+# Merging PR #18805
+# Merging PR #18774
+# Merging PR #18600
+# Checked out new branch wip-pdonnell-testing-20171108.054517
+# Created tag testing/wip-pdonnell-testing-20171108.054517
+#
+#
 # Merging PR #1234567 and #2345678 into a new test branch with a testing label added to the PR:
 #
 # $ src/script/ptl-tool.py 1234567 2345678 --label wip-pdonnell-testing
@@ -97,48 +140,72 @@
 # Merging PR #18192
 # Leaving HEAD detached; no branch anchors your commit
 
-
 # TODO
 # Look for check failures?
-# redmine issue update: http://www.redmine.org/projects/redmine/wiki/Rest_Issues
 
 import argparse
 import codecs
 import datetime
-import getpass
-import git
+from getpass import getuser
+import git # https://github.com/gitpython-developers/gitpython
 import itertools
 import json
 import logging
 import os
 import re
+try:
+    from redminelib import Redmine  # https://pypi.org/project/python-redmine/
+except ModuleNotFoundError:
+    Redmine = None
 import requests
+import signal
 import sys
 
 from os.path import expanduser
 
+BASE_PROJECT = os.getenv("PTL_TOOL_BASE_PROJECT", "ceph")
+BASE_REPO = os.getenv("PTL_TOOL_BASE_REPO", "ceph")
+BASE_REMOTE_URL = os.getenv("PTL_TOOL_BASE_REMOTE_URL", f"https://github.com/{BASE_PROJECT}/{BASE_REPO}.git")
+CI_REPO = os.getenv("PTL_TOOL_CI_REPO", "ceph-ci")
+CI_REMOTE_URL = os.getenv("PTL_TOOL_CI_REMOTE_URL", f"git@github.com:{BASE_PROJECT}/{CI_REPO}.git")
+GITDIR = os.getenv("PTL_TOOL_GITDIR", ".")
+GITHUB_USER = os.getenv("PTL_TOOL_GITHUB_USER", os.getenv("PTL_TOOL_USER", getuser()))
+GITHUB_TOKEN = None
+try:
+    with open(expanduser("~/.github_token")) as f:
+        GITHUB_TOKEN = f.read().strip()
+except FileNotFoundError:
+    pass
+GITHUB_TOKEN = os.getenv("PTL_TOOL_GITHUB_TOKEN", GITHUB_TOKEN)
+INDICATIONS = [
+    re.compile(r"(Reviewed-by: .+ <[\w@.-]+>)", re.IGNORECASE),
+    re.compile(r"(Acked-by: .+ <[\w@.-]+>)", re.IGNORECASE),
+    re.compile(r"(Tested-by: .+ <[\w@.-]+>)", re.IGNORECASE),
+]
+REDMINE_CUSTOM_FIELD_ID_SHAMAN_BUILD = 26
+REDMINE_CUSTOM_FIELD_ID_QA_RUNS = 27
+REDMINE_CUSTOM_FIELD_ID_QA_RELEASE = 28
+REDMINE_CUSTOM_FIELD_ID_QA_TAGS = 3
+REDMINE_CUSTOM_FIELD_ID_GIT_BRANCH = 29
+REDMINE_ENDPOINT = "https://tracker.ceph.com"
+REDMINE_PROJECT_QA = "ceph-qa"
+REDMINE_TRACKER_QA = "QA Run"
+REDMINE_USER = os.getenv("PTL_TOOL_REDMINE_USER", getuser())
+REDMINE_API_KEY = None
+try:
+    with open(expanduser("~/.redmine_key")) as f:
+        REDMINE_API_KEY = f.read().strip()
+except FileNotFoundError:
+    pass
+REDMINE_API_KEY = os.getenv("PTL_TOOL_REDMINE_API_KEY", REDMINE_API_KEY)
+SPECIAL_BRANCHES = ('main', 'luminous', 'jewel', 'HEAD')
+TEST_BRANCH = os.getenv("PTL_TOOL_TEST_BRANCH", "wip-{user}-testing-%Y%m%d.%H%M%S")
+USER = os.getenv("PTL_TOOL_USER", getuser())
+
 log = logging.getLogger(__name__)
 log.addHandler(logging.StreamHandler())
 log.setLevel(logging.INFO)
 
-BASE_PROJECT = os.getenv("PTL_TOOL_BASE_PROJECT", "ceph")
-BASE_REPO = os.getenv("PTL_TOOL_BASE_REPO", "ceph")
-BASE_REMOTE = os.getenv("PTL_TOOL_BASE_REMOTE", "upstream")
-BASE_PATH = os.getenv("PTL_TOOL_BASE_PATH", "refs/remotes/upstream/")
-GITDIR = os.getenv("PTL_TOOL_GITDIR", ".")
-USER = os.getenv("PTL_TOOL_USER", getpass.getuser())
-with open(expanduser("~/.github.key")) as f:
-    PASSWORD = f.read().strip()
-TEST_BRANCH = os.getenv("PTL_TOOL_TEST_BRANCH", "wip-{user}-testing-%Y%m%d.%H%M%S")
-
-SPECIAL_BRANCHES = ('main', 'luminous', 'jewel', 'HEAD')
-
-INDICATIONS = [
-    re.compile("(Reviewed-by: .+ <[\w@.-]+>)", re.IGNORECASE),
-    re.compile("(Acked-by: .+ <[\w@.-]+>)", re.IGNORECASE),
-    re.compile("(Tested-by: .+ <[\w@.-]+>)", re.IGNORECASE),
-]
-
 # find containing git dir
 git_dir = GITDIR
 max_levels = 6
@@ -151,8 +218,8 @@ while not os.path.exists(git_dir + '/.git'):
 CONTRIBUTORS = {}
 NEW_CONTRIBUTORS = {}
 with codecs.open(git_dir + "/.githubmap", encoding='utf-8') as f:
-    comment = re.compile("\s*#")
-    patt = re.compile("([\w-]+)\s+(.*)")
+    comment = re.compile(r"\s*#")
+    patt = re.compile(r"([\w-]+)\s+(.*)")
     for line in f:
         if comment.match(line):
             continue
@@ -162,13 +229,16 @@ with codecs.open(git_dir + "/.githubmap", encoding='utf-8') as f:
 BZ_MATCH = re.compile("(.*https?://bugzilla.redhat.com/.*)")
 TRACKER_MATCH = re.compile("(.*https?://tracker.ceph.com/.*)")
 
+def gitauth():
+    return (GITHUB_USER, GITHUB_TOKEN)
+
 def get(session, url, params=None, paging=True):
     if params is None:
         params = {}
     params['per_page'] = 100
 
     log.debug(f"Fetching {url}")
-    response = session.get(url, auth=(USER, PASSWORD), params=params)
+    response = session.get(url, auth=gitauth(), params=params)
     log.debug(f"Response = {response}; links = {response.headers.get('link', '')}")
     if response.status_code != 200:
         log.error(f"Failed to fetch {url}: {response}")
@@ -182,7 +252,7 @@ def get(session, url, params=None, paging=True):
             log.debug(f"Fetching {url}")
             new_params = dict(params)
             new_params.update({'page': page})
-            response = session.get(url, auth=(USER, PASSWORD), params=new_params)
+            response = session.get(url, auth=gitauth(), params=new_params)
             log.debug(f"Response = {response}; links = {response.headers.get('link', '')}")
             if response.status_code != 200:
                 log.error(f"Failed to fetch {url}: {response}")
@@ -250,6 +320,8 @@ def get_credits(session, pr, pr_req):
 def build_branch(args):
     base = args.base
     branch = datetime.datetime.utcnow().strftime(args.branch).format(user=USER)
+    if args.branch_release:
+        branch = branch + "-" + args.branch_release
     if args.debug_build:
         branch = branch + "-debug"
     label = args.label
@@ -271,9 +343,10 @@ def build_branch(args):
 
     G = git.Repo(args.git)
 
-    # First get the latest base branch and PRs from BASE_REMOTE
-    remote = getattr(G.remotes, BASE_REMOTE)
-    remote.fetch()
+    if args.create_qa or args.update_qa:
+        log.info("connecting to %s", REDMINE_ENDPOINT)
+        R = Redmine(REDMINE_ENDPOINT, username=REDMINE_USER, key=REDMINE_API_KEY)
+        log.debug("connected")
 
     prs = args.prs
     if args.pr_label is not None:
@@ -300,30 +373,35 @@ def build_branch(args):
     else:
         log.info("Detaching HEAD onto base: {}".format(base))
         try:
-            base_path = args.base_path + base
-            base = next(ref for ref in G.refs if ref.path == base_path)
+            G.git.fetch(BASE_REMOTE_URL, base)
             # So we know that we're not on an old test branch, detach HEAD onto ref:
-            base.checkout()
-        except StopIteration:
+            c = G.commit('FETCH_HEAD')
+        except git.exc.GitCommandError:
+            log.debug("could not fetch %s from %s", base, BASE_REMOTE_URL)
             log.info(f"Trying to checkout uninterpreted base {base}")
             c = G.commit(base)
-            G.git.checkout(c)
+        G.git.checkout(c)
         assert G.head.is_detached
 
+    qa_tracker_description = []
+
     for pr in prs:
         pr = int(pr)
         log.info("Merging PR #{pr}".format(pr=pr))
 
         remote_ref = "refs/pull/{pr}/head".format(pr=pr)
-        fi = remote.fetch(remote_ref)
-        if len(fi) != 1:
-            log.error("PR {pr} does not exist?".format(pr=pr))
+        try:
+            G.git.fetch(BASE_REMOTE_URL, remote_ref)
+        except git.exc.GitCommandError:
+            log.error("could not fetch %s from %s", remote_ref, BASE_REMOTE_URL)
             sys.exit(1)
-        tip = fi[0].ref.commit
+        tip = G.commit("FETCH_HEAD")
 
         endpoint = f"https://api.github.com/repos/{BASE_PROJECT}/{BASE_REPO}/pulls/{pr}"
         response = next(get(session, endpoint, paging=False))
 
+        qa_tracker_description.append(f'* "PR #{pr}":{response["html_url"]} -- {response["title"].strip()}')
+
         message = "Merge PR #%d into %s\n\n* %s:\n" % (pr, merge_branch_name, remote_ref)
 
         for commit in G.iter_commits(rev="HEAD.."+str(tip)):
@@ -354,12 +432,23 @@ def build_branch(args):
             G.git.commit("--amend", "--no-edit")
 
         if label:
-            req = session.post("https://api.github.com/repos/{project}/{repo}/issues/{pr}/labels".format(pr=pr, project=BASE_PROJECT, repo=BASE_REPO), data=json.dumps([label]), auth=(USER, PASSWORD))
+            req = session.post("https://api.github.com/repos/{project}/{repo}/issues/{pr}/labels".format(pr=pr, project=BASE_PROJECT, repo=BASE_REPO), data=json.dumps([label]), auth=gitauth())
             if req.status_code != 200:
                 log.error("PR #%d could not be labeled %s: %s" % (pr, label, req))
                 sys.exit(1)
             log.info("Labeled PR #{pr} {label}".format(pr=pr, label=label))
 
+    if args.stop_at_built:
+        log.warning("Stopping execution (SIGSTOP) with built branch for further modification. Foreground when execution should resume (typically `fg`).")
+        old_head = G.head.commit
+        signal.raise_signal(signal.SIGSTOP)
+        log.warning("Resuming execution.")
+        new_head = G.head.commit
+        if old_head != new_head:
+            rev = f'{old_head}..{new_head}'
+            for commit in G.iter_commits(rev=rev):
+                qa_tracker_description.append(f'* "commit {commit}":{CI_REMOTE_URL}/commit/{commit} -- {commit.summary}')
+
     # If the branch is 'HEAD', leave HEAD detached (but use "main" for commit message)
     if branch == 'HEAD':
         log.info("Leaving HEAD detached; no branch anchors your commits")
@@ -375,10 +464,80 @@ def build_branch(args):
 
         if created_branch:
             # tag it for future reference.
-            tag = "testing/%s" % branch
-            git.refs.tag.Tag.create(G, tag)
+            tag_name = "testing/%s" % branch
+            tag = git.refs.tag.Tag.create(G, tag_name)
             log.info("Created tag %s" % tag)
 
+    if args.create_qa or args.update_qa:
+        if not created_branch:
+            log.error("branch already exists!")
+            sys.exit(1)
+        project = R.project.get(REDMINE_PROJECT_QA)
+        log.debug("got redmine project %s", project)
+        user = R.user.get('current')
+        log.debug("got redmine user %s", user)
+        for tracker in project.trackers:
+            if tracker['name'] == REDMINE_TRACKER_QA:
+                tracker = tracker
+        if tracker is None:
+            log.error("could not find tracker in project: %s", REDMINE_TRACKER_QA)
+        log.debug("got redmine tracker %s", tracker)
+
+        # Use hard-coded custom field ids because there is apparently no way to
+        # figure these out via the python library
+        custom_fields = []
+        custom_fields.append({'id': REDMINE_CUSTOM_FIELD_ID_SHAMAN_BUILD, 'value': branch})
+        custom_fields.append({'id': REDMINE_CUSTOM_FIELD_ID_QA_RUNS, 'value': branch})
+        if args.qa_release:
+            custom_fields.append({'id': REDMINE_CUSTOM_FIELD_ID_QA_RELEASE, 'value': args.qa_release})
+        if args.qa_tags:
+            custom_fields.append({'id': REDMINE_CUSTOM_FIELD_ID_QA_TAGS, 'value': args.qa_tags})
+
+        if not args.no_push_ci:
+            G.git.push(CI_REMOTE_URL, branch) # for shaman
+            G.git.push(CI_REMOTE_URL, tag.name) # for archival
+        origin_url = f'{BASE_PROJECT}/{CI_REPO}/commits/{tag.name}'
+        custom_fields.append({'id': REDMINE_CUSTOM_FIELD_ID_GIT_BRANCH, 'value': origin_url})
+
+        issue_kwargs = {
+          "assigned_to_id": user['id'],
+          "custom_fields": custom_fields,
+          "description": '\n'.join(qa_tracker_description),
+          "project_id": project['id'],
+          "subject": branch,
+          "watcher_user_ids": user['id'],
+        }
+
+        if args.update_qa:
+            issue = R.issue.get(args.update_qa)
+            if issue.project.id != project.id:
+                log.error(f"issue {issue.url} project {issue.project} does not match {project}")
+                sys.exit(1)
+            if issue.tracker.id != tracker.id:
+                log.error(f"issue {issue.url} tracker {issue.tracker} does not match {tracker}")
+                sys.exit(1)
+
+            log.debug("updating issue with kwargs: %s", issue_kwargs)
+            notes = f"""
+            Updating branch to {branch}.
+            """
+            if R.issue.update(issue.id, **issue_kwargs):
+                log.info("updated redmine qa issue: %s", issue.url)
+            else:
+                log.error(f"failed to update {issue}")
+                sys.exit(1)
+        elif args.create_qa:
+            log.debug("creating issue with kwargs: %s", issue_kwargs)
+            issue = R.issue.create(**issue_kwargs)
+            log.info("created redmine qa issue: %s", issue.url)
+
+            for pr in prs:
+                log.debug(f"Posting QA Run in comment for ={pr}")
+                endpoint = f"https://api.github.com/repos/{BASE_PROJECT}/{BASE_REPO}/issues/{pr}/comments"
+                body = f"This PR is under test in [{issue.url}]({issue.url})."
+                r = session.post(endpoint, auth=gitauth(), data=json.dumps({'body':body}))
+                log.debug(f"= {r}")
+
 def main():
     parser = argparse.ArgumentParser(description="Ceph PTL tool")
     default_base = 'main'
@@ -391,17 +550,37 @@ def main():
         default_label = False
     else:
         argv = sys.argv[1:]
-    parser.add_argument('--branch', dest='branch', action='store', default=default_branch, help='branch to create ("HEAD" leaves HEAD detached; i.e. no branch is made)')
-    parser.add_argument('--debug-build', dest='debug_build', action='store_true', help='append -debug to branch name prompting ceph-build to build with CMAKE_BUILD_TYPE=Debug')
-    parser.add_argument('--merge-branch-name', dest='merge_branch_name', action='store', default=False, help='name of the branch for merge messages')
     parser.add_argument('--base', dest='base', action='store', default=default_base, help='base for branch')
-    parser.add_argument('--base-path', dest='base_path', action='store', default=BASE_PATH, help='base for branch')
+    parser.add_argument('--branch', dest='branch', action='store', default=default_branch, help='branch to create ("HEAD" leaves HEAD detached; i.e. no branch is made)')
+    parser.add_argument('--branch-release', dest='branch_release', action='store', help='release name to embed in branch (for shaman)')
+    parser.add_argument('--create-qa', dest='create_qa', action='store_true', help='create QA run ticket')
+    parser.add_argument('--debug', dest='debug', action='store_true', help='turn debugging on')
+    parser.add_argument('--debug-build', dest='debug_build', action='store_true', help='append -debug to branch name prompting ceph-build to build with CMAKE_BUILD_TYPE=Debug')
     parser.add_argument('--git-dir', dest='git', action='store', default=git_dir, help='git directory')
     parser.add_argument('--label', dest='label', action='store', default=default_label, help='label PRs for testing')
-    parser.add_argument('--pr-label', dest='pr_label', action='store', help='label PRs for testing')
+    parser.add_argument('--merge-branch-name', dest='merge_branch_name', action='store', default=False, help='name of the branch for merge messages')
     parser.add_argument('--no-credits', dest='credits', action='store_false', help='skip indication search (Reviewed-by, etc.)')
+    parser.add_argument('--pr-label', dest='pr_label', action='store', help='label PRs for testing')
+    parser.add_argument('--qa-release', dest='qa_release', action='store', help='QA release for tracker')
+    parser.add_argument('--qa-tags', dest='qa_tags', action='store', help='QA tags for tracker')
+    parser.add_argument('--stop-at-built', dest='stop_at_built', action='store_true', help='stop execution when branch is built')
+    parser.add_argument('--update-qa', dest='update_qa', action='store', help='update QA run ticket')
+    parser.add_argument('--no-push-ci', dest='no_push_ci', action='store_true',
+                        help='don\'t push branch to ceph-ci repo')
     parser.add_argument('prs', metavar="PR", type=int, nargs='*', help='Pull Requests to merge')
     args = parser.parse_args(argv)
+
+    if args.create_qa and args.update_qa:
+        log.error("--create-qa and --update-qa are mutually exclusive switches")
+        sys.exit(1)
+
+    if args.debug:
+        log.setLevel(logging.DEBUG)
+
+    if (args.create_qa or args.update_qa) and Redmine is None:
+        log.error("redmine library is not available so cannot create qa tracker ticket")
+        sys.exit(1)
+
     return build_branch(args)
 
 if __name__ == "__main__":
