@@ -36,7 +36,6 @@ typedef union {
 		uint64_t reserved_1_63         : 63;
 #endif /* Word 0 - End */
 	} s;
-	/* struct zip_vqx_ena_s cn; */
 } zip_vqx_ena_t;
 
 /**
@@ -64,7 +63,18 @@ typedef union {
 		uint64_t reserved_49_63        : 15;
 #endif /* Word 0 - End */
 	} s;
-	/* struct zip_vqx_sbuf_addr_s cn; */
+
+	struct zip_vqx_sbuf_addr_s9x {
+#if defined(__BIG_ENDIAN_BITFIELD) /* Word 0 - Big Endian */
+		uint64_t reserved_53_63        : 11;
+		uint64_t ptr                   : 46;
+		uint64_t off                   : 7;
+#else /* Word 0 - Little Endian */
+		uint64_t off                   : 7;
+		uint64_t ptr                   : 46;
+		uint64_t reserved_53_63        : 11;
+#endif /* Word 0 - End */
+	} s9x;
 } zip_vqx_sbuf_addr_t;
 
 /**
@@ -84,7 +94,6 @@ typedef union {
 		uint64_t reserved_20_63        : 44;
 #endif /* Word 0 - End */
 	} s;
-	/* struct zip_quex_doorbell_s cn; */
 } zip_quex_doorbell_t;
 
 /**
@@ -104,7 +113,6 @@ union zip_nptr_s {
 		uint64_t addr                  : 64;
 #endif /* Word 0 - End */
 	} s;
-	/* struct zip_nptr_s_s cn83xx; */
 };
 
 /**
@@ -197,10 +205,9 @@ union zip_inst_s {
 		uint64_t ef                    : 1;
 		/** Beginning of file */
 		uint64_t bf                    : 1;
-		// uint64_t reserved_3_4          : 2;
 		/** Comp/decomp operation */
 		uint64_t op                    : 2;
-		/** Data sactter */
+		/** Data scatter */
 		uint64_t ds                    : 1;
 		/** Data gather */
 		uint64_t dg                    : 1;
@@ -210,7 +217,6 @@ union zip_inst_s {
 		uint64_t hg                    : 1;
 		uint64_t dg                    : 1;
 		uint64_t ds                    : 1;
-		//uint64_t reserved_3_4          : 2;
 		uint64_t op                    : 2;
 		uint64_t bf                    : 1;
 		uint64_t ef                    : 1;
@@ -382,7 +388,7 @@ union zip_inst_s {
 		uint64_t bf                    : 1;
 		/** Comp/decomp operation */
 		uint64_t op                    : 2;
-		/** Data sactter */
+		/** Data scatter */
 		uint64_t ds                    : 1;
 		/** Data gather */
 		uint64_t dg                    : 1;
@@ -615,8 +621,6 @@ union zip_zres_s {
 		uint64_t hash3                 : 64;
 #endif /* Word 7 - End */
 	} /** ZIP Result Structure */s;
-
-	/* struct zip_zres_s_s cn83xx; */
 };
 
 /**
@@ -683,6 +687,7 @@ union zip_zptr_s {
 #define ZIP_COMP_E_TIMEOUT       (0xc)
 #define ZIP_COMP_E_INSTR_ERR     (0xd)
 #define ZIP_COMP_E_HCTX_ERR      (0xe)
+#define ZIP_COMP_E_PTR_ERR       (0xf)
 #define ZIP_COMP_E_STOP          (3)
 
 /**
