@@ -10,7 +10,8 @@
 
 #define R(name, flags)                                                         \
 	SSO_DEQ_TMO_SEG(cn10k_sso_hws_deq_tmo_seg_##name, flags)               \
-	SSO_DEQ_TMO_SEG(cn10k_sso_hws_reas_deq_tmo_seg_##name, flags | NIX_RX_REAS_F)
+	SSO_DEQ_TMO_SEG(cn10k_sso_hws_reas_deq_tmo_seg_##name,                 \
+			flags | NIX_RX_REAS_F)
 
-NIX_RX_FASTPATH_MODES_0_15
+NIX_RX_FASTPATH_MODES_32_47
 #undef R
