@@ -7,12 +7,9 @@
 
 #include <stdint.h>
 
-#include <rte_ethdev_driver.h>
+#include <ethdev_driver.h>
 
 
-uint16_t eth_ark_xmit_pkts_noop(void *vtxq,
-				struct rte_mbuf **tx_pkts,
-				uint16_t nb_pkts);
 uint16_t eth_ark_xmit_pkts(void *vtxq,
 			   struct rte_mbuf **tx_pkts,
 			   uint16_t nb_pkts);
