@@ -40,6 +40,11 @@
 #include <stdarg.h>
 #include "igzip_lib.h"		/* Normally you use isa-l.h instead for external programs */
 
+#if defined (HAVE_THREADS)
+# include <pthread.h>
+# include "crc.h"
+#endif
+
 #if !defined (VERSION)
 # if defined (ISAL_VERSION)
 #  define VERSION ISAL_VERSION
@@ -59,8 +64,17 @@
 #define BUF_SIZE 1024
 #define BLOCK_SIZE (1024 * 1024)
 
+#define MAX_FILEPATH_BUF 4096
+
 #define UNIX 3
 
+#define NAME_DEFAULT 0
+#define NO_NAME 1
+#define YES_NAME 2
+
+#define NO_TEST 0
+#define TEST 1
+
 #define LEVEL_DEFAULT 2
 #define DEFAULT_SUFFIX_LEN 3
 char *default_suffixes[] = { ".gz", ".z" };
@@ -152,6 +166,15 @@ struct cli_options {
 	int force;
 	int quiet_level;
 	int verbose_level;
+	int name;
+	int test;
+	int threads;
+	uint8_t *in_buf;
+	uint8_t *out_buf;
+	uint8_t *level_buf;
+	size_t in_buf_size;
+	size_t out_buf_size;
+	size_t level_buf_size;
 };
 
 struct cli_options global_options;
@@ -171,7 +194,15 @@ void init_options(struct cli_options *options)
 	options->force = false;
 	options->quiet_level = 0;
 	options->verbose_level = 0;
-	options->verbose_level = 0;
+	options->name = NAME_DEFAULT;
+	options->test = NO_TEST;
+	options->in_buf = NULL;
+	options->out_buf = NULL;
+	options->level_buf = NULL;
+	options->in_buf_size = 0;
+	options->out_buf_size = 0;
+	options->level_buf_size = 0;
+	options->threads = 1;
 };
 
 int is_interactive(void)
@@ -193,7 +224,7 @@ size_t get_filesize(FILE * fp)
 	fseeko(fp, 0, SEEK_END);
 #endif
 	fgetpos(fp, &pos);
-	file_size = *(size_t *) & pos;
+	file_size = *(size_t *)&pos;
 	fsetpos(fp, &pos_curr);	/* Restore position */
 
 	return file_size;
@@ -258,6 +289,10 @@ int usage(int exit_code)
 		  " -S, --suffix <.suf>  suffix to use while (de)compressing\n"
 		  " -V, --version        show version number\n"
 		  " -v, --verbose        verbose mode\n"
+		  " -N, --name           save/use file name and timestamp in compress/decompress\n"
+		  " -n, --no-name        do not save/use file name and timestamp in compress/decompress\n"
+		  " -t, --test           test compressed file integrity\n"
+		  " -T, --threads <n>    use n threads to compress if enabled\n"
 		  " -q, --quiet          suppress warnings\n\n"
 		  "with no infile, or when infile is - , read standard input\n\n",
 		  ISAL_DEF_MAX_LEVEL);
@@ -288,7 +323,7 @@ void *malloc_safe(size_t size)
 FILE *fopen_safe(char *file_name, char *mode)
 {
 	FILE *file;
-	char answer = 0, tmp;
+	int answer = 0, tmp;
 
 	/* Assumes write mode always starts with w */
 	if (mode[0] == 'w') {
@@ -364,20 +399,18 @@ size_t fwrite_safe(void *buf, size_t word_size, size_t buf_size, FILE * out, cha
 	return write_size;
 }
 
-void open_files(FILE ** in, FILE ** out, char *infile_name, size_t infile_name_len,
-		char *outfile_name)
+void open_in_file(FILE ** in, char *infile_name)
 {
 	*in = NULL;
-	*out = NULL;
-
 	if (infile_name == NULL)
 		*in = stdin;
 	else
 		*in = fopen_safe(infile_name, "rb");
+}
 
-	if (*in == NULL)
-		return;
-
+void open_out_file(FILE ** out, char *outfile_name)
+{
+	*out = NULL;
 	if (global_options.use_stdout)
 		*out = stdout;
 	else if (outfile_name != NULL)
@@ -390,6 +423,156 @@ void open_files(FILE ** in, FILE ** out, char *infile_name, size_t infile_name_l
 	}
 }
 
+#if defined(HAVE_THREADS)
+
+#define MAX_THREADS 8
+#define MAX_JOBQUEUE 16		/* must be a power of 2 */
+
+enum job_status {
+	JOB_UNALLOCATED = 0,
+	JOB_ALLOCATED,
+	JOB_SUCCESS,
+	JOB_FAIL
+};
+
+struct thread_job {
+	uint8_t *next_in;
+	uint32_t avail_in;
+	uint8_t *next_out;
+	uint32_t avail_out;
+	uint32_t total_out;
+	uint32_t type;
+	uint32_t status;
+};
+struct thread_pool {
+	pthread_t threads[MAX_THREADS];
+	struct thread_job job[MAX_JOBQUEUE];
+	pthread_mutex_t mutex;
+	pthread_cond_t cond;
+	int head;
+	int tail;
+	int queue;
+	int shutdown;
+};
+
+// Globals for thread pool
+struct thread_pool pool;
+
+static inline int pool_has_space()
+{
+	return ((pool.head + 1) % MAX_JOBQUEUE) != pool.tail;
+}
+
+static inline int pool_has_work()
+{
+	return (pool.queue != pool.head);
+}
+
+int pool_get_work()
+{
+	assert(pool.queue != pool.head);
+	pool.queue = (pool.queue + 1) % MAX_JOBQUEUE;
+	return pool.queue;
+}
+
+int pool_put_work(struct isal_zstream *stream)
+{
+	pthread_mutex_lock(&pool.mutex);
+	if (!pool_has_space() || pool.shutdown) {
+		pthread_mutex_unlock(&pool.mutex);
+		return 1;
+	}
+	int idx = (pool.head + 1) % MAX_JOBQUEUE;
+	pool.job[idx].next_in = stream->next_in;
+	pool.job[idx].avail_in = stream->avail_in;
+	pool.job[idx].next_out = stream->next_out;
+	pool.job[idx].avail_out = stream->avail_out;
+	pool.job[idx].status = JOB_ALLOCATED;
+	pool.job[idx].type = stream->end_of_stream == 0 ? 0 : 1;
+	pool.head = idx;
+	pthread_cond_signal(&pool.cond);
+	pthread_mutex_unlock(&pool.mutex);
+	return 0;
+}
+
+void *thread_worker(void *none)
+{
+	struct isal_zstream wstream;
+	int check;
+	int work_idx;
+	int level = global_options.level;
+	int level_size = level_size_buf[level];
+	uint8_t *level_buf = malloc_safe(level_size);
+	log_print(VERBOSE, "Start worker\n");
+
+	while (!pool.shutdown) {
+		pthread_mutex_lock(&pool.mutex);
+		while (!pool_has_work() && !pool.shutdown) {
+			pthread_cond_wait(&pool.cond, &pool.mutex);
+		}
+		if (pool.shutdown) {
+			pthread_mutex_unlock(&pool.mutex);
+			continue;
+		}
+
+		work_idx = pool_get_work();
+		pthread_cond_signal(&pool.cond);
+		pthread_mutex_unlock(&pool.mutex);
+
+		isal_deflate_stateless_init(&wstream);
+		wstream.next_in = pool.job[work_idx].next_in;
+		wstream.next_out = pool.job[work_idx].next_out;
+		wstream.avail_in = pool.job[work_idx].avail_in;
+		wstream.avail_out = pool.job[work_idx].avail_out;
+		wstream.end_of_stream = pool.job[work_idx].type;
+		wstream.flush = FULL_FLUSH;
+		wstream.level = global_options.level;
+		wstream.level_buf = level_buf;
+		wstream.level_buf_size = level_size;
+
+		check = isal_deflate_stateless(&wstream);
+		log_print(VERBOSE, "Worker finished job %d, out=%d\n",
+			  work_idx, wstream.total_out);
+
+		pool.job[work_idx].total_out = wstream.total_out;
+		pool.job[work_idx].status = JOB_SUCCESS + check;	// complete or fail
+		if (check)
+			break;
+	}
+	free(level_buf);
+	log_print(VERBOSE, "Worker quit\n");
+	pthread_exit(NULL);
+}
+
+int pool_create()
+{
+	int i;
+	int nthreads = global_options.threads - 1;
+	pool.head = 0;
+	pool.tail = 0;
+	pool.queue = 0;
+	pool.shutdown = 0;
+	for (i = 0; i < nthreads; i++)
+		pthread_create(&pool.threads[i], NULL, thread_worker, NULL);
+
+	log_print(VERBOSE, "Created %d pool threads\n", nthreads);
+	return 0;
+}
+
+void pool_quit()
+{
+	int i;
+	pthread_mutex_lock(&pool.mutex);
+	pool.shutdown = 1;
+	pthread_mutex_unlock(&pool.mutex);
+	pthread_cond_broadcast(&pool.cond);
+	for (i = 0; i < global_options.threads - 1; i++)
+		pthread_join(pool.threads[i], NULL);
+	log_print(VERBOSE, "Deleted %d pool threads\n", i);
+}
+
+#endif // defined(HAVE_THREADS)
+
 int compress_file(void)
 {
 	FILE *in = NULL, *out = NULL;
@@ -400,11 +583,14 @@ int compress_file(void)
 	struct isal_gzip_header gz_hdr;
 	int ret, success = 0;
 
-	char *infile_name = global_options.infile_name, *outfile_name =
-	    global_options.outfile_name;
+	char *infile_name = global_options.infile_name;
+	char *outfile_name = global_options.outfile_name;
+	char *allocated_name = NULL;
 	char *suffix = global_options.suffix;
 	size_t infile_name_len = global_options.infile_name_len;
+	size_t outfile_name_len = global_options.outfile_name_len;
 	size_t suffix_len = global_options.suffix_len;
+
 	int level = global_options.level;
 
 	if (suffix == NULL) {
@@ -413,36 +599,53 @@ int compress_file(void)
 	}
 
 	if (infile_name_len == stdin_file_name_len &&
+	    infile_name != NULL &&
 	    memcmp(infile_name, stdin_file_name, infile_name_len) == 0) {
 		infile_name = NULL;
 		infile_name_len = 0;
 	}
 
 	if (outfile_name == NULL && infile_name != NULL && !global_options.use_stdout) {
-		outfile_name = malloc_safe(infile_name_len + suffix_len + 1);
-		strcpy(outfile_name, infile_name);
-		strcat(outfile_name, suffix);
+		outfile_name_len = infile_name_len + suffix_len;
+		allocated_name = malloc_safe(outfile_name_len + 1);
+		outfile_name = allocated_name;
+		strncpy(outfile_name, infile_name, infile_name_len + 1);
+		strncat(outfile_name, suffix, outfile_name_len + 1);
 	}
 
-	open_files(&in, &out, infile_name, infile_name_len, outfile_name);
-	if (in == NULL || out == NULL)
+	open_in_file(&in, infile_name);
+	if (in == NULL)
 		goto compress_file_cleanup;
 
-	inbuf_size = BLOCK_SIZE;
-	outbuf_size = BLOCK_SIZE;
+	if (infile_name_len != 0 && infile_name_len == outfile_name_len
+	    && infile_name != NULL && outfile_name != NULL
+	    && strncmp(infile_name, outfile_name, infile_name_len) == 0) {
+		log_print(ERROR, "igzip: Error input and output file names must differ\n");
+		goto compress_file_cleanup;
+	}
 
-	inbuf = malloc_safe(inbuf_size);
-	outbuf = malloc_safe(outbuf_size);
-	level_size = level_size_buf[level];
-	level_buf = malloc_safe(level_size);
+	open_out_file(&out, outfile_name);
+	if (out == NULL)
+		goto compress_file_cleanup;
+
+	inbuf_size = global_options.in_buf_size;
+	outbuf_size = global_options.out_buf_size;
+
+	inbuf = global_options.in_buf;
+	outbuf = global_options.out_buf;
+	level_size = global_options.level_buf_size;
+	level_buf = global_options.level_buf;
 
 	isal_gzip_header_init(&gz_hdr);
-	gz_hdr.time = get_posix_filetime(in);
+	if (global_options.name == NAME_DEFAULT || global_options.name == YES_NAME) {
+		gz_hdr.time = get_posix_filetime(in);
+		gz_hdr.name = infile_name;
+	}
 	gz_hdr.os = UNIX;
-	gz_hdr.name = infile_name;
 	gz_hdr.name_buf_len = infile_name_len + 1;
 
 	isal_deflate_init(&stream);
+	stream.avail_in = 0;
 	stream.flush = NO_FLUSH;
 	stream.level = level;
 	stream.level_buf = level_buf;
@@ -453,32 +656,129 @@ int compress_file(void)
 
 	isal_write_gzip_header(&stream, &gz_hdr);
 
-	do {
-		if (stream.avail_in == 0) {
-			stream.next_in = inbuf;
-			stream.avail_in =
-			    fread_safe(stream.next_in, 1, inbuf_size, in, infile_name);
-			stream.end_of_stream = feof(in);
-		}
+	if (global_options.threads > 1) {
+#if defined(HAVE_THREADS)
+		int q;
+		int end_of_stream = 0;
+		uint32_t crc = 0;
+		uint64_t total_in = 0;
 
-		if (stream.next_out == NULL) {
-			stream.next_out = outbuf;
-			stream.avail_out = outbuf_size;
-		}
+		// Write the header
+		fwrite_safe(outbuf, 1, stream.total_out, out, outfile_name);
 
-		ret = isal_deflate(&stream);
+		do {
+			size_t nread;
+			size_t inbuf_used = 0;
+			size_t outbuf_used = 0;
+			uint8_t *iptr = inbuf;
+			uint8_t *optr = outbuf;
 
-		if (ret != ISAL_DECOMP_OK) {
-			log_print(ERROR,
-				  "igzip: Error encountered while compressing file %s\n",
-				  infile_name);
-			goto compress_file_cleanup;
-		}
+			for (q = 0; q < MAX_JOBQUEUE - 1; q++) {
+				inbuf_used += BLOCK_SIZE;
+				outbuf_used += 2 * BLOCK_SIZE;
+				if (inbuf_used > inbuf_size || outbuf_used > outbuf_size)
+					break;
 
-		fwrite_safe(outbuf, 1, stream.next_out - outbuf, out, outfile_name);
-		stream.next_out = NULL;
+				nread = fread_safe(iptr, 1, BLOCK_SIZE, in, infile_name);
+				crc = crc32_gzip_refl(crc, iptr, nread);
+				end_of_stream = feof(in);
+				total_in += nread;
+				stream.next_in = iptr;
+				stream.next_out = optr;
+				stream.avail_in = nread;
+				stream.avail_out = 2 * BLOCK_SIZE;
+				stream.end_of_stream = end_of_stream;
+				ret = pool_put_work(&stream);
+				if (ret || end_of_stream)
+					break;
+
+				iptr += BLOCK_SIZE;
+				optr += 2 * BLOCK_SIZE;
+			}
+
+			while (pool.tail != pool.head) {	// Unprocessed jobs
+				int t = (pool.tail + 1) % MAX_JOBQUEUE;
+				if (pool.job[t].status >= JOB_SUCCESS) {	// Finished next
+					if (pool.job[t].status > JOB_SUCCESS) {
+						success = 0;
+						log_print(ERROR,
+							  "igzip: Error encountered while compressing file %s\n",
+							  infile_name);
+						goto compress_file_cleanup;
+					}
+					fwrite_safe(pool.job[t].next_out, 1,
+						    pool.job[t].total_out, out, outfile_name);
+
+					pool.job[t].total_out = 0;
+					pool.job[t].status = 0;
+					pool.tail = t;
+					pthread_cond_broadcast(&pool.cond);
+				}
+				// Pick up a job while we wait
+				pthread_mutex_lock(&pool.mutex);
+				if (!pool_has_work()) {
+					pthread_mutex_unlock(&pool.mutex);
+					continue;
+				}
+
+				int work_idx = pool_get_work();
+				pthread_cond_signal(&pool.cond);
+				pthread_mutex_unlock(&pool.mutex);
+
+				isal_deflate_stateless_init(&stream);
+				stream.next_in = pool.job[work_idx].next_in;
+				stream.next_out = pool.job[work_idx].next_out;
+				stream.avail_in = pool.job[work_idx].avail_in;
+				stream.avail_out = pool.job[work_idx].avail_out;
+				stream.end_of_stream = pool.job[work_idx].type;
+				stream.flush = FULL_FLUSH;
+				stream.level = global_options.level;
+				stream.level_buf = level_buf;
+				stream.level_buf_size = level_size;
+				int check = isal_deflate_stateless(&stream);
+				log_print(VERBOSE, "Self   finished job %d, out=%d\n",
+					  work_idx, stream.total_out);
+				pool.job[work_idx].total_out = stream.total_out;
+				pool.job[work_idx].status = JOB_SUCCESS + check;	// complete or fail
+			}
+		} while (!end_of_stream);
+
+		// Write gzip trailer
+		fwrite_safe(&crc, sizeof(uint32_t), 1, out, outfile_name);
+		fwrite_safe(&total_in, sizeof(uint32_t), 1, out, outfile_name);
+
+#else // No compiled threading support but asked for threads > 1
+		assert(1);
+#endif
+	} else {		// Single thread
+		do {
+			if (stream.avail_in == 0) {
+				stream.next_in = inbuf;
+				stream.avail_in =
+				    fread_safe(stream.next_in, 1, inbuf_size, in, infile_name);
+				stream.end_of_stream = feof(in);
+			}
+
+			if (stream.next_out == NULL) {
+				stream.next_out = outbuf;
+				stream.avail_out = outbuf_size;
+			}
+
+			ret = isal_deflate(&stream);
+
+			if (ret != ISAL_DECOMP_OK) {
+				log_print(ERROR,
+					  "igzip: Error encountered while compressing file %s\n",
+					  infile_name);
+				goto compress_file_cleanup;
+			}
+
+			fwrite_safe(outbuf, 1, stream.next_out - outbuf, out, outfile_name);
+			stream.next_out = NULL;
+
+		} while (!feof(in) || stream.avail_out == 0);
+	}
 
-	} while (!feof(in) || stream.avail_out == 0);
 	success = 1;
 
       compress_file_cleanup:
@@ -491,17 +791,8 @@ int compress_file(void)
 			remove(infile_name);
 	}
 
-	if (global_options.outfile_name == NULL && outfile_name != NULL)
-		free(outfile_name);
-
-	if (inbuf != NULL)
-		free(inbuf);
-
-	if (outbuf != NULL)
-		free(outbuf);
-
-	if (level_buf != NULL)
-		free(level_buf);
+	if (allocated_name != NULL)
+		free(allocated_name);
 
 	return (success == 0);
 }
@@ -513,10 +804,12 @@ int decompress_file(void)
 	size_t inbuf_size, outbuf_size;
 	struct inflate_state state;
 	struct isal_gzip_header gz_hdr;
-	int ret = 0, success = 0;
+	const int terminal = 0, implicit = 1, stripped = 2;
+	int ret = 0, success = 0, outfile_type = terminal;
 
-	char *infile_name = global_options.infile_name, *outfile_name =
-	    global_options.outfile_name;
+	char *infile_name = global_options.infile_name;
+	char *outfile_name = global_options.outfile_name;
+	char *allocated_name = NULL;
 	char *suffix = global_options.suffix;
 	size_t infile_name_len = global_options.infile_name_len;
 	size_t outfile_name_len = global_options.outfile_name_len;
@@ -525,53 +818,69 @@ int decompress_file(void)
 	uint32_t file_time;
 
 	if (infile_name_len == stdin_file_name_len &&
+	    infile_name != NULL &&
 	    memcmp(infile_name, stdin_file_name, infile_name_len) == 0) {
 		infile_name = NULL;
 		infile_name_len = 0;
 	}
 
-	if (outfile_name == NULL && infile_name != NULL && !global_options.use_stdout) {
-		while (suffix_index < sizeof(default_suffixes) / sizeof(*default_suffixes)) {
-			if (suffix == NULL) {
-				suffix = default_suffixes[suffix_index];
-				suffix_len = default_suffixes_lens[suffix_index];
-				suffix_index++;
+	if (outfile_name == NULL && !global_options.use_stdout) {
+		if (infile_name != NULL) {
+			outfile_type = stripped;
+			while (suffix_index <
+			       sizeof(default_suffixes) / sizeof(*default_suffixes)) {
+				if (suffix == NULL) {
+					suffix = default_suffixes[suffix_index];
+					suffix_len = default_suffixes_lens[suffix_index];
+					suffix_index++;
+				}
+
+				outfile_name_len = infile_name_len - suffix_len;
+				if (infile_name_len >= suffix_len
+				    && memcmp(infile_name + outfile_name_len, suffix,
+					      suffix_len) == 0)
+					break;
+				suffix = NULL;
+				suffix_len = 0;
 			}
 
-			outfile_name_len = infile_name_len - suffix_len;
-			if (infile_name_len >= suffix_len
-			    && memcmp(infile_name + outfile_name_len, suffix, suffix_len) == 0)
-				break;
-			suffix = NULL;
-			suffix_len = 0;
+			if (suffix == NULL && global_options.test == NO_TEST) {
+				log_print(ERROR, "igzip: %s: unknown suffix -- ignored\n",
+					  infile_name);
+				return 1;
+			}
 		}
-
-		if (suffix == NULL) {
-			log_print(ERROR, "igzip: %s: unknown suffix -- ignored\n",
-				  infile_name);
-			return 1;
+		if (global_options.name == YES_NAME) {
+			outfile_name_len = 0;
+			outfile_type = implicit;
+		}
+		if (outfile_type != terminal) {
+			allocated_name = malloc_safe(outfile_name_len >=
+						     MAX_FILEPATH_BUF ? outfile_name_len +
+						     1 : MAX_FILEPATH_BUF);
+			outfile_name = allocated_name;
 		}
-
-		outfile_name = malloc_safe(outfile_name_len + 1);
-		memcpy(outfile_name, infile_name, outfile_name_len);
-		outfile_name[outfile_name_len] = 0;
 	}
 
-	open_files(&in, &out, infile_name, infile_name_len, outfile_name);
-	if (in == NULL || out == NULL)
+	open_in_file(&in, infile_name);
+	if (in == NULL)
 		goto decompress_file_cleanup;
 
 	file_time = get_posix_filetime(in);
 
-	inbuf_size = BLOCK_SIZE;
-	outbuf_size = BLOCK_SIZE;
-
-	inbuf = malloc_safe(inbuf_size);
-	outbuf = malloc_safe(outbuf_size);
+	inbuf_size = global_options.in_buf_size;
+	outbuf_size = global_options.out_buf_size;
+	inbuf = global_options.in_buf;
+	outbuf = global_options.out_buf;
 
 	isal_gzip_header_init(&gz_hdr);
+	if (outfile_type == implicit) {
+		gz_hdr.name = outfile_name;
+		gz_hdr.name_buf_len = MAX_FILEPATH_BUF;
+	}
+
 	isal_inflate_init(&state);
-	state.crc_flag = IGZIP_GZIP_NO_HDR;
+	state.crc_flag = ISAL_GZIP_NO_HDR_VER;
 	state.next_in = inbuf;
 	state.avail_in = fread_safe(state.next_in, 1, inbuf_size, in, infile_name);
 
@@ -582,6 +891,30 @@ int decompress_file(void)
 		goto decompress_file_cleanup;
 	}
 
+	if (outfile_type == implicit)
+		file_time = gz_hdr.time;
+
+	if (outfile_name != NULL && infile_name != NULL
+	    && (outfile_type == stripped
+		|| (outfile_type == implicit && outfile_name[0] == 0))) {
+		outfile_name_len = infile_name_len - suffix_len;
+		memcpy(outfile_name, infile_name, outfile_name_len);
+		outfile_name[outfile_name_len] = 0;
+	}
+
+	if (infile_name_len != 0 && infile_name_len == outfile_name_len
+	    && infile_name != NULL && outfile_name != NULL
+	    && strncmp(infile_name, outfile_name, infile_name_len) == 0) {
+		log_print(ERROR, "igzip: Error input and output file names must differ\n");
+		goto decompress_file_cleanup;
+	}
+
+	if (global_options.test == NO_TEST) {
+		open_out_file(&out, outfile_name);
+		if (out == NULL)
+			goto decompress_file_cleanup;
+	}
+
 	do {
 		if (state.avail_in == 0) {
 			state.next_in = inbuf;
@@ -600,7 +933,8 @@ int decompress_file(void)
 			goto decompress_file_cleanup;
 		}
 
-		fwrite_safe(outbuf, 1, state.next_out - outbuf, out, outfile_name);
+		if (out != NULL)
+			fwrite_safe(outbuf, 1, state.next_out - outbuf, out, outfile_name);
 
 	} while (!feof(in) || state.avail_out == 0);
 
@@ -623,14 +957,8 @@ int decompress_file(void)
 			remove(infile_name);
 	}
 
-	if (global_options.outfile_name == NULL && outfile_name != NULL)
-		free(outfile_name);
-
-	if (inbuf != NULL)
-		free(inbuf);
-
-	if (outbuf != NULL)
-		free(outbuf);
+	if (allocated_name != NULL)
+		free(allocated_name);
 
 	return (success == 0);
 }
@@ -638,7 +966,7 @@ int decompress_file(void)
 int main(int argc, char *argv[])
 {
 	int c;
-	char optstring[] = "hcdz0123456789o:S:kfqVv";
+	char optstring[] = "hcdz0123456789o:S:kfqVvNntT:";
 	int long_only_flag;
 	int ret = 0;
 	int bad_option = 0;
@@ -661,17 +989,15 @@ int main(int argc, char *argv[])
 		{"quiet", no_argument, NULL, 'q'},
 		{"version", no_argument, NULL, 'V'},
 		{"verbose", no_argument, NULL, 'v'},
+		{"no-name", no_argument, NULL, 'n'},
+		{"name", no_argument, NULL, 'N'},
+		{"test", no_argument, NULL, 't'},
+		{"threads", required_argument, NULL, 'T'},
 		/* Possible future extensions
-		   {"test", no_argument, NULL, 't'},
-		   {"no-name", no_argument, NULL, 'n'},
-		   {"name", no_argument, NULL, 'N'},
 		   {"recursive, no_argument, NULL, 'r'},
-		   {"check", no_argument, NULL, 'C'},
-		   {"no-check", no_argument, NULL, 0},
 		   {"list", no_argument, NULL, 'l'},
 		   {"benchmark", optional_argument, NULL, 'b'},
 		   {"benchmark_end", required_argument, NULL, 'e'},
-		   {"threads", optional_argument, NULL, 'T'},
 		 */
 		{0, 0, 0, 0}
 	};
@@ -733,6 +1059,24 @@ int main(int argc, char *argv[])
 		case 'V':
 			print_version();
 			return 0;
+		case 'N':
+			global_options.name = YES_NAME;
+			break;
+		case 'n':
+			global_options.name = NO_NAME;
+			break;
+		case 't':
+			global_options.test = TEST;
+			global_options.mode = DECOMPRESS_MODE;
+			break;
+		case 'T':
+#if defined(HAVE_THREADS)
+			c = atoi(optarg);
+			c = c > MAX_THREADS ? MAX_THREADS : c;
+			c = c < 1 ? 1 : c;
+			global_options.threads = c;
+#endif
+			break;
 		case 'h':
 			usage(0);
 		default:
@@ -764,9 +1108,24 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
+	global_options.in_buf_size = BLOCK_SIZE;
+	global_options.out_buf_size = BLOCK_SIZE;
+
+#if defined(HAVE_THREADS)
+	if (global_options.threads > 1) {
+		global_options.in_buf_size += (BLOCK_SIZE * MAX_JOBQUEUE);
+		global_options.out_buf_size += (BLOCK_SIZE * MAX_JOBQUEUE * 2);
+		pool_create();
+	}
+#endif
+	global_options.in_buf = malloc_safe(global_options.in_buf_size);
+	global_options.out_buf = malloc_safe(global_options.out_buf_size);
+	global_options.level_buf_size = level_size_buf[global_options.level];
+	global_options.level_buf = malloc_safe(global_options.level_buf_size);
+
 	if (global_options.mode == COMPRESS_MODE) {
 		if (optind >= argc)
-			compress_file();
+			ret |= compress_file();
 		while (optind < argc) {
 			global_options.infile_name = argv[optind];
 			global_options.infile_name_len = strlen(global_options.infile_name);
@@ -776,7 +1135,7 @@ int main(int argc, char *argv[])
 
 	} else if (global_options.mode == DECOMPRESS_MODE) {
 		if (optind >= argc)
-			decompress_file();
+			ret |= decompress_file();
 		while (optind < argc) {
 			global_options.infile_name = argv[optind];
 			global_options.infile_name_len = strlen(global_options.infile_name);
@@ -784,6 +1143,13 @@ int main(int argc, char *argv[])
 			optind++;
 		}
 	}
+#if defined(HAVE_THREADS)
+	if (global_options.threads > 1)
+		pool_quit();
+#endif
 
+	free(global_options.in_buf);
+	free(global_options.out_buf);
+	free(global_options.level_buf);
 	return ret;
 }
