@@ -4,6 +4,8 @@
 #include <fmt/format.h>
 #include "gtest/gtest.h"
 
+#include <iostream> // for std::cout
+
 using namespace std;
 
 template < typename T >
@@ -327,7 +329,7 @@ TEST(EncodingException, Macros) {
   } tests[] = {
     {
       DECODE_ERR_OLDVERSION(__PRETTY_FUNCTION__, 100, 200),
-      fmt::format("{} no longer understand old encoding version 100 < 200: Malformed input",
+      fmt::format("{} no longer understands old encoding version 100 < 200: Malformed input",
                   __PRETTY_FUNCTION__)
     },
     {
